/* ----------------------------------------------------------------------------
  ex1.C
  mbwall 28jul94
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
   Example program for the SimpleGA class and 2DBinaryStringGenome class.
This program tries to fill the 2Dgenome with alternating 1's and 0's. 
  Notice that one-point crossover is not necessarily the best kind of crossover
to use if you want to generate a 'good' genome with this kind of objective 
function. 
---------------------------------------------------------------------------- */
#include <stdio.h>
#include <iostream.h>
#include <ga/ga.h>		// this is the include file for the ga library

float objective(GAGenome &);

int
main()
{
  cout << "This program tries to fill a 2DBinaryStringGenome with\n";
  cout << "alternating 1s and 0s using a SimpleGA\n\n"; cout.flush();

// Declare variables for the GA parameters and set them to some default values.

  int width    = 10;
  int height   = 5;
  int popsize  = 30;
  int ngen     = 400;
  float pmut   = 0.001;
  float pcross = 0.9;

// Now create the GA and run it.  First we create a genome of the type that
// we want to use in the GA.  The ga doesn't use this genome in the
// optimization - it just uses it to clone a population of genomes.

  GA2DBinaryStringGenome chrom(width, height, objective);

// Now that we have the genome, we create the genetic algorithm and set
// its parameters - number of generations, mutation probability, and crossover
// probability.  We tell the ga to keep the single best genome of all the 
// generations, then tell the GA to evolve itself.

  GASimpleGA ga(chrom);
  ga.populationSize(popsize);
  ga.nGenerations(ngen);
  ga.pMutation(pmut);
  ga.pCrossover(pcross);
  ga.nBestGenomes(1);
  ga.evolve();

// Now we print out the best genome and write the best of generation (bog)
// scores to a file.  Here we write to a file called "bog.dat".

  cout << ga.bestOfAll() << "\n";
  ga.scores("bog.dat");
  cout << "best of generation data are in 'bog.dat'\n";

// That's it!
  return 0;
}
 



// This is the objective function.  All it does is check for alternating 0s and
// 1s.  If the gene is odd and contains a 1, the fitness is incremented by 1.
// If the gene is even and contains a 0, the fitness is incremented by 1.  No
// penalties are assigned. 
//   We have to do the cast because a plain, generic GAGenome doesn't have 
// the members that a GA2DBinaryStringGenome has.  And it's ok to cast it
// because we know that we will only get GA2DBinaryStringGenomes and
// nothing else.

float
objective(GAGenome & c)
{
  GA2DBinaryStringGenome & chrom = (GA2DBinaryStringGenome &)c;
  float objective=0.0;
  int count=0;
  for(int i=0; i<chrom.width(); i++){
    for(int j=0; j<chrom.height(); j++){
      if(chrom.gene(i,j) == 0 && count%2 == 0)
	objective += 1.0;
      if(chrom.gene(i,j) == 1 && count%2 != 0)
	objective += 1.0;
      count++;
    }
  }
  return(objective);
}
