/* ----------------------------------------------------------------------------
  ex4.C
  mbwall 28jul94
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
   Example program for the SteadyStateGA class and 3DBinaryStringGenome 
class.  This program tries to fill the 3Dgenome with alternating 1's and
0's.  Notice that the steady state ga needs many more 'generations' than the
simple ga, but it will usually converge to the optimum with fewer evaluations
of the objective function.  So if you have a compute-intensive objective
function, steady state ga may be the way to go.
  You can control the amount of overlap in a steady-state GA by using the
pReplacement member function.
---------------------------------------------------------------------------- */
#include <stdio.h>
#include <iostream.h>
#include <ga/ga.h>		// this is the include file for the ga library

float objective(GAGenome &);

int
main()
{
  cout << "This program tries to fill a 3DBinaryStringGenome with\n";
  cout << "alternating 1's and 0's using a SteadyStateGA\n\n"; cout.flush();

// Declare variables for the GA parameters and set them to some default values.

  int depth    = 3;
  int width    = 10;
  int height   = 5;
  int popsize  = 50;
  int ngen     = 2000;
  float pmut   = 0.001;
  float pcross = 0.9;

// Now create the GA and run it.  First we create a genome of the type that
// we want to use in the GA.  The ga doesn't use this genome in the
// optimization - it just uses it to clone a population of genomes.

  GA3DBinaryStringGenome chrom(width, height, depth, objective);

// Now that we have the genome, we create the genetic algorithm and set
// its parameters - number of generations, mutation probability, and crossover
// probability.  By default the GA keeps track of the best of generation scores
// and also keeps one genome as the 'best of all' - the best genome 
// that it encounters from all of the generations.

  GASteadyStateGA ga(chrom);
  ga.populationSize(popsize);
  ga.nGenerations(ngen);
  ga.pMutation(pmut);
  ga.pCrossover(pcross);
  ga.scoreFrequency(10);	// keep the best score of every 10th generation
  ga.evolve();

// Now we print out the best genome and write the best of generation (bog)
// scores to a file.  Here we write to a file called "bog.dat".

  cout << "the ga generated:\n" << ga.bestOfAll() << "\n";
  ga.scores("bog.dat");
  cout << "best of generation data are in 'bog.dat'\n";

// That's it!

  return 0;
}
 



// This is the objective function.  All it does is give one point for every
// odd bit that is set and one point for every even bit that is not set.

float
objective(GAGenome & c)
{
  GA3DBinaryStringGenome & chrom = (GA3DBinaryStringGenome &)c;
  float value=0.0;
  int count=0;
  int I=chrom.width(), J=chrom.height(), K=chrom.depth();
  for(int i=0; i<I; i++){
    for(int j=0; j<J; j++){
      for(int k=0; k<K; k++){
	if(chrom.gene(i,j,k) == 0 && count%2 == 0)
	  value += 1.0;
	else if(chrom.gene(i,j,k) == 1 && count%2 != 0)
	  value += 1.0;
	count++;
      }
    }
  }
  return(value);
}
