/* ----------------------------------------------------------------------------
  ex9.C
  mbwall 10apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
   Sample program that illustrates how to use a GA to find the maximum value
of a continuous function in two variables.  This program uses a binary-to-
decimal genome.
---------------------------------------------------------------------------- */
#include <stdio.h>
#include <iostream.h>
#include <fstream.h>
#include <ga/ga.h>

float objective(GAGenome &);

int
main()
{
  cout << "This program finds the maximum value in the function\n";
  cout << "  y = - x1^2 - x2^2\n";
  cout << "with the constraints\n";
  cout << "     -5 <= x1 <= 5\n";
  cout << "     -5 <= x2 <= 5\n";
  cout << "\n\n";

// Declare variables for the GA parameters and set them to some default values.

  int popsize  = 30;
  int ngen     = 100;
  float pmut   = 0.01;
  float pcross = 0.6;

// Create a phenotype for two variables.  The number of bits you can use to
// represent any number is limited by the type of computer you are using.  In
// this case, we use 16 bits to represent a floating point number whose value
// can range from -5 to 5, inclusive.  The bounds on x1 and x2 can be applied
// here and/or in the objective function.

  GABin2DecPhenotype map;
  map.add(16, -5, 5);
  map.add(16, -5, 5);

// Create the template genome using the phenotype map we just made.

  GABin2DecGenome chrom(map, objective);

// Now create the GA using the genome and run it.  We'll use sigma truncation
// scaling so that we can handle negative objective scores.

  GASimpleGA ga(chrom);
  GASigmaTruncationScaling scaling;
  ga.populationSize(popsize);
  ga.nGenerations(ngen);
  ga.pMutation(pmut);
  ga.pCrossover(pcross);
  ga.scalingScheme(scaling);
  ga.evolve();

// Dump the results of the GA to the screen.

  chrom = ga.bestOfAll();
  cout << "the ga found an optimum at the point (";
  cout << chrom.phenotype(0) << ", " << chrom.phenotype(1) << ")";
  cout << "\n\n"; cout.flush();

// Dump the best of generation scores to a file.

  ga.scores("bog.dat");
  cout << "best of generation data are in 'bog.dat'\n";

  return 0;
}
 

// This objective function tries to maximize the value of the function
//
//                  y = -(x1*x1 + x2*x2)
//
float
objective(GAGenome & c)
{
  GABin2DecGenome & chrom = (GABin2DecGenome &)c;

  float y;
  y = -chrom.phenotype(0) * chrom.phenotype(0);
  y -= chrom.phenotype(1) * chrom.phenotype(1);
  return y;
}
