/* ----------------------------------------------------------------------------
  bitstr.h
  mbwall 13sep95
  Copyright 1995 Massachusetts Institute of Technology

  This code can be freely distributed and modified under the terms of the GNU
  public license.   See the COPYING file for details.

 DESCRIPTION:
   Header file for the bitstring example.  This code uses the BitString object
from the GNU class library for the derivation of a new class of genomes.  Here
we define the genome and all of the basic operators that it needs to function.
---------------------------------------------------------------------------- */
#ifndef _bitstr_h_
#define _bitstr_h_

#include <ga/genome.h>
#include "BitString.h"


// This is the class definition for the BitString genome.  It is derived from
// the Genome class and the GNU BitString class.  We define here only the
// additional methods that we'll need in order to use it a genome in GAlib.
//   The identity definition is to take care of the polymorphic nature of 
// GAlib genomes.  You can use any number above 200 when defining your own
// genome type.  Anything under 200 is reserved for use in GAlib internals.
//   The only methods that we MUST define are the clone method and the genome-
// specific operators.
//   I have defined a 'gene' method here as a convenience routine.  It simply
// calls the BitString's bit access member function to determine if a bit is
// set or not.  When you call 'gene' with a second argument, it sets the
// specified bit in the bitstring.
//   Unlike the binary string genomes included in GAlib, this genome is not
// resizable.

class BitStringGenome : public GAGenome, public BitString {
  GADefineIdentity("BitStringGenome", 200):

public:
  BitStringGenome(const unsigned int x,
		  GAObjectiveFunction f=(GAObjectiveFunction)0,
		  void * u=(void *)0,
		  GA * g=(GA *)0);
  BitStringGenome(const BitStringGenome& orig){copy(orig);}
  BitStringGenome& operator=(const BitStringGenome& orig)
    {copy(orig); return *this;}
  BitStringGenome& operator=(const GAGenome& orig)
    {copy(orig); return *this;}
  virtual ~BitStringGenome(){}
  virtual GAGenome *clone(const GACloneMethod flag=gaCloneContents) const;
  virtual void copy(const GAGenome&);

  friend ostream& operator<<(ostream& os, const BitStringGenome& arg)
    {arg._write(os); return os;}
  friend int operator==(const BitStringGenome& a, const BitStringGenome& b)
    {return a._equal(b);}
  friend int operator!=(const BitStringGenome& a, const BitStringGenome& b)
    {return !a._equal(b);}

  int gene(const unsigned int x=0) const {return test(x);}
  int gene(const unsigned int x, const int b) {assign(x,b); return test(x);}

protected:
  void _write (ostream& os) const {printon(os);}
  int _equal(const GAGenome & c) const {
    BitString & b = (BitStringGenome&)c;
    return b == *this;
  }
};



// Here are the function prototypes for the BitStringGenome operators.
void BitStringUniformRandomInitializer(GAGenome&);
int  BitStringUniformRandomFlipMutator(GAGenome&, float p);
void BitStringUniformCrossover(GAGenome&, const GAGenome&, const GAGenome&);


#endif
