// $Header: /usr/people/mbwall/src/ga/ga/RCS/allele.h,v 1.7 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  allele.h
  mbwall 21mar95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
   Here we define a class of alleles.  An allele is a possible value for a gene
and an allele set is a list of possible values (I use 'set' because it doesn't
imply the specific implementation of the container class).

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_allele_h_
#define _ga_allele_h_

#include <iostream.h>
#include <string.h>
#include <ga/random.h>

#define GA_ALLELE_CHUNK 10


/* ----------------------------------------------------------------------------
AlleleSet
-------------------------------------------------------------------------------
  This object contains a set of alleles for use with a similarly typed genome.
  This object can be used with any type that has operator= defined for it.
  This should be implemented as a derivative of the Array class?

  There is no way to 'resize' an allele set.  You must add to it or remove
elements from it.  You can control the chunksize, however.

  The base template class assumes that the objects in the allele set are 
complex, i.e. it is not OK to do a bit-copy of each object.  We should do 
specializations for non-complex objects (or perhaps a separate class that does
bit-copies rather than logical-copies?)

*** should the assignment operator check to be sure that no allele is 
    duplicated, or is it OK to have duplicate alleles in a set?  For now we
    allow duplicates (via either the add or assignemnt ops).
---------------------------------------------------------------------------- */
template <class T>
class GAAlleleSet {
public:
  GAAlleleSet(const unsigned int n=0) :
  sz(n), csz(GA_ALLELE_CHUNK), SZ(GA_ALLELE_CHUNK) {
    while(SZ < n) SZ += csz;
    a = new T [SZ];
  }
  GAAlleleSet(const unsigned int n, const T array []) : 
  sz(n), csz(GA_ALLELE_CHUNK), SZ(GA_ALLELE_CHUNK) {
    while(SZ < n) SZ += csz;
    a = new T [SZ];
    for(int i=0; i<sz; i++)
      a[i] = *(array+i);
  }
  GAAlleleSet(const unsigned int n, const unsigned int csize) :
  sz(n), csz(csize), SZ(csize) {
    while(SZ < n) SZ += csz;
    a = new T [SZ];
  }
  virtual ~GAAlleleSet(){delete [] a;}
  GAAlleleSet & operator=(const T array []){
    for(int i=0; i<sz; i++)
      a[i] = *(array+i);
    return *this;
  }
  T & add(const T & allele){
    if(sz <= SZ){
      SZ += csz;
      T * tmp = a;
      a = new T [SZ];
      for(int i=0; i<sz; i++)
	a[i] = tmp[i];
//      memcpy(a, tmp, sz*sizeof(T));
      delete [] tmp;
    }
    a[sz] = allele;
    sz++;
    return a[sz];
  }
  void remove(T & allele){	// yuk.  should return the one we remove?
    for(int i=0; i<sz; i++){
      if(a[i] == allele){
	for(int j=i; j<sz-1; j++)
	  a[j] = a[j+1];
//	memmove(&(a[i]), &(a[i+1]), (sz-i-1)*sizeof(T));
	sz--;
	break;
      }
    }
  }
  void remove(const unsigned int x){ // yuk.  should return the one we remove?
    for(int j=x; j<sz-1; j++)
      a[j] = a[j+1];
//  memmove(&(a[i]), &(a[i+1]), (sz-i-1)*sizeof(T));
    sz--;
  }
  const T & allele() const {return a[GARandomInt(0,sz-1)];}
  T & allele(){return a[GARandomInt(0,sz-1)];}
  const T & allele(const unsigned int i) const {return a[i];}
  T & allele(const unsigned int i){return a[i];}
  int size() const {return sz;}
  int chunksize() const {return csz;}
  int chunksize(const unsigned int csize) {return csz=csize;}

  friend ostream & operator<<(ostream & os, const GAAlleleSet<T> & arg)
    {arg._write(os); return os;}
  friend istream & operator>>(istream & is, GAAlleleSet<T> & arg)
    {arg._read(is); return is;}
  friend int operator==(const GAAlleleSet<T> & a, const GAAlleleSet<T> & b)
    {return a._equal(b);}
  friend int operator!=(const GAAlleleSet<T> & a, const GAAlleleSet<T> & b)
    {return a._notequal(b);}

protected:
  unsigned int csz;		// how big are the chunks to allocate?
  unsigned int sz;		// number we have
  unsigned int SZ;		// how many have we allocated?
  T * a;

  void _read(istream &){}	// can't write this apriori
  void _write(ostream & os=cout) const {
    for(int i=0; i<sz; i++)
      os << a[i];
  }
  int _equal(const GAAlleleSet<T> & orig) const {
    if(sz != orig.sz) return 0;
    int i;
    for(i=0; i<sz && a[i] == orig.a[i]; i++);
    return((i == sz) ? 1 : 0);
  }
  int _notequal(const GAAlleleSet<T> & orig) const {
    if(sz != orig.sz) return 1;
    int i;
    for(i=0; i<sz && a[i] == orig.a[i]; i++);
    return((i == sz) ? 0 : 1);
  }

private:
  GAAlleleSet(const GAAlleleSet<T> &){}
  GAAlleleSet & operator=(const GAAlleleSet<T> &){return *this;}
};

#endif
