// $Header: /usr/people/mbwall/src/ga/ga/RCS/array1.ch.h,v 1.3 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  array1.ch.h
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the interface for the 1D array genome.
  Similar to the binary string genome, but here instead of alleles of 0,1
we have user-definable alleles.  In order to use this genome you must 
specify an allele set.  If no set of alleles is defined, the genome will
post error messages.
  You can use ANY kind of object in this genome.  But notice that it is
really easy to optimize this for some of the simpler types.  I'll try to do 
that for common instantiations (int, char).
  The objects in the array must have the following operators defined:
  =  ==  != 
  >> must be defined if you use the default read methods

 TO DO:
*** Need to do some specializations for simple data types so we can use memcpy,
    memmove, etc rather than the for loops.


 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_array1_ch_h_
#define _ga_array1_ch_h_

#include <ga/arraytmpl.h>
#include <ga/genome.h>
#include <ga/allele.h>


/* ----------------------------------------------------------------------------
1DArrayGenome
-------------------------------------------------------------------------------
  We don't do any error checking on the assignment to const array of type T.
---------------------------------------------------------------------------- */
template <class T>
class GA1DArrayGenome : public GAGenome, public GAArray<T> {
  GADefineIdentity("GA1DArrayGenome", 61):

public:
  GA1DArrayGenome(const unsigned int x,
		  GAAlleleSet<T> & a,
		  GAObjectiveFunction f=(GAObjectiveFunction)0,
		  void * u=(void *)0,
		  GA * g=(GA *)0);
  GA1DArrayGenome(const GA1DArrayGenome<T> & orig);
  GA1DArrayGenome& operator=(const GA1DArrayGenome<T> & orig)
    {copy(orig); return *this;}
  GA1DArrayGenome& operator=(const GAGenome& orig)
    {copy(orig); return *this;}
  GA1DArrayGenome& operator=(const T array []) // no err checks!
    {for(int i=0; i<sz; i++) gene(i) = *(array+i); return *this;}
  virtual ~GA1DArrayGenome(){}
  virtual GAGenome *clone(const GACloneMethod flag=gaCloneContents) const;
  virtual void copy(const GAGenome &);

  friend ostream & operator<<(ostream & os, const GA1DArrayGenome<T> & arg)
    {arg._write(os); return os;}
  friend istream & operator>>(istream & is, GA1DArrayGenome<T> & arg)
    {arg._read(is); return is;}
  friend int operator==(const GA1DArrayGenome<T> & a,
			const GA1DArrayGenome<T> & b)
    {return a._equal(b);}
  friend int operator!=(const GA1DArrayGenome<T> & a,
			const GA1DArrayGenome<T> & b)
    {return a._notequal(b);}

  const T & allele(const unsigned int i) const {return as->allele(i);}
  GAAlleleSet<T> & alleles() const {return *as;}
  GAAlleleSet<T> & alleles(GAAlleleSet<T> & set){as = &set; return *as;}

  const T & gene(const unsigned int x=0) const {return a[x];}
  T & gene(const unsigned int x=0) {return a[x];}
  T & gene(const unsigned int x, const T & value){
    if(a[x] != value) a[x] = value;
    return a[x];
  }
  const T & operator[](const unsigned int x) const {return gene(x);}
  T & operator[](const unsigned int x) {return a[x];}
  int length() const {return nx;}
  int length(const int x){resize(x); return nx;}
  int resize(const int x);
  int resizeBehaviour() const {return((maxX == minX) ? gaNoResize:(int)maxX);}
  int resizeBehaviour(const unsigned int lx, const unsigned int ux);
  void copy(const GA1DArrayGenome<T> & orig,
	    const unsigned int r, const unsigned int x,
	    const unsigned int length){
    int l=length;
    if(l > 0 && x < orig.nx && r < nx){
      if(x + l > orig.nx) l = orig.nx - x;
      if(r + l > nx) l = nx - r;
      GAArray<T>::copy(orig,r,x,l);
    }
  }
  void swap(const unsigned int i, const unsigned int j){GAArray<T>::swap(i,j);}


protected:
  GAAlleleSet<T> * as;		// the allele set (pt to original, not copy)
  unsigned int nx;		// how long is the data string?
  unsigned int minX;		// what is the lower limit?
  unsigned int maxX;		// what is the upper limit? 
  void _read(istream & is);
  void _write (ostream & os) const;
  int _equal(const GAGenome & c) const {
    GA1DArrayGenome<T> & b = (GA1DArrayGenome<T> &)c;
    return((this == &c) ? 1 : 
	   ((nx != b.nx) ? 0 : GAArray<T>::equal(b,0,0,nx)));
  }
  int _notequal(const GAGenome & c) const {
    GA1DArrayGenome<T> & b = (GA1DArrayGenome<T> &)c;
    return((this == &c) ? 0 :
	   ((nx != b.nx) ? 1 : GAArray<T>::equal(b,0,0,nx) ? 0 : 1));
  }

private:
  GA1DArrayGenome() : GAArray<T>(0){}
};


#ifdef BORLAND_INST
#include <ga/array1.ch.C>
#endif

#endif

