// $Header: /usr/people/mbwall/src/ga/ga/RCS/array1.op.h,v 1.2 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  array1.op.h
  mbwall 5may95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Header for the 1D array genome operators.

  These methods are provided as examples of how to implement basic initializer,
mutator, and crossover operators.  We cannot assign these to the template
genomes by default since template functions are distinguished by their
signatures (each template function has a different signature) and we have 
defined the operators as virtual functions (the signatures must be the same).
  There are a couple of ways to use this code.  The easiest thing to do is to
use the preprocessor directives to define the type and dimension of the 
genome you are going to instantiate.  The possibilities are:

  ARRAY_TYPE			this is the type that the array contains

  Then #include this file.  For example, to use an integer array,

#define ARRAY_TYPE int
#include <ga/array1.op.h>

  If you are going to instantiate arrays of more than one type, you must copy
the parts of this file that you need to your own code and change the function 
names so that you don't get naming conflicts.
  If you are going to customize the initialization, mutation, or crossover
operators, then you'll probably want to copy chunks of this code to your own
code.  In that case you'll just have to change the template part of these
routines to be the type that you're using.  For example, if you are creating
a GAArrayGenome<int> then replace GAArrayGenome<T> with
GAArrayGenome<int> in all of these routines. You should modify the routines
as needed to get the initialization, mutation, and crossover behaviour that
you want.
  Beware that the crossover operator is closely coupled to the crossover site
object.  You can create your own crossover object too, if you like.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_array1_op_h_
#define _ga_array1_op_h_

#include <ga/genome.h>

void GA1DArrayRandomInitializer(GAGenome&);
void GA1DArrayOrderedInitializer(GAGenome&);

int  GA1DArrayFlipMutator(GAGenome&, float);
int  GA1DArraySwapMutator(GAGenome&, float);

void GA1DArrayUniformCrossover(GAGenome&, const GAGenome&, const GAGenome&);
void GA1DArray1PtCrossover(GAGenome&, const GAGenome&, const GAGenome&);
void GA1DArray2PtCrossover(GAGenome&, const GAGenome&, const GAGenome&);
void GA1DArrayEvenCrossover(GAGenome &, const GAGenome &, const GAGenome &);
void GA1DArrayOddCrossover(GAGenome &, const GAGenome &, const GAGenome &);
void GA1DArrayPartialMatchCrossover(GAGenome&,const GAGenome&,const GAGenome&);
void GA1DArrayOrderCrossover(GAGenome&, const GAGenome&, const GAGenome&);
void GA1DArrayCycleCrossover(GAGenome&, const GAGenome&, const GAGenome&);

#ifdef ARRAY_TYPE
#include <ga/array1.op.C>
#endif

#endif
