// $Header: /usr/people/mbwall/src/ga/ga/RCS/array1.xs.C,v 1.1 1995/06/30 21:27:55 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  array1.xs.C
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the interface for the 1D array genome crossover sites.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <ga/array1.xs.h>

#if USE_1PT_CROSSOVER == 1

template <class T> void
GA1DArray1PtCrossSite<T>::pick(GAGenome & c){
  GA1DArrayGenome<T> & m = (GA1DArrayGenome<T> &)c;
  x = GARandomInt(0, m.length()); x1 = x2 = gaNoSite;
}

template <class T> void
GA1DArray1PtCrossSite<T>::
pick(GAGenome & c, const GAGenome & c1, const GAGenome & c2){
  GA1DArrayGenome<T> & m  = (GA1DArrayGenome<T> &)c;
  GA1DArrayGenome<T> & mom = (GA1DArrayGenome<T> &)c1;
  GA1DArrayGenome<T> & dad = (GA1DArrayGenome<T> &)c2;
  x  = GARandomInt(0, m.length());
  x1 = GARandomInt(0, mom.length());
  x2 = GARandomInt(0, dad.length());
}

template <class T> void
GA1DArray1PtCrossSite<T>::mirror(GAGenome &, const GAGenome & c1){
  x  = ((GA1DArray1PtCrossSite<T> &)(c1.crossoverSite())).x;
  x1 = ((GA1DArray1PtCrossSite<T> &)(c1.crossoverSite())).x2;
  x2 = ((GA1DArray1PtCrossSite<T> &)(c1.crossoverSite())).x1;
}

#endif



#if USE_2PT_CROSSOVER == 1

template <class T> void
GA1DArray2PtCrossSite<T>::pick(GAGenome & c){
  GA1DArrayGenome<T> & m = (GA1DArrayGenome<T> &)c;
  x[0] = GARandomInt(0, m.length()); x1[0] = x2[0] = gaNoSite;
  x[1] = GARandomInt(0, m.length()); x1[1] = x2[1] = gaNoSite;
  if(x[1] < x[0]){int tmp=x[0]; x[0]=x[1]; x[1]=tmp;}
}

template <class T> void
GA1DArray2PtCrossSite<T>::
pick(GAGenome & c, const GAGenome & c1, const GAGenome & c2){
  GA1DArrayGenome<T> & m  = (GA1DArrayGenome<T> &)c;
  GA1DArrayGenome<T> & mom = (GA1DArrayGenome<T> &)c1;
  GA1DArrayGenome<T> & dad = (GA1DArrayGenome<T> &)c2;
  x[0]  = GARandomInt(0, m.length());
  x[1]  = GARandomInt(0, m.length());
  if(x[1] < x[0]){int tmp=x[0]; x[0]=x[1]; x[1]=tmp;}
  x1[0] = GARandomInt(0, mom.length());
  x1[1] = GARandomInt(0, mom.length());
  if(x1[1] < x1[0]){int tmp=x1[0]; x1[0]=x1[1]; x1[1]=tmp;}
  x2[0] = GARandomInt(0, dad.length());
  x2[1] = GARandomInt(0, dad.length());
  if(x2[1] < x2[0]){int tmp=x2[0]; x2[0]=x2[1]; x2[1]=tmp;}
}

template <class T> void
GA1DArray2PtCrossSite<T>::mirror(GAGenome &, const GAGenome & c1){
  x[0]  = ((GA1DArray2PtCrossSite<T> &)(c1.crossoverSite())).x[0];
  x1[0] = ((GA1DArray2PtCrossSite<T> &)(c1.crossoverSite())).x2[0];
  x2[0] = ((GA1DArray2PtCrossSite<T> &)(c1.crossoverSite())).x1[0];
  x[1]  = ((GA1DArray2PtCrossSite<T> &)(c1.crossoverSite())).x[1];
  x1[1] = ((GA1DArray2PtCrossSite<T> &)(c1.crossoverSite())).x2[1];
  x2[1] = ((GA1DArray2PtCrossSite<T> &)(c1.crossoverSite())).x1[1];
}

#endif




#if USE_PARTIAL_MATCH_CROSSOVER == 1 || \
    USE_ORDER_CROSSOVER == 1 || \
    USE_CYCLE_CROSSOVER == 1

template <class T> void 
GA1DArrayOrderedCrossSite<T>::pick(GAGenome & c){
  GA1DArrayGenome<T> & child = (GA1DArrayGenome<T> &)c;
  a = GARandomInt(0, child.size());
  b = GARandomInt(0, child.size());
  if(b<a){
    int tmp=a;
    a=b;
    b=tmp;
  }
}


template <class T> void 
GA1DArrayOrderedCrossSite<T>::
pick(GAGenome & c, const GAGenome &, const GAGenome &){
  pick(c);
}


template <class T> void 
GA1DArrayOrderedCrossSite<T>::mirror(GAGenome &, const GAGenome & c1)
{
  a = ((GA1DArrayOrderedCrossSite<T> &)(c1.crossoverSite())).a;
  b = ((GA1DArrayOrderedCrossSite<T> &)(c1.crossoverSite())).b;
}

#endif
