// $Header: /usr/people/mbwall/src/ga/ga/RCS/array1.xs.h,v 1.3 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  array1.xs.h
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the interface for the 1D array genome crossover sites.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_array1_xs_h_
#define _ga_array1_xs_h_

#include <ga/config.h>
#include <ga/xsite.h>


#if USE_1PT_CROSSOVER == 1
/* ----------------------------------------------------------------------------
 1D Array Single Point Crossover
---------------------------------------------------------------------------- */
template <class T>
class GA1DArray1PtCrossSite : public GACrossSite {
  GADefineIdentity("GA1DArray1PtCrossSite", 85):

public:
  GA1DArray1PtCrossSite(){x=x1=x2=gaNoSite;}
  GA1DArray1PtCrossSite(int a){x=x1=x2=a;}
  GA1DArray1PtCrossSite(const GA1DArray1PtCrossSite<T> & orig){copy(orig);}
  GA1DArray1PtCrossSite<T> & operator=(const GACrossSite & orig)
    {copy(orig); return *this;}
  GA1DArray1PtCrossSite<T> & operator=(const GA1DArray1PtCrossSite<T> & orig)
    {copy(orig); return *this;}
  virtual ~GA1DArray1PtCrossSite(){}
  virtual void copy(const GACrossSite & xs){
    if(&xs == this || !sameClass(xs)) return;
    GA1DArray1PtCrossSite<T> & orig = (GA1DArray1PtCrossSite<T> &)xs;
    x=orig.x; x1=orig.x1; x2=orig.x2;
  }
  virtual GACrossSite * clone() const
    {return new GA1DArray1PtCrossSite<T>(*this);}

  virtual void pick(GAGenome &);
  virtual void pick(GAGenome &, const GACrossSite & s){copy(s);}
  virtual void pick(GAGenome &, const GAGenome&, const GAGenome&);
  virtual void mirror(GAGenome &, const GAGenome &);
  int x, x1, x2;
};

#endif






#if USE_2PT_CROSSOVER == 1
/* ----------------------------------------------------------------------------
 1D Array Two Point Crossover
---------------------------------------------------------------------------- */
template <class T>
class GA1DArray2PtCrossSite : public GACrossSite {
  GADefineIdentity("GA1DArray2PtCrossSite", 86):

public:
  GA1DArray2PtCrossSite(){x[0]=x[1]=x1[0]=x1[1]=x2[0]=x2[1]=gaNoSite;}
  GA1DArray2PtCrossSite(int a, int b){x[0]=x1[0]=x2[0]=a; x[1]=x1[1]=x2[1]=b;}
  GA1DArray2PtCrossSite(const GA1DArray2PtCrossSite<T> & orig){copy(orig);}
  GA1DArray2PtCrossSite<T> & operator=(const GACrossSite & orig)
    {copy(orig); return *this;}
  GA1DArray2PtCrossSite<T> & operator=(const GA1DArray2PtCrossSite<T> & orig)
    {copy(orig); return *this;}
  virtual ~GA1DArray2PtCrossSite(){}
  virtual void copy(const GACrossSite & xs){
    if(&xs == this || !sameClass(xs)) return;
    GA1DArray2PtCrossSite<T> & orig = (GA1DArray2PtCrossSite<T> &)xs;
    x[0]=orig.x[0]; x1[0]=orig.x1[0]; x2[0]=orig.x2[0];
    x[1]=orig.x[1]; x1[1]=orig.x1[1]; x2[1]=orig.x2[1];
  }
  virtual GACrossSite * clone() const
    {return new GA1DArray2PtCrossSite<T>(*this);}

  virtual void pick(GAGenome &);
  virtual void pick(GAGenome &, const GACrossSite & s){copy(s);}
  virtual void pick(GAGenome &, const GAGenome&, const GAGenome&);
  virtual void mirror(GAGenome &, const GAGenome &);
  int x[2], x1[2], x2[2];
};

#endif


#if USE_PARTIAL_MATCH_CROSSOVER == 1 || \
    USE_ORDER_CROSSOVER == 1 || \
    USE_CYCLE_CROSSOVER == 1
/* ----------------------------------------------------------------------------
ArrayOrderedCrossoverSite - for any order-based crossovers on 1d arrays
---------------------------------------------------------------------------- */
template <class T>
class GA1DArrayOrderedCrossSite : public GACrossSite {
  GADefineIdentity("GA1DArrayOrderedCrossSite", 87):

public:
  GA1DArrayOrderedCrossSite(){a=gaNoSite; b=gaNoSite;}
  GA1DArrayOrderedCrossSite(int x, int y){a=x; b=y;}
  GA1DArrayOrderedCrossSite(const GA1DArrayOrderedCrossSite & orig)
    {copy(orig);}
  GA1DArrayOrderedCrossSite<T> & operator=(const GACrossSite & orig)
    {copy(orig); return *this;}
  GA1DArrayOrderedCrossSite<T> & operator=(const GA1DArrayOrderedCrossSite<T> & orig)
    {copy(orig); return *this;}
  virtual ~GA1DArrayOrderedCrossSite(){}
  virtual void copy(const GACrossSite & xs){
    if(&xs == this || !sameClass(xs)) return;
    GA1DArrayOrderedCrossSite<T> & orig = (GA1DArrayOrderedCrossSite<T> &)xs;
    a=orig.a; b=orig.b;
  }
  virtual GACrossSite * clone() const
    {return new GA1DArrayOrderedCrossSite<T>(*this);}

  virtual void pick(GAGenome &);
  virtual void pick(GAGenome &, const GACrossSite & s){copy(s);}
  virtual void pick(GAGenome &,const GAGenome&, const GAGenome&);
  virtual void mirror(GAGenome &, const GAGenome &);
  int a, b;
};

#endif


#ifdef BORLAND_INST
#include <ga/array1.xs.C>
#endif

#endif
