// $Header: /usr/people/mbwall/src/ga/ga/RCS/array2.ch.h,v 1.3 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  array2.ch.h
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the interface for the 2D array genome.  See comments in
1D array file.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_array2_ch_h_
#define _ga_array2_ch_h_

#include <ga/arraytmpl.h>
#include <ga/genome.h>
#include <ga/allele.h>


/* ----------------------------------------------------------------------------
2DArrayGenome
-------------------------------------------------------------------------------
---------------------------------------------------------------------------- */
template <class T>
class GA2DArrayGenome : public GAGenome, public GAArray<T> {
  GADefineIdentity("GA2DArrayGenome", 62):

public:
  GA2DArrayGenome(const unsigned int x, const unsigned int y,
		  GAAlleleSet<T> & a,
		  GAObjectiveFunction f=(GAObjectiveFunction)0,
		  void * u=(void *)0,
		  GA * g=(GA *)0);
  GA2DArrayGenome(const GA2DArrayGenome<T> & orig);
  GA2DArrayGenome& operator=(const GA2DArrayGenome<T>& orig)
    {copy(orig); return *this;}
  GA2DArrayGenome& operator=(const GAGenome& orig)
    {copy(orig); return *this;}
  GA2DArrayGenome& operator=(const T array []){
    for(int i=0; i<nx; i++)
      for(int j=0; j<ny; j++)
	gene(i,j) = *(array+j*nx+i);
    return *this;
  }
  virtual ~GA2DArrayGenome(){}
  virtual GAGenome * clone(const GACloneMethod flag=gaCloneContents) const;
  virtual void copy(const GAGenome & chrom);

  friend ostream & operator<<(ostream & os, const GA2DArrayGenome<T> & arg)
      {arg._write(os); return os;}
  friend istream & operator>>(istream & is, GA2DArrayGenome<T> & arg)
      {arg._read(is); return is;}
  friend int operator==(const GA2DArrayGenome<T> & a,
			const GA2DArrayGenome<T> & b)
    {return a._equal(b);}
  friend int operator!=(const GA2DArrayGenome<T> & a,
			const GA2DArrayGenome<T> & b)
    {return a._notequal(b);}

  const T & allele(const unsigned int i) const {return as->allele(i);}
  GAAlleleSet<T> & alleles() const {return *as;}
  GAAlleleSet<T> & alleles(GAAlleleSet<T> & set){as = &set; return *as;}

  const T & gene(const unsigned int x, const unsigned int y) const {
    return a[y*nx+x];
  }
  T & gene(const unsigned int x, const unsigned int y) {
    return a[y*nx+x];
  }
  T & gene(const unsigned int x, const unsigned int y, const T & value) {
    if(a[y*nx+x] != value) a[y*nx+x] = value;
    return a[y*nx+x];
  }
  int width() const {return nx;}
  int width(const int w){resize(w, ny); return nx;}
  int height() const {return ny;}
  int height(const int h){resize(nx, h); return ny;}
  int resize(const int x, const int y);
  int resizeBehaviour(const GADimension which) const {
    return((which == gaWidth) ? (maxX == minX ? gaNoResize : (int)maxX) :
	   ((which == gaHeight) ? (maxY == minY ? gaNoResize : (int)maxY):0));
  }
  int resizeBehaviour(const GADimension which,
		      const unsigned int lower, const unsigned int upper);
  int resizeBehaviour(const unsigned int lowerX, const unsigned int upperX, 
		      const unsigned int lowerY, const unsigned int upperY){
    return(resizeBehaviour(gaWidth, lowerX, upperX) * 
	   resizeBehaviour(gaHeight, lowerY, upperY));
  }
  void copy(const GA2DArrayGenome &, 
	    const unsigned int, const unsigned int,
	    const unsigned int, const unsigned int, 
	    const unsigned int, const unsigned int);
  void swap(const unsigned int a, const unsigned int b,
	    const unsigned int c, const unsigned int d){
    GAArray<T>::swap(b*nx+a, d*nx+c);
  }

protected:
  GAAlleleSet<T> * as;		// the allele set (pt to original, not copy)
  unsigned int nx, ny, minX, minY, maxX, maxY;
  void _read(istream & is=cin);
  void _write (ostream & os=cout) const;
  int _equal(const GAGenome & c) const;
  int _notequal(const GAGenome & c) const;
};

#ifdef BORLAND_INST
#include <ga/array2.ch.C>
#endif

#endif


