// $Header: /usr/people/mbwall/src/ga/ga/RCS/array2.op.C,v 1.2 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  array2.op.C
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Outline code for doing built-in operators for the 2D array genome.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <ga/config.h>
#include <ga/random.h>
#include <ga/array2.ch.h>
#include <ga/array2.xs.h>
#include <ga/array2.op.h>


/* ----------------------------------------------------------------------------
2D Array Genome
---------------------------------------------------------------------------- */
#if USE_RANDOM_INITIALIZER == 1
void 
GA2DArrayRandomInitializer(GAGenome & c)
{
  GA2DArrayGenome<ARRAY_TYPE> &child=(GA2DArrayGenome<ARRAY_TYPE> &)c;
  child.resize(gaResizeAny,gaResizeAny); // let chrom resize if it can
  for(int i=child.width()-1; i>=0; i--)
    for(int j=child.height()-1; j>=0; j--)
      child.gene(i, j, child.alleles().allele());
}
#endif


#if USE_FLIP_MUTATOR == 1
int 
GA2DArrayFlipMutator(GAGenome & c, float pmut)
{
  GA2DArrayGenome<ARRAY_TYPE> &child=(GA2DArrayGenome<ARRAY_TYPE> &)c;
  register int n, m, i, j;
  if(pmut <= 0.0) return(0);

  float nMut = pmut * (float)(child.size());
  if(nMut < 1.0){		// we have to do a flip test on each bit
    nMut = 0;
    for(i=child.width()-1; i>=0; i--){
      for(j=child.height()-1; j>=0; j--){
	if(GAFlipCoin(pmut)){
	  child.gene(i, j, child.alleles().allele());
	  nMut++;
	}
      }
    }
  }
  else{				// only flip the number of bits we need to flip
    for(n=0; n<nMut; n++){
      m = GARandomInt(0, child.size()-1);
      i = m % child.width();
      j = m / child.width();
      child.gene(i, j, child.alleles().allele());
    }
  }
  return((int)nMut);
}
#endif


#if USE_SWAP_MUTATOR == 1
int 
GA2DArraySwapMutator(GAGenome & c, float pmut)
{
  GA2DArrayGenome<ARRAY_TYPE> &child=(GA2DArrayGenome<ARRAY_TYPE> &)c;
  register int n, i;
  if(pmut <= 0.0) return(0);

  float nMut = pmut * (float)(child.size());
  int size = child.size()-1;
  if(nMut < 1.0){		// we have to do a flip test on each bit
    nMut = 0;
    for(i=size; i>=0; i--){
      if(GAFlipCoin(pmut)){
	child.GAArray<ARRAY_TYPE>::swap(i, GARandomInt(0, size));
	nMut++;
      }
    }
  }
  else{				// only flip the number of bits we need to flip
    for(n=0; n<nMut; n++)
      child.GAArray<ARRAY_TYPE>::swap(GARandomInt(0,size),GARandomInt(0,size));
  }
  return((int)nMut);
}
#endif



// Randomly take elements from each parent
#if USE_UNIFORM_CROSSOVER == 1
void
GA2DArrayUniformCrossover(GAGenome &c, const GAGenome &c1, const GAGenome &c2)
{
  GA2DArrayGenome<ARRAY_TYPE> &child=(GA2DArrayGenome<ARRAY_TYPE> &)c;
  GA2DArrayGenome<ARRAY_TYPE> &mom=(GA2DArrayGenome<ARRAY_TYPE> &)c1;
  GA2DArrayGenome<ARRAY_TYPE> &dad=(GA2DArrayGenome<ARRAY_TYPE> &)c2;

  if(child.resizeBehaviour(gaWidth) == gaNoResize ||
     child.resizeBehaviour(gaHeight) == gaNoResize){
    if(mom.width() != dad.width() || mom.height() != dad.height()){
      GAErr(c.className(), "GA2DArrayUniformCrossover", 
	    gaErrParentSizeMismatch);
      return;
    }
    if(child.width() != mom.width() || child.height() != mom.height()){
      GAErr(c.className(), "GA2DArrayUniformCrossover",
	    gaErrChildSizeMismatch);
      return;
    }
  }
  else{
    GAErr(c.className(), "GA2DArrayUniformCrossover", gaErrFixedOnly);
    return;
  }
  
  for(int i=child.width()-1; i>=0; i--)
    for(int j=child.height()-1; j>=0; j--)
      child.gene(i,j,(GARandomBit() ? mom.gene(i,j) : dad.gene(i,j)));
}
#endif


// This crossover does clipping (no padding) for resizables
#if USE_1PT_CROSSOVER == 1
void
GA2DArray1PtCrossover(GAGenome & c,
		      const GAGenome & c1, const GAGenome & c2)
{
  GA2DArrayGenome<ARRAY_TYPE> &child=(GA2DArrayGenome<ARRAY_TYPE> &)c;
  GA2DArrayGenome<ARRAY_TYPE> &mom=(GA2DArrayGenome<ARRAY_TYPE> &)c1;
  GA2DArrayGenome<ARRAY_TYPE> &dad=(GA2DArrayGenome<ARRAY_TYPE> &)c2;
  GA2DArray1PtCrossSite<ARRAY_TYPE> & xsite = 
    (GA2DArray1PtCrossSite<ARRAY_TYPE> &)child.crossoverSite();

  unsigned int x, y, x1, y1, x2, y2;
  if(child.resizeBehaviour(gaWidth) == gaNoResize){
    if(mom.width() != dad.width()){
      GAErr(c.className(), "GA2DArray1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.width() != mom.width()){
      GAErr(c.className(), "GA2DArray1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.x == gaNoSite) xsite.x = GARandomInt(0,child.width());
    xsite.x1 = xsite.x2 = xsite.x;
    x1 = x2 = x = child.width() - xsite.x;
  }
  else{
    if(xsite.x1 == gaNoSite) xsite.x1 = GARandomInt(0, mom.width());
    if(xsite.x2 == gaNoSite) xsite.x2 = GARandomInt(0, dad.width());
    xsite.x = Min(xsite.x1,xsite.x2);
    x1 = mom.width() - xsite.x1;
    x2 = dad.width() - xsite.x2; 
    x = Min(x1,x2);
  }
  
  if(child.resizeBehaviour(gaHeight) == gaNoResize){
    if(mom.height() != dad.height()){
      GAErr(c.className(), "GA2DArray1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.height() != mom.height()){
      GAErr(c.className(), "GA2DArray1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.y == gaNoSite) xsite.y = GARandomInt(0,child.height());
    xsite.y1 = xsite.y2 = xsite.y;
    y1 = y2 = y = child.height() - xsite.y;
  }
  else{
    if(xsite.y1 == gaNoSite) xsite.y1 = GARandomInt(0, mom.height());
    if(xsite.y2 == gaNoSite) xsite.y2 = GARandomInt(0, dad.height());
    xsite.y = Min(xsite.y1,xsite.y2);
    y1 = mom.height() - xsite.y1;
    y2 = dad.height() - xsite.y2; 
    y = Min(y1,y2);
  }
  child.resize(xsite.x+x, xsite.y+y);

  child.copy(mom, 
	     0, 0,
	     xsite.x1-xsite.x, xsite.y1-xsite.y,
	     xsite.x, xsite.y);
  child.copy(mom,
	     xsite.x, xsite.y,
	     xsite.x1, xsite.y1,
	     x, y);
  child.copy(dad,
	     0, xsite.y,
	     0, xsite.y2,
	     xsite.x, y);
  child.copy(dad, 
	     xsite.x, 0,
	     xsite.x2, 0,
	     x, xsite.y);
}
#endif


#if USE_EVEN_CROSSOVER == 1
void
GA2DArrayEvenCrossover(GAGenome & c,
			const GAGenome & c1, const GAGenome & c2)
{
  GA2DArrayGenome<ARRAY_TYPE> &child=(GA2DArrayGenome<ARRAY_TYPE> &)c;
  GA2DArrayGenome<ARRAY_TYPE> &mom=(GA2DArrayGenome<ARRAY_TYPE> &)c1;
  GA2DArrayGenome<ARRAY_TYPE> &dad=(GA2DArrayGenome<ARRAY_TYPE> &)c2;

  if(child.size() > mom.size() || child.size() > dad.size()){
    GAErr(c.className(), "GA2DArrayEvenCrossover", gaErrChildSizeMismatch);
    return;
  }
  for(int i=0; i<child.width(); i++)
    for(int j=0; j<child.height(); j++)
      child.gene(i, j, (((i*child.height()+j)%2 == 0) ?
			mom.gene(i,j) : dad.gene(i,j)));
}
#endif

#if USE_ODD_CROSSOVER == 1
void
GA2DArrayOddCrossover(GAGenome & c,
		       const GAGenome & c1, const GAGenome & c2)
{
  GA2DArrayGenome<ARRAY_TYPE> &child=(GA2DArrayGenome<ARRAY_TYPE> &)c;
  GA2DArrayGenome<ARRAY_TYPE> &mom=(GA2DArrayGenome<ARRAY_TYPE> &)c1;
  GA2DArrayGenome<ARRAY_TYPE> &dad=(GA2DArrayGenome<ARRAY_TYPE> &)c2;

  if(child.size() > mom.size() || child.size() > dad.size()){
    GAErr(c.className(), "GA2DArrayOddCrossover", gaErrChildSizeMismatch);
    return;
  }
  for(int i=0; i<child.width(); i++)
    for(int j=0; j<child.height(); j++)
      child.gene(i, j, (((i*child.height()+j)%2 == 0) ?
			dad.gene(i,j) : mom.gene(i,j)));
}
#endif
