// $Header: /usr/people/mbwall/src/ga/ga/RCS/array3.ch.C,v 1.2 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  array3.ch.C
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Source file for the 3D array genome.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include <ga/array3.ch.h>


/* ----------------------------------------------------------------------------
3DArrayGenome
---------------------------------------------------------------------------- */
template <class T> 
GA3DArrayGenome<T>::
GA3DArrayGenome(const unsigned int width,
		    const unsigned int height, 
		    const unsigned int depth,
		    GAAlleleSet<T> & s,
		    GAObjectiveFunction f,
		    void * u,
		    GA * g) :
GAGenome(GANoInitializer, GANoMutator, GANoCrossover, (GACrossSite *)0),
GAArray<T>(width*height*depth){
  of=f; ud=u; ga=g; as=&s;
  nx=minX=maxX=width; ny=minY=maxY=height; nz=minZ=maxZ=depth;
}


template <class T> 
GA3DArrayGenome<T>::
GA3DArrayGenome(const GA3DArrayGenome<T> & orig) : GAArray<T>(orig.sz){
  GA3DArrayGenome<T>::copy(orig);
}


template <class T> void
GA3DArrayGenome<T>::copy(const GAGenome & orig){
  if(&orig == this) return;
  if(!sameClass(orig)){
    GAErr(className(), "copy", gaErrObjectTypeMismatch);
    return;
  }
  GA3DArrayGenome<T> & c = (GA3DArrayGenome<T> &)orig;
  GAGenome::copy(c);
  GAArray<T>::copy(c);
  as = c.as;
  nx = c.nx; ny = c.ny; nz = c.nz;
  minX = c.minX; minY = c.minY; minZ = c.minZ;
  maxX = c.maxX; maxY = c.maxY; maxZ = c.maxZ;
}


template <class T> GAGenome *
GA3DArrayGenome<T>::clone(const GACloneMethod flag) const {
  GA3DArrayGenome<T> *cpy = new GA3DArrayGenome<T>(nx,ny,nz,*as);
  if(flag == gaCloneContents){
    cpy->copy(*this);
  }
  else{
    cpy->GAGenome::copy(*this);
    cpy->as = as;
    cpy->minX = minX; cpy->minY = minY; cpy->minZ = minZ; 
    cpy->maxX = maxX; cpy->maxY = maxY; cpy->maxZ = maxZ;
  }
  return cpy;
}


// *** doesn't try to copy previous bits!!!  won't work on complex objects
template <class T> int
GA3DArrayGenome<T>::resize(const int w, const int h, const int d)
{
  int width=w, height=h, depth=d;
  if(width == nx && height == ny && depth == nz) return sz;

  if(minX == maxX){		// resizing is not allowed for this one
    if(width < 0) return nx;	// do nothing
    minX = maxX = width;	// resize to specified width
  }
  else if(width >=0){
    if(width < minX) minX = width;
    if(width > maxX) maxX = width;
  }
  else width = GARandomInt(minX, maxX);

  if(minY == maxY){		// resizing is not allowed for this one
    if(height < 0) return ny;	// do nothing
    minY = maxY = height;	// resize to specified height
  }
  else if(height >=0){
    if(height < minY) minY = height;
    if(height > maxY) maxY = height;
  }
  else height = GARandomInt(minY, maxY);

  if(minZ == maxZ){		// resizing is not allowed for this one
    if(depth < 0) return nz;	// do nothing
    minZ = maxZ = depth;	// resize to specified depth
  }
  else if(depth >=0){
    if(depth < minZ) minZ = depth;
    if(depth > maxZ) maxZ = depth;
  }
  else depth = GARandomInt(minZ, maxZ);

  if(width < nx && height < ny){
    int z=Min(nz,depth);
    for(int k=0; k<z; k++)
      for(int j=0; j<height; j++)
	GAArray<T>::move(k*height*width+j*width,k*ny*nx+j*nx,width);
  }
  else if(width < nx){
    int z=Min(nz,depth);
    for(int k=0; k<z; k++)
      for(int j=0; j<ny; j++)
	GAArray<T>::move(k*ny*width+j*width,k*ny*nx+j*nx,width);
  }
  else if(height < ny){
    int z=Min(nz,depth);
    for(int k=0; k<z; k++)
      for(int j=0; j<height; j++)
	GAArray<T>::move(k*height*nx+j*nx,k*ny*nx+j*nx,nx);
  }

  GAArray<T>::size(width*height*depth);

  if(width > nx && height > ny){ // adjust the existing chunks of bits
    int z=Min(nz,depth);
    for(int k=z-1; k>=0; k--){
      int j;
      for(j=ny-1; j>=0; j--){
	GAArray<T>::move(k*height*width+j*width,k*ny*nx+j*nx,nx);
	for(int i=nx; i<width; i++)
	  a[k*height*width+j*width+i] = as->allele();
      }
      for(j=ny; j<height; j++)
	for(int i=0; i<width; i++)
	  a[k*height*width+j*width+i] = as->allele();
    }
  }
  else if(width > nx){
    int z=Min(nz,depth);
    for(int k=z-1; k>=0; k--){
      for(int j=height-1; j>=0; j--){
	GAArray<T>::move(k*height*width+j*width,k*height*nx+j*nx,nx);
	for(int i=nx; i<width; i++)
	  a[k*height*width+j*width+i] = as->allele();
      }
    }
  }
  else if(height > ny){
    int z=Min(nz,depth);
    for(int k=z-1; k>=0; k--){
      int j;
      for(j=ny-1; j>=0; j--)
	GAArray<T>::move(k*height*width+j*width,k*ny*width+j*width,width);
      for(j=ny; j<height; j++)
	for(int i=0; i<width; i++)
	  a[k*height*width+j*width+i] = as->allele();
    }
  }
  if(depth > nz){		// change in depth is always new bits
    for(int i=width*height*nz; i<width*height*depth; i++)
      a[i] = as->allele();
  }

  nx = width; ny = height; nz = depth;
  _evaluated = gaFalse; _nmut = 0;
  return sz;
}


template <class T> void
GA3DArrayGenome<T>::_read(istream & is)
{
  int i=0, j=0, k=0;
  do{
    is >> gene(i++, j, k);
    if(i >= nx){
      i=0;
      j++;
    }
    if(j >= ny){
      j=0;
      k++;
    }
  } while(is && is.peek() != EOF && k < nz);

  if(is.peek() == EOF && 
     ((k < nz) ||		// didn't get some lines
      (j < ny && j != 0) ||	// didn't get some lines
      (i < nx && i != 0)))	// didn't get some lines
    GAErr(className(), "_read", gaErrUnexpectedEOF);
}


template <class T> void
GA3DArrayGenome<T>::_write(ostream & os) const 
{
  for(int k=0; k<nz; k++){
    for(int j=0; j<ny; j++){
      for(int i=0; i<nx; i++){
	os << gene(i,j,k);
      }
      os << "\n";
    }
    os << "\n";
  }
}


template <class T> int
GA3DArrayGenome<T>::
resizeBehaviour(const GADimension which,
		const unsigned int lower, const unsigned int upper)
{
  if(upper < lower){
    GAErr(className(), "resizeBehaviour", gaErrBadResizeBehaviour);
    return resizeBehaviour(which);
  }

  switch(which){
  case gaWidth:
    minX = lower; maxX = upper;
    if(nx > upper) GA3DArrayGenome<T>::resize(upper,ny,nz);
    if(nx < lower) GA3DArrayGenome<T>::resize(lower,ny,nz);
    break;

  case gaHeight:
    minY = lower; maxY = upper;
    if(ny > upper) GA3DArrayGenome<T>::resize(nx,upper,nz);
    if(ny < lower) GA3DArrayGenome<T>::resize(nx,lower,nz);
    break;

  case gaDepth:
    minZ = lower; maxZ = upper;
    if(nz > upper) GA3DArrayGenome<T>::resize(nx,ny,upper);
    if(nz < lower) GA3DArrayGenome<T>::resize(nx,ny,lower);
    break;

  default:
    break;
  }

  return resizeBehaviour(which);
}


template <class T> void
GA3DArrayGenome<T>::
copy(const GA3DArrayGenome<T> & orig,
     const unsigned int r, const unsigned int s, const unsigned int t,
     const unsigned int x, const unsigned int y, const unsigned int z,
     const unsigned int width, const unsigned int height,
     const unsigned int depth)
{
  int w=width, h=height, d=depth;
  if(w == 0 || x >= orig.nx || r >= nx ||
     h == 0 || y >= orig.ny || s >= ny ||
     d == 0 || z >= orig.nz || t >= nz) return;
  if(x + w > orig.nx) w = orig.nx - x;
  if(y + h > orig.ny) h = orig.ny - y;
  if(z + d > orig.nz) d = orig.nz - z;
  if(r + w > nx) w = nx - r;
  if(s + h > ny) h = ny - s;
  if(t + d > nz) d = nz - t;

  for(int k=0; k<d; k++)
    for(int j=0; j<h; j++)
      GAArray<T>::copy(orig,
		       (t+k)*ny*nx + (s+j)*nx + r,
		       (z+k)*orig.ny*orig.nx + (y+j)*orig.nx + x, w);
}


template <class T> int 
GA3DArrayGenome<T>::_equal(const GAGenome & c) const
{
  if(this == &c) return 1;
  GA3DArrayGenome<T> & b = (GA3DArrayGenome<T> &)c;
  if(nx != b.nx || ny != b.ny || nz != b.nz) return 0;
  int val=0;
  for(int k=0; k<nz && val==0; k++)
    for(int j=0; j<ny && val==0; j++)
      val = GAArray<T>::equal(b,k*ny*nx,k*ny*nx,nx) ? 0 : 1;
  return(val ? 0 : 1);
}


template <class T> int 
GA3DArrayGenome<T>::_notequal(const GAGenome & c) const
{
  if(this == &c) return 0;
  GA3DArrayGenome<T> & b = (GA3DArrayGenome<T> &)c;
  if(nx != b.nx || ny != b.ny || nz != b.nz) return 1;
  int val=0;
  for(int k=0; k<nz && val==0; k++)
    for(int j=0; j<ny && val==0; j++)
      val = GAArray<T>::equal(b,k*ny*nx,k*ny*nx,nx) ? 0 : 1;
  return(val ? 1 : 0);
}
