// $Header: /usr/people/mbwall/src/ga/ga/RCS/array3.ch.h,v 1.3 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  array3.ch.h
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the interface for the 3D array genome.  See comments in
1D array file.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_array3_ch_h_
#define _ga_array3_ch_h_

#include <ga/arraytmpl.h>
#include <ga/genome.h>
#include <ga/allele.h>


/* ----------------------------------------------------------------------------
3DArrayGenome
-------------------------------------------------------------------------------
---------------------------------------------------------------------------- */
template <class T>
class GA3DArrayGenome : public GAGenome, public GAArray<T> {
  GADefineIdentity("GA3DArrayGenome", 63):

public:
  GA3DArrayGenome(const unsigned int x,
		  const unsigned int y,
		  const unsigned int z,
		  GAAlleleSet<T> & a,
		  GAObjectiveFunction f=(GAObjectiveFunction)0,
		  void * u=(void *)0,
		  GA * g=(GA *)0);
  GA3DArrayGenome(const GA3DArrayGenome & orig);
  GA3DArrayGenome& operator=(const GA3DArrayGenome<T>& orig)
    {copy(orig); return *this;}
  GA3DArrayGenome& operator=(const GAGenome& orig)
    {copy(orig); return *this;}
  GA3DArrayGenome& operator=(const T array []){
    for(int i=0; i<nx; i++)
      for(int j=0; j<ny; j++)
	for(int k=0; k<nz; k++)
	  gene(i,j,k) = *(array+k*ny*nx+j*nx+i);
    return *this;
  }
  virtual ~GA3DArrayGenome(){}
  virtual GAGenome * clone(const GACloneMethod flag=gaCloneContents) const;
  virtual void copy(const GAGenome & chrom);

  friend ostream & operator<<(ostream & os, const GA3DArrayGenome<T> & arg)
      {arg._write(os); return os;}
  friend istream & operator>>(istream & is, GA3DArrayGenome<T> & arg)
      {arg._read(is); return is;}
  friend int operator==(const GA3DArrayGenome<T> & a,
			const GA3DArrayGenome<T> & b)
    {return a._equal(b);}
  friend int operator!=(const GA3DArrayGenome<T> & a,
			const GA3DArrayGenome<T> & b)
    {return a._notequal(b);}

  const T & allele(const unsigned int i) const {return as->allele(i);}
  GAAlleleSet<T> & alleles() const {return *as;}
  GAAlleleSet<T> & alleles(GAAlleleSet<T> & set){as = &set; return *as;}

  const T & gene(const unsigned int x, const unsigned int y,
	   const unsigned int z) const {
    return a[z*ny*nx + y*nx + x];
  }
  T & gene(const unsigned int x, const unsigned int y,
	   const unsigned int z) {
    return a[z*ny*nx + y*nx + x];
  }
  T & gene(const unsigned int x, const unsigned int y,
	   const unsigned int z, const T & value){
    if(a[z*ny*nx + y*nx + x] != value) a[z*ny*nx + y*nx + x] = value;
    return a[z*ny*nx + y*nx + x];
  }
  int width() const {return nx;}
  int width(const int w){resize(w, ny, nz); return nx;}
  int height() const {return ny;}
  int height(const int h){resize(nx, h, nz); return ny;}
  int depth() const {return nz;}
  int depth(const int d){resize(nx, ny, d); return nz;}
  int resize(const int x, const int y, const int z);
  int resizeBehaviour(const GADimension which) const {
    return((which == gaWidth) ? (maxX == minX ? gaNoResize : (int)maxX) :
	   ((which == gaHeight) ? (maxY == minY ? gaNoResize : (int)maxY) : 
	    ((which == gaDepth) ? (maxZ == minZ ? gaNoResize : (int)maxZ):0)));
  }
  int resizeBehaviour(const GADimension which,
		      const unsigned int lower, const unsigned int upper);
  int resizeBehaviour(const unsigned int lowerX, const unsigned int upperX, 
		      const unsigned int lowerY, const unsigned int upperY, 
		      const unsigned int lowerZ, const unsigned int upperZ){
    return(resizeBehaviour(gaWidth,  lowerX, upperX) * 
	   resizeBehaviour(gaHeight, lowerY, upperY) * 
	   resizeBehaviour(gaDepth,  lowerZ, upperZ));
  }
  void copy(const GA3DArrayGenome &, 
	    const unsigned int, const unsigned int, const unsigned int,
	    const unsigned int, const unsigned int, const unsigned int,
	    const unsigned int, const unsigned int, const unsigned int);
  void swap(const unsigned int a, const unsigned int b, const unsigned int c,
	    const unsigned int d, const unsigned int e, const unsigned int f){
    GAArray<T>::swap(c*ny*nx+b*nx+a, f*ny*nx+e*nx+d);
  }

protected:
  GAAlleleSet<T> * as;		// the allele set (pt to original, not copy)
  unsigned int nx, ny, nz, minX, minY, minZ, maxX, maxY, maxZ;
  void _read(istream & is=cin);
  void _write (ostream & os=cout) const;
  int _equal(const GAGenome & c) const;
  int _notequal(const GAGenome & c) const;
};

#ifdef BORLAND_INST
#include <ga/array3.ch.C>
#endif

#endif
