// $Header: /usr/people/mbwall/src/ga/ga/RCS/array3.op.C,v 1.2 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  array3.op.C
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Outline code for doing built-in operators for the 3D array genome.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <ga/config.h>
#include <ga/random.h>
#include <ga/array3.ch.h>
#include <ga/array3.xs.h>
#include <ga/array3.op.h>


/* ----------------------------------------------------------------------------
3D Array Genome
---------------------------------------------------------------------------- */
#if USE_RANDOM_INITIALIZER == 1
void 
GA3DArrayRandomInitializer(GAGenome & c)
{
  GA3DArrayGenome<ARRAY_TYPE> &child=(GA3DArrayGenome<ARRAY_TYPE> &)c;
  child.resize(gaResizeAny,gaResizeAny,gaResizeAny); // let chrom resize
  for(int i=child.width()-1; i>=0; i--)
    for(int j=child.height()-1; j>=0; j--)
      for(int k=child.depth()-1; k>=0; k--)
	child.gene(i, j, k, child.alleles().allele());
}
#endif

#if USE_FLIP_MUTATOR == 1
int 
GA3DArrayFlipMutator(GAGenome & c, float pmut)
{
  GA3DArrayGenome<ARRAY_TYPE> &child=(GA3DArrayGenome<ARRAY_TYPE> &)c;
  register int n, m, d, i, j, k;
  if(pmut <= 0.0) return(0);

  float nMut = pmut * (float)(child.size());
  if(nMut < 1.0){		// we have to do a flip test on each bit
    nMut = 0;
    for(i=child.width()-1; i>=0; i--){
      for(j=child.height()-1; j>=0; j--){
        for(k=child.depth()-1; k>=0; k--){
	  if(GAFlipCoin(pmut)){
	    child.gene(i, j, k, child.alleles().allele());
	    nMut++;
	  }
	}
      }
    }
  }
  else{				// only flip the number of bits we need to flip
    for(n=0; n<nMut; n++){
      m = GARandomInt(0, child.size()-1);
      d = child.height() * child.depth();
      i = m / d;
      j = (m % d) / child.depth();
      k = (m % d) % child.depth();
      child.gene(i, j, k, child.alleles().allele());
    }
  }
  return((int)nMut);
}
#endif

#if USE_SWAP_MUTATOR == 1
int 
GA3DArraySwapMutator(GAGenome & c, float pmut)
{
  GA3DArrayGenome<ARRAY_TYPE> &child=(GA3DArrayGenome<ARRAY_TYPE> &)c;
  register int n, i;
  if(pmut <= 0.0) return(0);

  float nMut = pmut * (float)(child.size());
  int size = child.size()-1;
  if(nMut < 1.0){		// we have to do a flip test on each bit
    nMut = 0;
    for(i=size; i>=0; i--){
      if(GAFlipCoin(pmut)){
	child.GAArray<ARRAY_TYPE>::swap(i, GARandomInt(0, size));
	nMut++;
      }
    }
  }
  else{				// only flip the number of bits we need to flip
    for(n=0; n<nMut; n++)
      child.GAArray<ARRAY_TYPE>::swap(GARandomInt(0, size),
				      GARandomInt(0, size));
  }
  return((int)nMut);
}
#endif



// Randomly take elements from each parent.
#if USE_UNIFORM_CROSSOVER == 1
void
GA3DArrayUniformCrossover(GAGenome &c, const GAGenome &c1, const GAGenome &c2)
{
  GA3DArrayGenome<ARRAY_TYPE> &child=(GA3DArrayGenome<ARRAY_TYPE> &)c;
  GA3DArrayGenome<ARRAY_TYPE> &mom=(GA3DArrayGenome<ARRAY_TYPE> &)c1;
  GA3DArrayGenome<ARRAY_TYPE> &dad=(GA3DArrayGenome<ARRAY_TYPE> &)c2;

  if(child.resizeBehaviour(gaWidth) == gaNoResize ||
     child.resizeBehaviour(gaHeight) == gaNoResize ||
     child.resizeBehaviour(gaDepth) == gaNoResize){
    if(mom.width() != dad.width() ||
       mom.height() != dad.height() ||
       mom.depth() != dad.depth()){
      GAErr(c.className(), "GA3DArrayUniformCrossover", 
	    gaErrParentSizeMismatch);
      return;
    }
    if(child.width() != mom.width() ||
       child.height() != mom.height() || 
       child.depth() != mom.depth()){
      GAErr(c.className(), "GA3DArrayUniformCrossover",
	    gaErrChildSizeMismatch);
      return;
    }
  }
  else{
    GAErr(c.className(), "GA3DArrayUniformCrossover", gaErrFixedOnly);
    return;
  }
  
  for(int i=child.width()-1; i>=0; i--)
    for(int j=child.height()-1; j>=0; j--)
      for(int k=child.depth()-1; k>=0; k--)
      child.gene(i,j,k,(GARandomBit() ? mom.gene(i,j,k) : dad.gene(i,j,k)));
}
#endif


// this crossover does clipping (not padding) on resizables
#if USE_1PT_CROSSOVER == 1
void
GA3DArray1PtCrossover(GAGenome & c,
		       const GAGenome & c1, const GAGenome & c2)
{
  GA3DArrayGenome<ARRAY_TYPE> &child=(GA3DArrayGenome<ARRAY_TYPE> &)c;
  GA3DArrayGenome<ARRAY_TYPE> &mom=(GA3DArrayGenome<ARRAY_TYPE> &)c1;
  GA3DArrayGenome<ARRAY_TYPE> &dad=(GA3DArrayGenome<ARRAY_TYPE> &)c2;
  GA3DArray1PtCrossSite<ARRAY_TYPE> & xsite = 
    (GA3DArray1PtCrossSite<ARRAY_TYPE> &)child.crossoverSite();

  int x, y, z, x1, y1, z1, x2, y2, z2;
  if(child.resizeBehaviour(gaWidth) == gaNoResize){
    if(mom.width() != dad.width()){
      GAErr(c.className(), "GA3DArray1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.width() != mom.width()){
      GAErr(c.className(), "GA3DArray1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.x == gaNoSite) xsite.x = GARandomInt(0,child.width());
    xsite.x1 = xsite.x2 = xsite.x;
    x1 = x2 = x = child.width() - xsite.x;
  }
  else{
    if(xsite.x1 == gaNoSite) xsite.x1 = GARandomInt(0, mom.width());
    if(xsite.x2 == gaNoSite) xsite.x2 = GARandomInt(0, dad.width());
    xsite.x = Min(xsite.x1,xsite.x2);
    x1 = mom.width() - xsite.x1;
    x2 = dad.width() - xsite.x2; 
    x = Min(x1,x2);
  }
  
  if(child.resizeBehaviour(gaHeight) == gaNoResize){
    if(mom.height() != dad.height()){
      GAErr(c.className(), "GA3DArray1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.height() != mom.height()){
      GAErr(c.className(), "GA3DArray1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.y == gaNoSite) xsite.y = GARandomInt(0,child.height());
    xsite.y1 = xsite.y2 = xsite.y;
    y1 = y2 = y = child.height() - xsite.y;
  }
  else{
    if(xsite.y1 == gaNoSite) xsite.y1 = GARandomInt(0, mom.height());
    if(xsite.y2 == gaNoSite) xsite.y2 = GARandomInt(0, dad.height());
    xsite.y = Min(xsite.y1,xsite.y2);
    y1 = mom.height() - xsite.y1;
    y2 = dad.height() - xsite.y2; 
    y = Min(y1,y2);
  }
  
  if(child.resizeBehaviour(gaDepth) == gaNoResize){
    if(mom.depth() != dad.depth()){
      GAErr(c.className(), "GA3DArray1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.depth() != mom.depth()){
      GAErr(c.className(), "GA3DArray1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.z == gaNoSite) xsite.z = GARandomInt(0,child.depth());
    xsite.z1 = xsite.z2 = xsite.z;
    z1 = z2 = z = child.depth() - xsite.z;
  }
  else{
    if(xsite.z1 == gaNoSite) xsite.z1 = GARandomInt(0, mom.depth());
    if(xsite.z2 == gaNoSite) xsite.z2 = GARandomInt(0, dad.depth());
    xsite.z = Min(xsite.z1,xsite.z2);
    z1 = mom.depth() - xsite.z1;
    z2 = dad.depth() - xsite.z2; 
    z = Min(z1,z2);
  }
  child.resize(xsite.x+x, xsite.y+y, xsite.z+z);

  child.copy(mom,
	     0, 0, 0,
	     xsite.x1-xsite.x, xsite.y1-xsite.y, xsite.z1-xsite.z,
	     xsite.x, xsite.y, xsite.z);
  child.copy(mom,
	     xsite.x, xsite.y, 0,
	     xsite.x1, xsite.y1, xsite.z1-xsite.z,
	     x, y, xsite.z);
  child.copy(mom,
	     0, xsite.y, xsite.z,
	     xsite.x1-xsite.x, xsite.y1, xsite.z1,
	     xsite.x, y, z);
  child.copy(mom,
	     xsite.x, 0, xsite.z,
	     xsite.x1, xsite.y1-xsite.y, xsite.z1,
	     x, xsite.y, z);
  child.copy(dad,
	     0, xsite.y, 0,
	     xsite.x2-xsite.x, xsite.y2, xsite.z2-xsite.z,
	     xsite.x, y, xsite.z);
  child.copy(dad,
	     xsite.x, 0, 0,
	     xsite.x2, xsite.y2-xsite.y, xsite.z2-xsite.z,
	     x, xsite.y, xsite.z);
  child.copy(dad,
	     0, 0, xsite.z,
	     xsite.x2-xsite.x, xsite.y2-xsite.y, xsite.z2,
	     xsite.x, xsite.y, z);
  child.copy(dad,
	     xsite.x, xsite.y, xsite.z,
	     xsite.x2, xsite.y2, xsite.z2,
	     x, y, z);
}
#endif

#if USE_EVEN_CROSSOVER == 1
void
GA3DArrayEvenCrossover(GAGenome & c,
			const GAGenome & c1, const GAGenome & c2)
{
  GA3DArrayGenome<ARRAY_TYPE> &child=(GA3DArrayGenome<ARRAY_TYPE> &)c;
  GA3DArrayGenome<ARRAY_TYPE> &mom=(GA3DArrayGenome<ARRAY_TYPE> &)c1;
  GA3DArrayGenome<ARRAY_TYPE> &dad=(GA3DArrayGenome<ARRAY_TYPE> &)c2;

  if(child.size() > mom.size() || child.size() > dad.size()){
    GAErr(c.className(), "GA3DArrayEvenCrossover", gaErrChildSizeMismatch);
    return;
  }
  for(int i=0; i<child.width(); i++)
    for(int j=0; j<child.height(); j++)
      for(int k=0; j<child.depth(); k++)
	child.gene(i, j, k, 
		   (((i*child.height()*child.depth()+j*child.depth()+k)%2==0) ?
		    mom.gene(i,j,k) : dad.gene(i,j,k)));
}
#endif

#if USE_ODD_CROSSOVER == 1
void
GA3DArrayOddCrossover(GAGenome & c,
		       const GAGenome & c1, const GAGenome & c2)
{
  GA3DArrayGenome<ARRAY_TYPE> &child=(GA3DArrayGenome<ARRAY_TYPE> &)c;
  GA3DArrayGenome<ARRAY_TYPE> &mom=(GA3DArrayGenome<ARRAY_TYPE> &)c1;
  GA3DArrayGenome<ARRAY_TYPE> &dad=(GA3DArrayGenome<ARRAY_TYPE> &)c2;

  if(child.size() > mom.size() || child.size() > dad.size()){
    GAErr(c.className(), "GA3DArrayOddCrossover", gaErrChildSizeMismatch);
    return;
  }
  for(int i=0; i<child.width(); i++)
    for(int j=0; j<child.height(); j++)
      for(int k=0; j<child.depth(); k++)
	child.gene(i, j, k, 
		   (((i*child.height()*child.depth()+j*child.depth()+k)%2==0) ?
		    dad.gene(i,j,k) : mom.gene(i,j,k)));
}
#endif
