// $Header: /usr/people/mbwall/src/ga/ga/RCS/bin2dec.C,v 1.2 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  bin2dec.C
  mbwall 29jun95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Binary-to-decimal converters.  temporary until we do the bit string object.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <stdio.h>
#include <limits.h>
#include <string.h>
#include <ga/config.h>
#include <ga/error.h>
#include <ga/bin2dec.h>

static char errbuf1[80];
static char errbuf2[80];
void _GAEncodeBase(int, unsigned long int, GABit *, int, int);

/* ----------------------------------------------------------------------------
  Binary to decimal conversion.  When we read in a stream of bits, the first
bit that we get is the least significant bit (ie 2^0) then we go up from there.
This is backwards from what you might usually write (where the least significan
bit is on the right side instead of the left) but it makes it easier to read in
the bits from a stream, compare strings, etc.
  The alleles for a binary string are (1,0).  A value of 0 is unset, anything 
else is set.
  I have defined a char version for testing purposes.  In that one, the allele
set is (1,0) and ('1','0').
---------------------------------------------------------------------------- */
// The string must be null-terminated or you'll get bogus results.
float 
GABin2Dec(const char * s)
{
  if(s == (char *)NULL) return 0.0;
  return GABin2Dec(s, (int)strlen(s));
}

float 
GABin2Dec(const char * s, const unsigned int l)
{
  if(s == (char *)NULL) return(0.0);
  int length = l;
  if(length > _GA_MAX_BITS){
    GAErr("GABin2Dec", gaErrBinStrTooLong);
    length = _GA_MAX_BITS;
  }
  unsigned long int tmp=1;
  float sum=0.0;
  for(int i=length-1; i>(-1); i--){   // most significant bit first
//  for(int i=0; i<length; i++){ // least significant bit first
    if(s[i] != 0 && s[i] != '0') sum += tmp;
    tmp <<= 1;
  }
  return(sum);
}

float 
GABin2Dec(const char * s, const unsigned int l,
	  const float min, const float max)
{
  if(s == (char *)NULL) return(0.0);
  int length = l;
  if(length > _GA_MAX_BITS){
    GAErr("GABin2Dec", gaErrBinStrTooLong);
    length = _GA_MAX_BITS;
  }
  unsigned long int tmp=1;		// is a long enough bits?
  float sum=0.0;
  for(int i=length-1; i>(-1); i--){   // most significant bit first
//  for(int i=0; i<length; i++){ // least significant bit first
    if(s[i] != 0 && s[i] != '0') sum += tmp;
    tmp <<= 1;
  }
  float sumMax = tmp - 1;
  return(min + (max - min)*sum/sumMax);
}




// Convert the string of bits into a decimal value.  The first version of this
// routine does no scaling - you'll get some power of two back.  The second
// version scales the result to the range of values that you pass to it.
//   I just duplicated everything because its a pretty small little program.
float 
GABin2Dec(const GABit * s, const unsigned int l)
{
  if(s == (GABit *)0) return(0.0);
  int length = l;
  if(length > _GA_MAX_BITS){
    GAErr("GABin2Dec", gaErrBinStrTooLong);
    length = _GA_MAX_BITS;
  }
  unsigned long int tmp=1;
  float sum=0.0;
  for(int i=length-1; i>(-1); i--){   // most significant bit first
//  for(int i=0; i<length; i++){ // least significant bit first
    if(s[i] != 0) sum += tmp;
    tmp <<= 1;
  }
  return(sum);
}

float 
GABin2Dec(const GABit * s, const unsigned int length,
	  const float min, const float max)
{
  if(s == (GABit *)NULL) return(0.0);
  int n = length;
  if(n > _GA_MAX_BITS){
    GAErr("GABin2Dec", gaErrBinStrTooLong);
    n = _GA_MAX_BITS;
  }

//  unsigned long int maxint=1, sum=0;
//  for(int i=0; i<n; i++)	// 0th bit is most significant
//    if(s[i]) sum += maxint << n-i-1;
//  maxint <<= n;
//  maxint--;
//  return(min + (max - min)*(float)sum/(float)maxint);

  unsigned long int maxint=1;
  float sum=0;
  for(int i=n-1; i>(-1); i--){	// 0th bit is most significant
    if(s[i]) sum += maxint;
    maxint <<= 1;
  }
  maxint--;
  return(min + (max - min)*sum/(float)maxint);

//  unsigned long int tmp=1;
//  float sum=0.0;
//  for(int i=length-1; i>(-1); i--){   // most significant bit first
//  for(int i=0; i<length; i++){ // least significant bit first
//    if(s[i] != 0) sum += tmp;
//    tmp <<= 1;
//  }
//  float sumMax = tmp - 1;
//  return(min + (max - min)*sum/sumMax);
}



/* ----------------------------------------------------------------------------
  Utility routine to encode bits of a decimal number.  This routine recursively
loops through the decimal value and grabs the remainder (modulo the base) and
sticks the result into each 'bit' (if base is 2 then they're bits, otherwise 
they're the equivalent for whatever base you're doing).
  This will only do unsigned integers.
---------------------------------------------------------------------------- */
void
_GAEncodeBase(int base, unsigned long int val, GABit * binstr, int n, int c)
{
  if(c < 0) return;		// if this happens we should post an error
				// it means we didn't get a perfect encoding
  binstr[c] = val % base;
  unsigned long int quotient = val / base;
  if(quotient) _GAEncodeBase(base, quotient, binstr, n, c-1);
}


/* ----------------------------------------------------------------------------
  Decimal to binary conversion.  Returns the string that was passed to us.  No
NULL termination.  If there aren't enough bits in the string to do the 
representation, we represent all that we can then return.  We do not notify
the caller that the conversion was incomplete.
  This encodes the bits with the most significant bit first (to left of all 
other bits).  A higher index in the binstr will yield a less significant bit.
---------------------------------------------------------------------------- */
float
GADec2Bin(const unsigned long int d, GABit * binstr, const unsigned int nbits)
{
  if(binstr == (GABit *)NULL) return 0;
  memset(binstr, 0, nbits*sizeof(GABit));
  _GAEncodeBase(2, d, binstr, 0, nbits-1);
  return d;
}


/* ----------------------------------------------------------------------------
  This uses a threshhold rounding.  First we figure out what our discretization
will be based on the bit-resolution we're given (the number of bits we can use
to represent the interval).  Then we start at the min value and increment up 
until we reach the desired value, then stop.  We use the corresponding integer
value to do the encoding.
  We go up to, then settle on the first interval past the one we equal.
  Convert the min-max range to integer range that the bits can handle.
  We use unsigned long so the sign bit doesn't flip should we leftshift past 
the last bit.
---------------------------------------------------------------------------- */
float
GADec2Bin(const float val, GABit * binstr, const unsigned int nbits,
	  const float minval, const float maxval)
{
  if(binstr == (GABit *)NULL || nbits <= 0) return 0;
  if(val < minval || maxval < val) return 0;
  int n=nbits;
  if(n > _GA_MAX_BITS){
    GAErr("GABin2Dec", gaErrBinStrTooLong);
    n = _GA_MAX_BITS;
  }

  unsigned long int nintervals=1; // this limits the number of bits we can do
  if(n == _GA_MAX_BITS)		  // get the power of 2 we need
    nintervals = ULONG_MAX;
  else {
    nintervals <<= n;
    nintervals--;
  }
  float interval = maxval-minval;
  interval /= (float)nintervals;       // interval is the discretization size
  float sum = (val-minval)/interval;   // figure out how many intervals we need
  if(n == _GA_MAX_BITS)
    nintervals = (unsigned long int)sum; // make it an integer
  else
    nintervals = (long int)sum;          // make it an integer
  sum = minval+(float)nintervals*interval; // get the value we can represent

  if(sum != val){
    sprintf(errbuf1,"desired: %f\tactual: %f\tdiscretization: %f",
	    val,sum,interval);
    sprintf(errbuf2,"  nbits: %d\t\tmin: %f\t\tmax: %f",n,minval,maxval);
    GAErr("GADec2Bin", gaErrDataLost, errbuf1, errbuf2);
  }

  memset(binstr, 0, n*sizeof(GABit));
  _GAEncodeBase(2, nintervals, binstr, 0, n-1);
  return(sum);
}
