// $Header: /usr/people/mbwall/src/ga/ga/RCS/bin2dec.ch.C,v 1.7 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  bin2dec.ch.C
  mbwall 19apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Source file for the binary-to-decimal genome.

 TO DO:

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <ctype.h>
#include <string.h>
#include <ga/error.h>
#include <ga/bin2dec.h>
#include <ga/bin2dec.ch.h>


void
GABin2DecGenome::copy(const GAGenome & orig)
{
  if(&orig == this) return;
  if(!sameClass(orig)){
    GAErr(className(), "copy", gaErrObjectTypeMismatch);
    return;
  }
  GABin2DecGenome & c = (GABin2DecGenome &)orig;
  GA1DBinaryStringGenome::copy(orig);
  phenotypes(c.phenotypes());
}


// We keep only a pointer to the phenotype, we do not make a copy of it.  The
// intent is to reduce the overhead of the genome's memory requirements.
// This means that you'd better not destroy the phenotype before the genome
// are finished using it.
GABin2DecPhenotype &
GABin2DecGenome::phenotypes(GABin2DecPhenotype & p)
{
  ptype = &p;
  GA1DBinaryStringGenome::resize(p.size());
  return *ptype;
}


// We access the data string directly here.  This could be dangerous (if the
// bitstream ever changes on us it will affect the way this method sees the
// data string).
//   Eventually we may need to cache the decimal values in an array of floats,
// but for now we call the converter routine every time each phenotype is 
// requested.
float
GABin2DecGenome::phenotype(const int n) const
{
  if(n < 0 || n >= ptype->nPhenotypes()){
    GAErr(className(), "phenotype", gaErrBadPhenotypeID);
    return(0.0);
  }
  return(GABin2Dec(&(data[ptype->offset(n)]), ptype->length(n), 
		   ptype->min(n), ptype->max(n)));
}


// Set the bits of the binary string based on the decimal value that is passed
// to us.  Notice that the number you pass may or may not be set properly.  It
// depends on the resolution defined in the phenotype.  If you didn't define 
// enough resolution, then there may be no way to represent the number.
//   We round off to the closest representable value, then return the number 
// that we actually entered (the rounded value).
// *** this is dangerous!  we're accessing the superclass' data representation
// directly, so if the representation changes to a bit stream, this will break.
float
GABin2DecGenome::phenotype(const int n, const float v)
{
  if(n < 0 || n >= ptype->nPhenotypes()){
    GAErr(className(), "phenotype", gaErrBadPhenotypeID);
    return 0;
  }
  if(v < ptype->min(n) || v > ptype->max(n)){
    GAErr(className(), "phenotype", gaErrBadPhenotypeValue);
    return 0;
  }
  return(GADec2Bin(v,
		   &(data[ptype->offset(n)]), ptype->length(n),
		   ptype->min(n), ptype->max(n)));
}


// Read the incoming data.  To do this we need to know what kind of data is
// coming our way.  For now we'll only read binary data (ie streams of 0 and 1)
// until I figure out a better way to implement this.
// *** do a better read method for the bin2dec genome!
void
GABin2DecGenome::_read(istream & is)
{
  int isbinary=1;		// *** for now assume everything is binary

  if(isbinary){
    GA1DBinaryStringGenome::_read(is);
  }
  else{
    float value;
    for(int i=0; i<phenotypes().nPhenotypes(); i++){
      is >> value;
      phenotype(i, value);
    }
  }
}


void
GABin2DecGenome::_write(ostream & os) const 
{
  for(int i=0; i<phenotypes().nPhenotypes(); i++)
    os << phenotype(i) << " ";
}


// For two bin2dec genomes to be equal they must have the same bits AND the
// same phenotypes.  For now all we check for is the bits.  
int 
operator==(const GABin2DecGenome & a, const GABin2DecGenome & b)
{
  return((operator==((GA1DBinaryStringGenome &)a,
		     (GA1DBinaryStringGenome &)b) && 
	  (a.ptype == b.ptype || a.phenotypes() == b.phenotypes())) ?
	 gaTrue : gaFalse);
}


int 
operator!=(const GABin2DecGenome & a, const GABin2DecGenome & b)
{
  return((operator!=((GA1DBinaryStringGenome &)a,
		     (GA1DBinaryStringGenome &)b) ||
	  a.phenotypes() != b.phenotypes()) ?
	 gaTrue : gaFalse);
}
