// $Header: /usr/people/mbwall/src/ga/ga/RCS/bin2dec.ch.h,v 1.6 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  bin2dec.ch.h
  mbwall 19apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the interface for the binary-to-decimal genome.

 TO DO:
*** Eventually we may want to cache the values in the phenotype mapping, but 
    for now we'll generate them on the fly.
*** Need to write a _read method that can interpret binary/decimal input.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_bin2dec_ch_h_
#define _ga_bin2dec_ch_h_

#include <ga/binstr1.ch.h>
#include <ga/bin2dec.ph.h>

/* ----------------------------------------------------------------------------
Bin2DecGenome
-------------------------------------------------------------------------------
  We'll assume that the user allocates space for the phenotype and is 
responsible for freeing that space.  We don't make a copy - we just keep a
pointer to the object, so be sure the user doesn't free the space behind our
backs!
---------------------------------------------------------------------------- */
class GABin2DecGenome : public GA1DBinaryStringGenome {
  GADefineIdentity("GABin2DecGenome", 60):

public:
  GABin2DecGenome(GABin2DecPhenotype & p,
		      GAObjectiveFunction f=(GAObjectiveFunction)0,
		      void * u=(void *)0,
		      GA * g=(GA *)0):
  GA1DBinaryStringGenome(p.size(), f, u, g)
    {init(); phenotypes(p);}
  GABin2DecGenome(const GABin2DecGenome & orig):
  GA1DBinaryStringGenome(orig.sz)
    {init(); copy(orig);}
  GABin2DecGenome& operator=(const GABin2DecGenome& arg)
    {copy(arg); return *this;}
  GABin2DecGenome& operator=(const GAGenome& arg)
    {copy(arg); return *this;}
  virtual ~GABin2DecGenome(){}
  virtual GAGenome *clone(const GACloneMethod flag=gaCloneContents) const {
    GABin2DecGenome *cpy = new GABin2DecGenome(*ptype,of,ud,ga);
    if(flag == gaCloneContents){
      cpy->copy(*this);
    }
    else{
      cpy->GAGenome::copy(*this); 
      cpy->maxX = maxX; cpy->minX = minX;
    }
    return cpy;
  }
  virtual void copy(const GAGenome & chrom);

  friend ostream & operator<<(ostream & os, const GABin2DecGenome & arg)
    {arg._write(os); return os;}
  friend istream & operator>>(istream & is, GABin2DecGenome & arg)
    {arg._read(is); return is;}
  friend int operator==(const GABin2DecGenome &, const GABin2DecGenome &);
  friend int operator!=(const GABin2DecGenome &, const GABin2DecGenome &);

// specific to this class
  GABin2DecPhenotype & phenotypes(GABin2DecPhenotype & p);
  GABin2DecPhenotype & phenotypes() const {return *ptype;}
  int nPhenotypes() const {return ptype->nPhenotypes();}
  float phenotype(const int n, const float val);
  float phenotype(const int n) const;

protected:
  GABin2DecPhenotype * ptype;
  void _read (istream & is=cin);
  void _write (ostream & os=cout) const;

private:
  void init(){ptype=(GABin2DecPhenotype *)0;}
};


#endif
