// $Header: /usr/people/mbwall/src/ga/ga/RCS/bin2dec.ph.C,v 1.4 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  bin2dec.ph.C
  mbwall 14aug94
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:

 TO DO:

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <string.h>
#include <ga/types.h>
#include <ga/bin2dec.ph.h>

#define GA_B2D_CHUNKSIZE 20

/* ----------------------------------------------------------------------------
 Bin2DecPhenotype

  This is the phenotype for converting binary strings to decimal values.  There
are limits to the size of the numbers you can use (ie you're limited to the
number of bits that can represent a float - see the utilities file for more 
information).
---------------------------------------------------------------------------- */
GABin2DecPhenotype::GABin2DecPhenotype(){
  csz = GA_B2D_CHUNKSIZE;
  N = n = sz = 0;
  nbits = oset = 0;
  minval = maxval = 0;
}


GABin2DecPhenotype::~GABin2DecPhenotype(){
  delete [] nbits; 
  delete [] oset;
  delete [] minval; 
  delete [] maxval; 
}


void
GABin2DecPhenotype::copy(const GABin2DecPhenotype & p){
  csz = p.csz;
  N = p.N;
  n = p.n;
  sz = p.sz;

  delete [] nbits;
  delete [] oset;
  delete [] minval;
  delete [] maxval;

  nbits = new unsigned short[N];
  oset = new unsigned short[N];
  minval = new float[N];
  maxval = new float[N];

  memcpy(nbits, p.nbits, n*sizeof(unsigned short));
  memcpy(oset, p.oset, n*sizeof(unsigned short));
  memcpy(minval, p.minval, n*sizeof(float));
  memcpy(maxval, p.maxval, n*sizeof(float));
}


// Add another phenotype to this phenotype object.  If needed, we allocate more
// space, otherwise just tag the new on then end.  We allocate space in chunks
// so we don't spend too much time doing memory allocation stuff.
void
GABin2DecPhenotype::add(const unsigned int nb,const float min,const float max){
  if(n + 1 > N){
    N += csz;

    unsigned short *nbtmp = nbits;
    nbits = new unsigned short[N];
    memcpy(nbits, nbtmp, n*sizeof(unsigned short));
    delete [] nbtmp;

    unsigned short *ostmp = oset;
    oset = new unsigned short[N];
    memcpy(oset, ostmp, n*sizeof(unsigned short));
    delete [] ostmp;

    float *mintmp = minval;
    minval = new float[N];
    memcpy(minval, mintmp, n*sizeof(float));
    delete [] mintmp;

    float *maxtmp = maxval;
    maxval = new float[N];
    memcpy(maxval, maxtmp, n*sizeof(float));
    delete [] maxtmp;
  }
  nbits[n] = nb;
  if(n > 0)
    oset[n] = oset[n-1] + nbits[n-1];
  else
    oset[n] = 0;
  minval[n] = min;
  maxval[n] = max;
  n++;
  sz += nb;
}


void
GABin2DecPhenotype::compact(){
  if(n == N) return;
  N = n;

  unsigned short *nbtmp = nbits;
  nbits = new unsigned short[N];
  memcpy(nbits, nbtmp, n*sizeof(unsigned short));
  delete [] nbtmp;

  unsigned short *ostmp = oset;
  oset = new unsigned short[N];
  memcpy(oset, ostmp, n*sizeof(unsigned short));
  delete [] ostmp;

  float *mintmp = minval;
  minval = new float[N];
  memcpy(minval, mintmp, n*sizeof(float));
  delete [] mintmp;

  float *maxtmp = maxval;
  maxval = new float[N];
  memcpy(maxval, maxtmp, n*sizeof(float));
  delete [] maxtmp;
}


int 
operator==(const GABin2DecPhenotype & a, const GABin2DecPhenotype & b){
  if(a.sz != b.sz || a.n != b.n) return gaFalse;
  if(memcmp(a.nbits, b.nbits, a.n*sizeof(unsigned short)) != 0 ||
     memcmp(a.oset, b.oset, a.n*sizeof(unsigned short)) != 0 ||
     memcmp(a.minval, b.minval, a.n*sizeof(float)) != 0 ||
     memcmp(a.maxval, b.maxval, a.n*sizeof(float)))
    return gaFalse;
  return(gaTrue);
}


int 
operator!=(const GABin2DecPhenotype & a, const GABin2DecPhenotype & b){
  if(a.sz != b.sz || a.n != b.n) return gaTrue;
  if(memcmp(a.nbits, b.nbits, a.n*sizeof(unsigned short)) != 0 ||
     memcmp(a.oset, b.oset, a.n*sizeof(unsigned short)) != 0 ||
     memcmp(a.minval, b.minval, a.n*sizeof(float)) != 0 ||
     memcmp(a.maxval, b.maxval, a.n*sizeof(float)))
    return gaTrue;
  return(gaFalse);
}
