// $Header: /usr/people/mbwall/src/ga/ga/RCS/binstr.h,v 1.3 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  binstr.h
  mbwall 30jun95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the interface for the binary string genome.  This is
temporary until we implement a proper bit string object.

 TO DO:
  we can get major improvements to speed by inlining functions and getting rid
of the error checking...  for example, inlining genome and removing the 
conditional makes it go from 7.5 seconds to 3.2 seconds (bm bl cs 1024 c 0.9)

  implement bitstream version of the binary string genomes

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_binstr_h_
#define _ga_binstr_h_

#include <ga/random.h>
#include <ga/genome.h>



// This defines what type to use for the bitstream data storage.  Use the
// smallest type available on your platform.  I'd like to use a real bitstream
// here, but then I'd have to do bit copies across word boundaries, and I'm
// not sure how to do that efficiently.
#define GA_BINSTR_DATA_TYPE unsigned char

#define GA_BINSTR_CHUNKSIZE 50	  // size of the chunks of bits we allocate


/* ----------------------------------------------------------------------------
BinaryStringGenome
-------------------------------------------------------------------------------
  This class should never be instantiated.  Here we define some of the members
and functions that are common to all binary string genomes of any
dimension.  Any instance of a binary string genome must have dimension,
however, so this class is pure virtual.  Derived classes need the resize, size,
etc members as well as the data member.  This class defines how the memory is
allocated and deallocated, derived classes define how the data should be 
interpreted.
  Binary string genomes are all represented as a sequence of bits.  We use
a single bitstream of longs.  Each derived class figures out whether the 
bitstream should be interpreted as a single dimension or multiple dimension
string.
  I'd like to do this as a stream of bits, but I don't know how to efficiently
copy bitstreams that may not fall on word boundaries.  If your machine supports
a data type smaller than a char, then change the type so that you use that
rather than the default char.
---------------------------------------------------------------------------- */
class GABinaryStringGenome : public GAGenome {
public:
  GABinaryStringGenome(GAInitializationOperator i=GANoInitializer,
		       GAMutationOperator m=GANoMutator,
		       GACrossoverOperator c=GANoCrossover,
		       GACrossSite * s=(GACrossSite *)0):
  GAGenome(i,m,c,s),sz(0)
    {csz=GA_BINSTR_CHUNKSIZE; sz=0; SZ=0; data=(GA_BINSTR_DATA_TYPE *)0;}
  virtual ~GABinaryStringGenome(){delete [] data;}
  virtual void copy(const GAGenome &);
  virtual GAGenome * clone(const GACloneMethod) const=0;
  int resize(const unsigned int);		// pass desired size, in bits
  int size(){return(sz);}

protected:
  short bit(const unsigned int a) const {
    return(data[a]);
  }
  short bit(const unsigned int a, const short val) {	// set/unset the bit
    _evaluated = gaFalse; _nmut = 0;
    return(data[a] = (val ? 1 : 0));
  }
  int equal(const GABinaryStringGenome & b,
	    const unsigned int r, const unsigned int x, const unsigned int l)
    const {
    return(memcmp(&(data[r]),&(b.data[x]),l*sizeof(GA_BINSTR_DATA_TYPE))?0:1);
  }
  void copy(const GABinaryStringGenome & orig,
	    const unsigned int r, const unsigned int x, unsigned int l){
    memcpy(&(data[r]), &(orig.data[x]), l*sizeof(GA_BINSTR_DATA_TYPE));
  }
  void move(const unsigned int r, const unsigned int x, unsigned int l){
    memmove(&(data[r]), &(data[x]), l*sizeof(GA_BINSTR_DATA_TYPE));
  }
  void set(const unsigned int a, const unsigned int l){
    memset(&(data[a]), 1, l*sizeof(GA_BINSTR_DATA_TYPE));
  }
  void unset(const unsigned int a, const unsigned int l){
    memset(&(data[a]), 0, l*sizeof(GA_BINSTR_DATA_TYPE));
  }
  void randomize(const unsigned int a, const unsigned int l){
    for(int i=0; i<l; i++) data[i+a] = (GA_BINSTR_DATA_TYPE)GARandomBit();
  }
  unsigned int sz;		// size of chrom
  unsigned int SZ;		// size of the memory allocated
  unsigned int csz;		// size of chunks we allocate
  GA_BINSTR_DATA_TYPE *data;	// the data themselves
};

#endif
