// $Header: /usr/people/mbwall/src/ga/ga/RCS/binstr1.ch.h,v 1.3 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  binstr1.ch.h
  mbwall 19apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the interface for the 1D binary string genome.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_binstr1_ch_h_
#define _ga_binstr1_ch_h_

#include <ga/config.h>
#include <ga/binstr.h>
#include <ga/genome.h>


/* ----------------------------------------------------------------------------
1DBinaryStringGenome
-------------------------------------------------------------------------------
bitcopy

  Copy bits from the specified genome using the location and length 
that are passed to us.  If the current genome is too short for the 
entire length, copy whatever we can.  If the original genome is too 
short for the specified length, copy whatever we can.  If either location
is out of bounds, return without doing anything.
  We do NOT check for negative values in the locations!
  This routine clips if the copy sizes do not match - it does NOT resize the
genome to fit the copy.  You'll have to do resizes before you call this
routine if you want the copy to fit the original.

==, !=
  Are two genomes equal?  Our test for equality is based upon the
contents of the genome, NOT the behaviour.  So as long as the bitstreams
match, the genomes are 'equal'.  This means that a resizeable genome
may be equal to a fixed-length genome.  But a chromsome with 500
bits allocated is not equal to a genome with 10 bits allocated unless
both are the same size.
---------------------------------------------------------------------------- */
class GA1DBinaryStringGenome : public GABinaryStringGenome {
  GADefineIdentity("GA1DBinaryStringGenome", 51):

public:
  GA1DBinaryStringGenome(const unsigned int x,
			     GAObjectiveFunction f=(GAObjectiveFunction)0,
			     void * u=(void *)0,
			     GA * g=(GA *)0);
  GA1DBinaryStringGenome(const GA1DBinaryStringGenome & orig);
  GA1DBinaryStringGenome& operator=(const GA1DBinaryStringGenome& arg)
    {copy(arg); return *this;}
  GA1DBinaryStringGenome& operator=(const GAGenome& arg)
    {copy(arg); return *this;}
  GA1DBinaryStringGenome& operator=(const short array []) // no err checks!
    {for(int i=0; i<sz; i++) gene(i,*(array+i)); return *this;}
  virtual ~GA1DBinaryStringGenome(){}
  virtual GAGenome *clone(const GACloneMethod flag=gaCloneContents) const {
    GA1DBinaryStringGenome *cpy = new 
      GA1DBinaryStringGenome(nx,of,ud,ga);
    if(flag == gaCloneContents){
      cpy->copy(*this);
    }
    else{
      cpy->GAGenome::copy(*this);
      cpy->maxX = maxX; cpy->minX = minX;
    }
    return cpy;
  }
  virtual void copy(const GAGenome & chrom);

  friend ostream & 
    operator<<(ostream & os, const GA1DBinaryStringGenome & arg)
      {arg._write(os); return os;}
  friend istream &
    operator>>(istream & is, GA1DBinaryStringGenome & arg)
      {arg._read(is); return is;}
  friend int operator==(const GA1DBinaryStringGenome & a,
			const GA1DBinaryStringGenome & b)
    {return a._equal(b);}
  friend int operator!=(const GA1DBinaryStringGenome & a,
			const GA1DBinaryStringGenome & b)
    {return a._notequal(b);}

// specific to this class
  short gene(const unsigned int x=0) const {
    return GABinaryStringGenome::bit(x);
  }
  short gene(const unsigned int x, const short value) {
    return((GABinaryStringGenome::bit(x) == value) ? value :
	   GABinaryStringGenome::bit(x,value));
  }
  short operator[](const unsigned int x) {return gene(x);}
  int length() const {return nx;}
  int length(const int x){resize(x); return nx;}
  int resize(const int x);
  int resizeBehaviour() const {return((maxX == minX) ? gaNoResize : maxX);}
  int resizeBehaviour(const unsigned int lx, const unsigned int ux);
  void copy(const GA1DBinaryStringGenome & orig,
	    const unsigned int r, const unsigned int x,
	    const unsigned int length){
    int l = length;
    if(l > 0 && x < orig.nx && r < nx){
      if(x + l > orig.nx) l = orig.nx - x;
      if(r + l > nx) l = nx - r;
      GABinaryStringGenome::copy(orig,r,x,l);
    }
  }
  void set(const unsigned int x, const unsigned int length){
    int l = length;
    if(x + l > nx) l = nx - x;
    GABinaryStringGenome::set(x, l);
  }
  void unset(const unsigned int x, const unsigned int length){
    int l = length;
    if(x + l > nx) l = nx - x;
    GABinaryStringGenome::unset(x, l);
  }

protected:
  unsigned int nx;		// how long is the data string?
  unsigned int minX;		// what is lower limit?
  int maxX;			// what is the limit?  0 is x, <0 is no limit
  void _read(istream & is);
  void _write (ostream & os) const;
  int _equal(const GAGenome & c) const {
    GA1DBinaryStringGenome & b = (GA1DBinaryStringGenome &)c;
    return((this == &c) ? 1 :
	   ((nx != b.nx) ? 0 :
	    GABinaryStringGenome::equal(b,0,0,nx)));
  }
  int _notequal(const GAGenome & c) const {
    GA1DBinaryStringGenome & b = (GA1DBinaryStringGenome &)c;
    return((this == &c) ? 0 :
	   ((nx != b.nx) ? 1 :
	    GABinaryStringGenome::equal(b,0,0,nx) ? 0 : 1));
  }
};

#endif
