// $Header: /usr/people/mbwall/src/ga/ga/RCS/binstr1.xs.h,v 1.1 1995/06/30 21:27:55 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  binstr1.xs.h
  mbwall 19apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the 1D binary string genome crossover sites.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_binstr1_xs_h_
#define _ga_binstr1_xs_h_

#include <ga/config.h>
#include <ga/xsite.h>

#if USE_1PT_CROSSOVER == 1 
/* ----------------------------------------------------------------------------
 1D Binary String Single Point Crossover
---------------------------------------------------------------------------- */
class GA1DBinStr1PtCrossSite : public GACrossSite {
  GADefineIdentity("GA1DBinStr1PtCrossSite", 76):

public:
  GA1DBinStr1PtCrossSite(){x=x1=x2=gaNoSite;}
  GA1DBinStr1PtCrossSite(int a){x=x1=x2=a;}
  GA1DBinStr1PtCrossSite(const GA1DBinStr1PtCrossSite & orig){copy(orig);}
  GA1DBinStr1PtCrossSite & operator=(const GACrossSite & orig)
    {copy(orig); return *this;}
  GA1DBinStr1PtCrossSite & operator=(const GA1DBinStr1PtCrossSite & orig)
    {copy(orig); return *this;}
  virtual ~GA1DBinStr1PtCrossSite(){}
  virtual void copy(const GACrossSite & xs){
    if(&xs != this && sameClass(xs)){
      GA1DBinStr1PtCrossSite & orig = (GA1DBinStr1PtCrossSite &)xs;
      x=orig.x; x1=orig.x1; x2=orig.x2;
    }
  }
  virtual GACrossSite * clone() const
    {return new GA1DBinStr1PtCrossSite(*this);}

  virtual void pick(GAGenome &);
  virtual void pick(GAGenome &, const GACrossSite & s){copy(s);}
  virtual void pick(GAGenome &, const GAGenome&, const GAGenome&);
  virtual void mirror(GAGenome &,const GAGenome &);
  int x, x1, x2;
};
#endif




#if USE_2PT_CROSSOVER == 1 
/* ----------------------------------------------------------------------------
 1D Binary String Two Point Crossover
---------------------------------------------------------------------------- */
class GA1DBinStr2PtCrossSite : public GACrossSite {
  GADefineIdentity("GA1DBinStr2PtCrossSite", 77):

public:
  GA1DBinStr2PtCrossSite(){x[0]=x[1]=x1[0]=x1[1]=x2[0]=x2[1]=gaNoSite;}
  GA1DBinStr2PtCrossSite(int a, int b){x[0]=x1[0]=x2[0]=a; x[1]=x1[1]=x2[1]=b;}
  GA1DBinStr2PtCrossSite(const GA1DBinStr2PtCrossSite & orig){copy(orig);}
  GA1DBinStr2PtCrossSite & operator=(const GACrossSite & orig)
    {copy(orig); return *this;}
  GA1DBinStr2PtCrossSite & operator=(const GA1DBinStr2PtCrossSite & orig)
    {copy(orig); return *this;}
  virtual ~GA1DBinStr2PtCrossSite(){}
  virtual void copy(const GACrossSite & xs){
    if(&xs != this && sameClass(xs)){
      GA1DBinStr2PtCrossSite & orig = (GA1DBinStr2PtCrossSite &)xs;
      x[0]=orig.x[0]; x1[0]=orig.x1[0]; x2[0]=orig.x2[0];
      x[1]=orig.x[1]; x1[1]=orig.x1[1]; x2[1]=orig.x2[1];
    }
  }
  virtual GACrossSite * clone() const
    {return new GA1DBinStr2PtCrossSite(*this);}

  virtual void pick(GAGenome &);
  virtual void pick(GAGenome &, const GACrossSite & s){copy(s);}
  virtual void pick(GAGenome &, const GAGenome&, const GAGenome&);
  virtual void mirror(GAGenome &,const GAGenome &);
  int x[2], x1[2], x2[2];
};

#endif

#endif
