// $Header: /usr/people/mbwall/src/ga/ga/RCS/binstr2.ch.C,v 1.2 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  binstr2.ch.C
  mbwall 19apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Source file for the 2D binary string genome.  See the 1D genome for comments.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <ctype.h>
#include <ga/config.h>
#include <ga/error.h>
#include <ga/random.h>
#include <ga/binstr2.ch.h>
#include <ga/binstr2.xs.h>
#include <ga/binstr2.op.h>


/* ----------------------------------------------------------------------------
2DBinaryStringGenome
---------------------------------------------------------------------------- */
static DEFAULT_2DBINSTR_CROSSSITE ga2DBinStrDefCrossSite;

GA2DBinaryStringGenome::
GA2DBinaryStringGenome(const unsigned int width,
		       const unsigned int height, 
		       GAObjectiveFunction f,
		       void * u,
		       GA * g) :
GABinaryStringGenome(DEFAULT_2DBINSTR_INITIALIZER,
		     DEFAULT_2DBINSTR_MUTATOR,
		     DEFAULT_2DBINSTR_CROSSOVER,
		     &ga2DBinStrDefCrossSite)
{
  of=f; ud=u; ga=g;
  nx=minX=maxX=0; ny=minY=maxY=0;
  resize(width, height);
}


GA2DBinaryStringGenome::
GA2DBinaryStringGenome(const GA2DBinaryStringGenome & orig)
{
  nx=minX=maxX=0; ny=minY=maxY=0;
  copy(orig);
}


void
GA2DBinaryStringGenome::copy(const GAGenome & orig)
{
  if(&orig == this) return;
  if(!sameClass(orig)){
    GAErr(className(), "copy", gaErrObjectTypeMismatch);
    return;
  }
  GABinaryStringGenome::copy(orig);
  GA2DBinaryStringGenome & c = (GA2DBinaryStringGenome &)orig;
  nx = c.nx; ny = c.ny; 
  minX = c.minX; minY = c.minY; 
  maxX = c.maxX; maxY = c.maxY;
} 


int
GA2DBinaryStringGenome::resize(const int w, const int h)
{
  int width=w, height=h;
  if(width == nx && height == ny) return sz;

  if(minX == maxX){		// resizing is not allowed for this one
    if(width < 0) return nx;	// do nothing
    minX = maxX = width;	// resize to specified width
  }
  else if(width >=0){
    if(width < minX) minX = width;
    if(width > maxX) maxX = width;
  }
  else width = GARandomInt(minX, maxX);

  if(minY == maxY){		// resizing is not allowed for this one
    if(height < 0) return ny;	// do nothing
    minY = maxY = height;	// resize to specified height
  }
  else if(height >=0){
    if(height < minY) minY = height;
    if(height > maxY) maxY = height;
  }
  else height = GARandomInt(minY, maxY);

// Move the bits into the right position.  If we're smaller, then shift to
// the smaller size before we do the resize (the resize method maintains bit
// integrety).  If we're larger, do the move after the resize.  If we're the 
// same size the we don't do anything.  When we're adding more bits, the new
// bits get set randomly to 0 or 1.

  if(width < nx){
    int y=Min(ny,height);
    for(int j=0; j<y; j++)
      move(j*width,j*nx,width);
  }
  GABinaryStringGenome::resize(width*height);
  if(width > nx){		// adjust the existing chunks of bits
    int y=Min(ny,height);
    for(int j=y-1; j>=0; j--){
      move(j*width,j*nx,nx);
      for(int i=nx; i<width; i++)
	bit(j*width+i, GARandomBit());
    }
  }
  if(height > ny){		// change in height is always new bits
    for(int i=width*ny; i<width*height; i++)
      bit(i, GARandomBit());
  }

  nx = width; ny = height;
  return sz;
}


void
GA2DBinaryStringGenome::_read(istream & is)
{
  static char c;
  int i=0, j=0;
  do{
    is >> c;
    if(isdigit(c)){
      gene(i, j, ((c == '0') ? 0 : 1));
      if(++i >= nx){		// ready for next row
	i=0;
	j++;
      }
    }
  } while(is && is.peek() != EOF && j < ny);

  if(is.peek() == EOF && 
     ((j < ny) ||	     // didn't get some lines
      (i < nx && i != 0)))   // stopped early on a row
    GAErr(className(), "_read", gaErrUnexpectedEOF);
}


void
GA2DBinaryStringGenome::_write(ostream & os) const 
{
  for(int j=0; j<ny; j++){
    for(int i=0; i<nx; i++)
      os << gene(i,j);
    os << "\n";
  }
}


int
GA2DBinaryStringGenome::
resizeBehaviour(const GADimension which,
		const unsigned int lower, const unsigned int upper)
{
  if(upper < lower){
    GAErr(className(), "resizeBehaviour", gaErrBadResizeBehaviour);
    return resizeBehaviour(which);
  }

  switch(which){
  case gaWidth:
    minX = lower; maxX = upper;
    if(nx > upper) resize(upper,ny);
    if(nx < lower) resize(lower,ny);
    break;

  case gaHeight:
    minY = lower; maxY = upper;
    if(ny > upper) resize(nx,upper);
    if(ny < lower) resize(nx,lower);
    break;

  default:
    break;
  }

  return resizeBehaviour(which);
}


void
GA2DBinaryStringGenome::copy(const GA2DBinaryStringGenome & orig,
				 const unsigned int r, const unsigned int s,
				 const unsigned int x, const unsigned int y,
				 const unsigned int width,
				 const unsigned int height)
{
  int w=width, h=height;
  if(w == 0 || x >= orig.nx || r >= nx ||
     h == 0 || y >= orig.ny || s >= ny) return;
  if(x + w > orig.nx) w = orig.nx - x;
  if(y + h > orig.ny) h = orig.ny - y;
  if(r + w > nx) w = nx - r;
  if(s + h > ny) h = ny - s;

  for(int j=0; j<h; j++)
    GABinaryStringGenome::copy(orig, (s+j)*nx+r, (y+j)*orig.nx+x, w);
}


void
GA2DBinaryStringGenome::set(const unsigned int x, const unsigned int y,
				const unsigned int width,
				const unsigned int height)
{
  int w=width, h=height;
  if(x + w > nx) w = nx - x;
  if(y + h > ny) h = ny - y;

  for(int j=0; j<h; j++)
    GABinaryStringGenome::set((y+j)*nx+x, w);
}


void
GA2DBinaryStringGenome::unset(const unsigned int x, const unsigned int y,
				  const unsigned int width, 
				  const unsigned int height)
{
  int w=width, h=height;
  if(x + w > nx) w = nx - x;
  if(y + h > ny) h = ny - y;

  for(int j=0; j<h; j++)
    GABinaryStringGenome::unset((y+j)*nx+x, w);
}


int 
GA2DBinaryStringGenome::_equal(const GAGenome & c) const
{
  if(this == &c) return 1;
  GA2DBinaryStringGenome & b = (GA2DBinaryStringGenome &)c;
  if(nx != b.nx || ny != b.ny) return 0;
  int val=0;
  for(int j=0; j<ny && val==0; j++)
    val = GABinaryStringGenome::equal(b,j*nx,j*nx,nx) ? 0 : 1;
  return(val ? 0 : 1);
}


int 
GA2DBinaryStringGenome::_notequal(const GAGenome & c) const
{
  if(this == &c) return 0;
  GA2DBinaryStringGenome & b = (GA2DBinaryStringGenome &)c;
  if(nx != b.nx || ny != b.ny) return 1;
  int val=0;
  for(int j=0; j<ny && val==0; j++)
    val = GABinaryStringGenome::equal(b,j*nx,j*nx,nx) ? 0 : 1;
  return(val ? 1 : 0);
}
