// $Header: /usr/people/mbwall/src/ga/ga/RCS/binstr2.ch.h,v 1.3 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  binstr2.ch.h
  mbwall 19apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the interface for the 2D binary string genome.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_binstr2_ch_h_
#define _ga_binstr2_ch_h_

#include <ga/config.h>
#include <ga/binstr.h>
#include <ga/genome.h>


/* ----------------------------------------------------------------------------
2DBinaryStringGenome
-------------------------------------------------------------------------------
---------------------------------------------------------------------------- */
class GA2DBinaryStringGenome : public GABinaryStringGenome {
  GADefineIdentity("GA2DBinaryStringGenome", 52):

public:
  GA2DBinaryStringGenome(const unsigned int x, const unsigned int y,
			     GAObjectiveFunction f=(GAObjectiveFunction)0,
			     void * u=(void *)0,
			     GA * g=(GA *)0);
  GA2DBinaryStringGenome(const GA2DBinaryStringGenome & orig);
  GA2DBinaryStringGenome& operator=(const GA2DBinaryStringGenome& arg)
    {copy(arg); return *this;}
  GA2DBinaryStringGenome& operator=(const GAGenome& arg)
    {copy(arg); return *this;}
  GA2DBinaryStringGenome& operator=(const short array []){
    for(int i=0; i<nx; i++)
      for(int j=0; j<ny; j++)
	gene(i,j,*(array+j*nx+i));
    return *this;
  }
  virtual ~GA2DBinaryStringGenome(){}
  virtual GAGenome *clone(const GACloneMethod flag=gaCloneContents) const {
    GA2DBinaryStringGenome *cpy = new 
      GA2DBinaryStringGenome(nx,ny,of,ud,ga);
    if(flag == gaCloneContents){
      cpy->copy(*this);
    }
    else{
      cpy->GAGenome::copy(*this);
      cpy->minX = minX; cpy->minY = minY; 
      cpy->maxX = maxX; cpy->maxY = maxY; 
    }
    return cpy;
  }
  virtual void copy(const GAGenome & chrom);

  friend ostream & 
    operator<<(ostream & os, const GA2DBinaryStringGenome & arg)
      {arg._write(os); return os;}
  friend istream & 
    operator>>(istream & is, GA2DBinaryStringGenome & arg)
      {arg._read(is); return is;}
  friend int operator==(const GA2DBinaryStringGenome & a,
			const GA2DBinaryStringGenome & b)
    {return a._equal(b);}
  friend int operator!=(const GA2DBinaryStringGenome & a,
			const GA2DBinaryStringGenome & b)
    {return a._notequal(b);}

// specific to this class
  short gene(const unsigned int x, const unsigned int y) const {
    return GABinaryStringGenome::bit(x+nx*y);
  }
  short gene(const unsigned int x, const unsigned int y, const short value) {
    return((GABinaryStringGenome::bit(x+nx*y) == value) ? value :
	   GABinaryStringGenome::bit(x+nx*y, value));
  }
  int width() const {return nx;}
  int width(const int w){resize(w, ny); return nx;}
  int height() const {return ny;}
  int height(const int h){resize(nx, h); return ny;}
  int resize(const int x, const int y);
  int resizeBehaviour(const GADimension which) const {
    return((which == gaWidth) ? (maxX == minX ? gaNoResize : maxX) : 
	   ((which == gaHeight) ? (maxY == minY ? gaNoResize : maxY) : 0));
  }
  int resizeBehaviour(const GADimension which,
		      const unsigned int lowerX, const unsigned int upperX);
  int resizeBehaviour(const unsigned int lowerX, const unsigned int upperX, 
		      const unsigned int lowerY, const unsigned int upperY){
    return(resizeBehaviour(gaWidth, lowerX, upperX) * 
	   resizeBehaviour(gaHeight, lowerY, upperY));
  }
  void copy(const GA2DBinaryStringGenome &, 
	    const unsigned int, const unsigned int,
	    const unsigned int, const unsigned int,
	    const unsigned int, const unsigned int);
  void set(const unsigned int, const unsigned int,
	   const unsigned int, const unsigned int);
  void unset(const unsigned int, const unsigned int,
	     const unsigned int, const unsigned int);

protected:
  unsigned int nx, ny, minX, minY;
  int maxX, maxY;
  void _read(istream & is=cin);
  void _write (ostream & os=cout) const;
  int _equal(const GAGenome & c) const;
  int _notequal(const GAGenome & c) const;
};

#endif
