// $Header: /usr/people/mbwall/src/ga/ga/RCS/binstr2.op.C,v 1.2 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  binstr2.op.C
  mbwall 19apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Source file for the 2D binary string genome built-in operators.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <string.h>
#include <ga/config.h>
#include <ga/error.h>
#include <ga/random.h>
#include <ga/binstr2.ch.h>
#include <ga/binstr2.xs.h>
#include <ga/binstr2.op.h>


/* ----------------------------------------------------------------------------
2D Binary String Genome
  The order for looping through indices is height-then-width (ie height loops
before a single width increment)
---------------------------------------------------------------------------- */
#if USE_RANDOM_INITIALIZER == 1
void 
GA2DBinStrRandomInitializer(GAGenome & c)
{
  GA2DBinaryStringGenome &child=(GA2DBinaryStringGenome &)c;
  child.resize(gaResizeAny, gaResizeAny);
  for(int i=child.width()-1; i>=0; i--)
    for(int j=child.height()-1; j>=0; j--)
      child.gene(i, j, GARandomBit());
}
#endif


#if USE_UNSET_INITIALIZER == 1
void 
GA2DBinStrUnsetInitializer(GAGenome & c)
{
  GA2DBinaryStringGenome &child=(GA2DBinaryStringGenome &)c;
  child.resize(gaResizeAny, gaResizeAny);
  child.unset(0, 0, child.width(), child.height());
}
#endif


#if USE_SET_INITIALIZER == 1
void 
GA2DBinStrSetInitializer(GAGenome & c)
{
  GA2DBinaryStringGenome &child=(GA2DBinaryStringGenome &)c;
  child.resize(gaResizeAny, gaResizeAny);	
  child.set(0, 0, child.width(), child.height());
}
#endif


#if USE_FLIP_MUTATOR == 1
int 
GA2DBinStrFlipMutator(GAGenome & c, float pmut)
{
  GA2DBinaryStringGenome &child=(GA2DBinaryStringGenome &)c;
  register int n, m, i, j;
  if(pmut <= 0.0) return(0);

  float nMut = pmut * (float)(child.size());
  if(nMut < 1.0){		// we have to do a flip test on each bit
    nMut = 0;
    for(i=child.width()-1; i>=0; i--){
      for(j=child.height()-1; j>=0; j--){
	if(GAFlipCoin(pmut)){
	  child.gene(i, j, ((child.gene(i,j) == 0) ? 1 : 0));
	  nMut++;
	}
      }
    }
  }
  else{				// only flip the number of bits we need to flip
    for(n=0; n<nMut; n++){
      m = GARandomInt(0, child.size()-1);
      i = m % child.width();
      j = m / child.width();
      child.gene(i, j, ((child.gene(i,j) == 0) ? 1 : 0));
    }
  }
  return((int)nMut);
}
#endif



// Randomly take bits from each parent.  For each bit we flip a coin to see if
// that bit should come from the mother or the father.  This will not work with
// resizable genomes!  If the genome is not the same size as the parents then
// we post an error message.  The crossover method does not need a crossover
// site.
#if USE_UNIFORM_CROSSOVER == 1
void
GA2DBinStrUniformCrossover(GAGenome &c, const GAGenome &c1, const GAGenome &c2)
{
  GA2DBinaryStringGenome &child=(GA2DBinaryStringGenome &)c;
  GA2DBinaryStringGenome &mom=(GA2DBinaryStringGenome &)c1;
  GA2DBinaryStringGenome &dad=(GA2DBinaryStringGenome &)c2;

  if(child.resizeBehaviour(gaWidth) == gaNoResize ||
     child.resizeBehaviour(gaHeight) == gaNoResize){
    if(mom.width() != dad.width() || mom.height() != dad.height()){
      GAErr(c.className(), "GA2DBinStrUniformCrossover", 
	    gaErrParentSizeMismatch);
      return;
    }
    if(child.width() != mom.width() || child.height() != mom.height()){
      GAErr(c.className(), "GA2DBinStrUniformCrossover",
	    gaErrChildSizeMismatch);
      return;
    }
  }
  else{
    GAErr(c.className(), "GA2DBinStrUniformCrossover", gaErrFixedOnly);
    return;
  }
  
  for(int i=child.width()-1; i>=0; i--)
    for(int j=child.height()-1; j>=0; j--)
      child.gene(i,j,(GARandomBit() ? mom.gene(i,j) : dad.gene(i,j)));
}
#endif



// This is a padded implementation of the resizeable crossover for 2D Binary
// String Genomes.  When a resizeable child is created, its dimensions are
// determined by the Max of the parents' chunks and the extra space is padded
// with random bits.
//   If resize is restricted in one dimension or the other then we pick a 
// crossover site in that dimension that is within the resize limits.
//   If resize is not restricted, we pad so that the dimension becomes
// max(momsite,dadsite) + max(momremainder,dadremainder).  The extra space is
// filled with random bits.
#if USE_1PT_PAD_CROSSOVER == 1
void
GA2DBinStr1PtPadCrossover(GAGenome & c,
			  const GAGenome & c1, const GAGenome & c2)
{
  GA2DBinaryStringGenome &child=(GA2DBinaryStringGenome &)c;
  GA2DBinaryStringGenome &mom=(GA2DBinaryStringGenome &)c1;
  GA2DBinaryStringGenome &dad=(GA2DBinaryStringGenome &)c2;
  GA2DBinStr1PtCrossSite & xsite =
    (GA2DBinStr1PtCrossSite &)child.crossoverSite();

  unsigned int x, y, x1, y1, x2, y2;
  if(child.resizeBehaviour(gaWidth) == gaNoResize){
    if(mom.width() != dad.width()){
      GAErr(c.className(), "GA2DBinStr1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.width() != mom.width()){
      GAErr(c.className(), "GA2DBinStr1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.x == gaNoSite) xsite.x = GARandomInt(0,child.width());
    xsite.x1 = xsite.x2 = xsite.x;
    x1 = x2 = x = child.width() - xsite.x;
  }
  else{
    if(xsite.x1 == gaNoSite) xsite.x1 = GARandomInt(0, mom.width());
    if(xsite.x2 == gaNoSite) xsite.x2 = GARandomInt(0, dad.width());
    xsite.x = Max(xsite.x1,xsite.x2);
    x1 = mom.width() - xsite.x1;
    x2 = dad.width() - xsite.x2; 
    x = Max(x1,x2);
  }
  
  if(child.resizeBehaviour(gaHeight) == gaNoResize){
    if(mom.height() != dad.height()){
      GAErr(c.className(), "GA2DBinStr1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.height() != mom.height()){
      GAErr(c.className(), "GA2DBinStr1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.y == gaNoSite) xsite.y = GARandomInt(0,child.height());
    xsite.y1 = xsite.y2 = xsite.y;
    y1 = y2 = y = child.height() - xsite.y;
  }
  else{
    if(xsite.y1 == gaNoSite) xsite.y1 = GARandomInt(0, mom.height());
    if(xsite.y2 == gaNoSite) xsite.y2 = GARandomInt(0, dad.height());
    xsite.y = Max(xsite.y1,xsite.y2);
    y1 = mom.height() - xsite.y1;
    y2 = dad.height() - xsite.y2; 
    y = Max(y1,y2);
  }
  child.resize(xsite.x+x, xsite.y+y);

  child.copy(mom, 
	     xsite.x-xsite.x1, xsite.y-xsite.y1,
	     0, 0,
	     xsite.x1, xsite.y1);
  child.copy(mom,
	     xsite.x, xsite.y,
	     xsite.x1, xsite.y1,
	     x1, y1);
  child.copy(dad,
	     xsite.x-xsite.x2, xsite.y,
	     0, xsite.y2,
	     xsite.x2, y2);
  child.copy(dad, 
	     xsite.x, xsite.y-xsite.y2,
	     xsite.x2, 0,
	     x2, xsite.y2);

// Now set the padded areas to have random contents.  In the most general form
// there are 16 sub-areas, 12 of which might need to be set.  For a given 
// genome at most 6 of these needs to be set.  This set of for loops should
// cover them all.

// need to write this code...

//  for(int i=child.width()-1; i>=0; i--){
//    for(int j=child.height()-1; j>=0; j--)
//      child.gene(i, j, (random() & 01));
//  }
}
#endif



// This is a clipping implementation of the resizeable crossover for 2D Binary
// String Genomes.  When a resizeable child is created, its dimensions are
// determined by the Min of the parents' chunks.  The extra space is clipped.
#if USE_1PT_CLIP_CROSSOVER == 1
void
GA2DBinStr1PtClipCrossover(GAGenome & c,
			   const GAGenome & c1, const GAGenome & c2)
{
  GA2DBinaryStringGenome &child=(GA2DBinaryStringGenome &)c;
  GA2DBinaryStringGenome &mom=(GA2DBinaryStringGenome &)c1;
  GA2DBinaryStringGenome &dad=(GA2DBinaryStringGenome &)c2;
  GA2DBinStr1PtCrossSite & xsite =
    (GA2DBinStr1PtCrossSite &)child.crossoverSite();

  unsigned int x, y, x1, y1, x2, y2;
  if(child.resizeBehaviour(gaWidth) == gaNoResize){
    if(mom.width() != dad.width()){
      GAErr(c.className(), "GA2DBinStr1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.width() != mom.width()){
      GAErr(c.className(), "GA2DBinStr1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.x == gaNoSite) xsite.x = GARandomInt(0,child.width());
    xsite.x1 = xsite.x2 = xsite.x;
    x1 = x2 = x = child.width() - xsite.x;
  }
  else{
    if(xsite.x1 == gaNoSite) xsite.x1 = GARandomInt(0, mom.width());
    if(xsite.x2 == gaNoSite) xsite.x2 = GARandomInt(0, dad.width());
    xsite.x = Min(xsite.x1,xsite.x2);
    x1 = mom.width() - xsite.x1;
    x2 = dad.width() - xsite.x2; 
    x = Min(x1,x2);
  }
  
  if(child.resizeBehaviour(gaHeight) == gaNoResize){
    if(mom.height() != dad.height()){
      GAErr(c.className(), "GA2DBinStr1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.height() != mom.height()){
      GAErr(c.className(), "GA2DBinStr1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.y == gaNoSite) xsite.y = GARandomInt(0,child.height());
    xsite.y1 = xsite.y2 = xsite.y;
    y1 = y2 = y = child.height() - xsite.y;
  }
  else{
    if(xsite.y1 == gaNoSite) xsite.y1 = GARandomInt(0, mom.height());
    if(xsite.y2 == gaNoSite) xsite.y2 = GARandomInt(0, dad.height());
    xsite.y = Min(xsite.y1,xsite.y2);
    y1 = mom.height() - xsite.y1;
    y2 = dad.height() - xsite.y2; 
    y = Min(y1,y2);
  }
  child.resize(xsite.x+x, xsite.y+y);

  child.copy(mom, 
	     0, 0,
	     xsite.x1-xsite.x, xsite.y1-xsite.y,
	     xsite.x, xsite.y);
  child.copy(mom,
	     xsite.x, xsite.y,
	     xsite.x1, xsite.y1,
	     x, y);
  child.copy(dad,
	     0, xsite.y,
	     0, xsite.y2,
	     xsite.x, y);
  child.copy(dad, 
	     xsite.x, 0,
	     xsite.x2, 0,
	     x, xsite.y);
}
#endif


#if USE_EVEN_CROSSOVER == 1
void
GA2DBinStrEvenCrossover(GAGenome & c,
			const GAGenome & c1, const GAGenome & c2)
{
  GA2DBinaryStringGenome &child=(GA2DBinaryStringGenome &)c;
  GA2DBinaryStringGenome &mom=(GA2DBinaryStringGenome &)c1;
  GA2DBinaryStringGenome &dad=(GA2DBinaryStringGenome &)c2;

  if(child.size() > mom.size() || child.size() > dad.size()){
    GAErr(c.className(), "GA2DBinStrEvenCrossover", gaErrChildSizeMismatch);
    return;
  }
  for(int i=0; i<child.width(); i++)
    for(int j=0; j<child.height(); j++)
      child.gene(i, j, (((i*child.height()+j)%2 == 0) ?
			mom.gene(i,j) : dad.gene(i,j)));
}
#endif


#if USE_ODD_CROSSOVER == 1
void
GA2DBinStrOddCrossover(GAGenome & c,
		       const GAGenome & c1, const GAGenome & c2)
{
  GA2DBinaryStringGenome &child=(GA2DBinaryStringGenome &)c;
  GA2DBinaryStringGenome &mom=(GA2DBinaryStringGenome &)c1;
  GA2DBinaryStringGenome &dad=(GA2DBinaryStringGenome &)c2;

  if(child.size() > mom.size() || child.size() > dad.size()){
    GAErr(c.className(), "GA2DBinStrOddCrossover", gaErrChildSizeMismatch);
    return;
  }
  for(int i=0; i<child.width(); i++)
    for(int j=0; j<child.height(); j++)
      child.gene(i, j, (((i*child.height()+j)%2 == 0) ?
			dad.gene(i,j) : mom.gene(i,j)));
}
#endif
