// $Header: /usr/people/mbwall/src/ga/ga/RCS/binstr3.ch.C,v 1.2 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  binstr3.ch.C
  mbwall 19apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Source file for the 3D binary string genome.  See the 1D genome for comments.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <ctype.h>
#include <ga/config.h>
#include <ga/error.h>
#include <ga/random.h>
#include <ga/binstr3.ch.h>
#include <ga/binstr3.xs.h>
#include <ga/binstr3.op.h>


/* ----------------------------------------------------------------------------
3DBinaryStringGenome
---------------------------------------------------------------------------- */
static DEFAULT_3DBINSTR_CROSSSITE ga3DBinStrDefCrossSite;

GA3DBinaryStringGenome::
GA3DBinaryStringGenome(const unsigned int width,
		       const unsigned int height,
		       const unsigned int depth,
		       GAObjectiveFunction f,
		       void * u,
		       GA * g) :
GABinaryStringGenome(DEFAULT_3DBINSTR_INITIALIZER,
		     DEFAULT_3DBINSTR_MUTATOR,
		     DEFAULT_3DBINSTR_CROSSOVER,
		     &ga3DBinStrDefCrossSite)
{
  of=f; ud=u; ga=g;
  nx=minX=maxX=0; ny=minY=maxY=0; nz=minZ=maxZ=0;
  resize(width, height, depth);
}


GA3DBinaryStringGenome::
GA3DBinaryStringGenome(const GA3DBinaryStringGenome & orig)
{
  nx=minX=maxX=0; ny=minY=maxY=0; nz=minZ=maxZ=0;
  copy(orig);
}


void
GA3DBinaryStringGenome::copy(const GAGenome & orig)
{
  if(&orig == this) return;
  if(!sameClass(orig)){
    GAErr(className(), "copy", gaErrObjectTypeMismatch);
    return;
  }
  GABinaryStringGenome::copy(orig);
  GA3DBinaryStringGenome & c = (GA3DBinaryStringGenome &)orig;
  nx = c.nx; ny = c.ny; nz = c.nz;
  minX = c.minX; minY = c.minY; minZ = c.minZ;
  maxX = c.maxX; maxY = c.maxY; maxZ = c.maxZ;
}


int
GA3DBinaryStringGenome::resize(const int w, const int h, const int d)
{
  int width=w, height=h, depth=d;
  if(width == nx && height == ny && depth == nz) return sz;

  if(minX == maxX){		// resizing is not allowed for this one
    if(width < 0) return nx;	// do nothing
    minX = maxX = width;	// resize to specified width
  }
  else if(width >=0){
    if(width < minX) minX = width;
    if(width > maxX) maxX = width;
  }
  else width = GARandomInt(minX, maxX);

  if(minY == maxY){		// resizing is not allowed for this one
    if(height < 0) return ny;	// do nothing
    minY = maxY = height;	// resize to specified height
  }
  else if(height >=0){
    if(height < minY) minY = height;
    if(height > maxY) maxY = height;
  }
  else height = GARandomInt(minY, maxY);

  if(minZ == maxZ){		// resizing is not allowed for this one
    if(depth < 0) return nz;	// do nothing
    minZ = maxZ = depth;	// resize to specified depth
  }
  else if(depth >=0){
    if(depth < minZ) minZ = depth;
    if(depth > maxZ) maxZ = depth;
  }
  else depth = GARandomInt(minZ, maxZ);

  if(width < nx && height < ny){
    int z=Min(nz,depth);
    for(int k=0; k<z; k++)
      for(int j=0; j<height; j++)
	move(k*height*width+j*width,k*ny*nx+j*nx,width);
  }
  else if(width < nx){
    int z=Min(nz,depth);
    for(int k=0; k<z; k++)
      for(int j=0; j<ny; j++)
	move(k*ny*width+j*width,k*ny*nx+j*nx,width);
  }
  else if(height < ny){
    int z=Min(nz,depth);
    for(int k=0; k<z; k++)
      for(int j=0; j<height; j++)
	move(k*height*nx+j*nx,k*ny*nx+j*nx,nx);
  }

  GABinaryStringGenome::resize(width*height*depth);

  if(width > nx && height > ny){ // adjust the existing chunks of bits
    int z=Min(nz,depth);
    for(int k=z-1; k>=0; k--){
      int j;
      for(j=ny-1; j>=0; j--){
	move(k*height*width+j*width,k*ny*nx+j*nx,nx);
	for(int i=nx; i<width; i++)
	  bit(k*height*width+j*width+i, GARandomBit());
      }
      for(j=ny; j<height; j++)
	for(int i=0; i<width; i++)
	  bit(k*height*width+j*width+i, GARandomBit());
    }
  }
  else if(width > nx){
    int z=Min(nz,depth);
    for(int k=z-1; k>=0; k--){
      for(int j=height-1; j>=0; j--){
	move(k*height*width+j*width,k*height*nx+j*nx,nx);
	for(int i=nx; i<width; i++)
	  bit(k*height*width+j*width+i, GARandomBit());
      }
    }
  }
  else if(height > ny){
    int z=Min(nz,depth);
    for(int k=z-1; k>=0; k--){
      int j;
      for(j=ny-1; j>=0; j--)
	move(k*height*width+j*width,k*ny*width+j*width,width);
      for(j=ny; j<height; j++)
	for(int i=0; i<width; i++)
	  bit(k*height*width+j*width+i, GARandomBit());
    }
  }
  if(depth > nz){		// change in depth is always new bits
    for(int i=width*height*nz; i<width*height*depth; i++)
      bit(i, GARandomBit());
  }

  nx = width; ny = height; nz = depth;
  return sz;
}


void
GA3DBinaryStringGenome::_read(istream & is)
{
  static char c;
  int i=0, j=0, k=0;
  do{
    is >> c;
    if(isdigit(c)){
      gene(i++, j, k, ((c == '0') ? 0 : 1));
      if(i >= nx){
	i=0;
	j++;
      }
      if(j >= ny){
	j=0;
	k++;
      }
    }
  } while(is && is.peek() != EOF && k < nz);

  if(is.peek() == EOF && 
     ((k < nz) ||		// didn't get some lines
      (j < ny && j != 0) ||	// didn't get some lines
      (i < nx && i != 0)))	// didn't get some lines
    GAErr(className(), "_read", gaErrUnexpectedEOF);
}


void
GA3DBinaryStringGenome::_write(ostream & os) const 
{
  for(int k=0; k<nz; k++){
    for(int j=0; j<ny; j++){
      for(int i=0; i<nx; i++){
	os << gene(i,j,k);
      }
      os << "\n";
    }
    os << "\n";
  }
}


int
GA3DBinaryStringGenome::
resizeBehaviour(const GADimension which,
		const unsigned int lower, const unsigned int upper)
{
  if(upper < lower){
    GAErr(className(), "resizeBehaviour", gaErrBadResizeBehaviour);
    return resizeBehaviour(which);
  }

  switch(which){
  case gaWidth:
    minX = lower; maxX = upper;
    if(nx > upper) resize(upper,ny,nz);
    if(nx < lower) resize(lower,ny,nz);
    break;

  case gaHeight:
    minY = lower; maxY = upper;
    if(ny > upper) resize(nx,upper,nz);
    if(ny < lower) resize(nx,lower,nz);
    break;

  case gaDepth:
    minZ = lower; maxZ = upper;
    if(nz > upper) resize(nx,ny,upper);
    if(nz < lower) resize(nx,ny,lower);
    break;

  default:
    break;
  }

  return resizeBehaviour(which);
}


void
GA3DBinaryStringGenome::
copy(const GA3DBinaryStringGenome & orig,
     const unsigned int r, const unsigned int s, const unsigned int t,
     const unsigned int x, const unsigned int y, const unsigned int z,
     const unsigned int width, const unsigned int height,
     const unsigned int depth)
{
  int w=width, h=height, d=depth;
  if(w == 0 || x >= orig.nx || r >= nx ||
     h == 0 || y >= orig.ny || s >= ny ||
     d == 0 || z >= orig.nz || t >= nz) return;
  if(x + w > orig.nx) w = orig.nx - x;
  if(y + h > orig.ny) h = orig.ny - y;
  if(z + d > orig.nz) d = orig.nz - z;
  if(r + w > nx) w = nx - r;
  if(s + h > ny) h = ny - s;
  if(t + d > nz) d = nz - t;

  for(int k=0; k<d; k++)
    for(int j=0; j<h; j++)
      GABinaryStringGenome::copy(orig,
				 (t+k)*ny*nx + (s+j)*nx + r,
				 (z+k)*orig.ny*orig.nx + (y+j)*orig.nx + x, w);
}


void
GA3DBinaryStringGenome::
set(const unsigned int x, const unsigned int y, const unsigned int z,
    const unsigned int width, const unsigned int height,
    const unsigned int depth)
{
  int w=width, h=height, d=depth;
  if(x + w > nx) w = nx - x;
  if(y + h > ny) h = ny - y;
  if(z + d > nz) d = nz - z;

  for(int k=0; k<d; k++)
    for(int j=0; j<h; j++)
      GABinaryStringGenome::set((z+k)*ny*nx + (y+j)*nx + x, w);
}


void
GA3DBinaryStringGenome::
unset(const unsigned int x, const unsigned int y, const unsigned int z,
      const unsigned int width, const unsigned int height,
      const unsigned int depth)
{
  int w=width, h=height, d=depth;
  if(x + w > nx) w = nx - x;
  if(y + h > ny) h = ny - y;
  if(z + d > nz) d = nz - z;

  for(int k=0; k<d; k++)
    for(int j=0; j<h; j++)
      GABinaryStringGenome::unset((z+k)*ny*nx + (y+j)*nx + x, w);
}


int 
GA3DBinaryStringGenome::_equal(const GAGenome & c) const
{
  if(this == &c) return 1;
  GA3DBinaryStringGenome & b = (GA3DBinaryStringGenome &)c;
  if(nx != b.nx || ny != b.ny || nz != b.nz) return 0;
  int val=0;
  for(int k=0; k<nz && val==0; k++)
    for(int j=0; j<ny && val==0; j++)
      val = GABinaryStringGenome::equal(b,k*ny*nx,k*ny*nx,nx) ? 0 : 1;
  return(val ? 0 : 1);
}


int 
GA3DBinaryStringGenome::_notequal(const GAGenome & c) const
{
  if(this == &c) return 0;
  GA3DBinaryStringGenome & b = (GA3DBinaryStringGenome &)c;
  if(nx != b.nx || ny != b.ny || nz != b.nz) return 1;
  int val=0;
  for(int k=0; k<nz && val==0; k++)
    for(int j=0; j<ny && val==0; j++)
      val = GABinaryStringGenome::equal(b,k*ny*nx,k*ny*nx,nx) ? 0 : 1;
  return(val ? 1 : 0);
}
