// $Header: /usr/people/mbwall/src/ga/ga/RCS/binstr3.ch.h,v 1.3 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  binstr3.ch.h
  mbwall 19apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the interface for the 3D binary string genome.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_binstr3_ch_h_
#define _ga_binstr3_ch_h_

#include <ga/config.h>
#include <ga/binstr.h>
#include <ga/genome.h>


/* ----------------------------------------------------------------------------
3DBinaryStringGenome
-------------------------------------------------------------------------------
---------------------------------------------------------------------------- */
class GA3DBinaryStringGenome : public GABinaryStringGenome {
  GADefineIdentity("GA3DBinaryStringGenome", 53):

public:
  GA3DBinaryStringGenome(const unsigned int x,
			     const unsigned int y,
			     const unsigned int z,
			     GAObjectiveFunction f=(GAObjectiveFunction)0,
			     void * u=(void *)0,
			     GA * g=(GA *)0);
  GA3DBinaryStringGenome(const GA3DBinaryStringGenome & orig);
  GA3DBinaryStringGenome& operator=(const GA3DBinaryStringGenome& arg)
    {copy(arg); return *this;}
  GA3DBinaryStringGenome& operator=(const GAGenome& arg)
    {copy(arg); return *this;}
  GA3DBinaryStringGenome& operator=(const short array []){
    for(int i=0; i<nx; i++)
      for(int j=0; j<ny; j++)
	for(int k=0; k<nz; k++)
	  gene(i,j,k,*(array+k*ny*nx+j*nx+i));
    return *this;
  }
  virtual ~GA3DBinaryStringGenome(){}
  virtual GAGenome *clone(const GACloneMethod flag=gaCloneContents) const {
    GA3DBinaryStringGenome *cpy = new 
      GA3DBinaryStringGenome(nx,ny,nz,of,ud,ga);
    if(flag == gaCloneContents){
      cpy->copy(*this);
    }
    else{
      cpy->GAGenome::copy(*this);
      cpy->minX = minX; cpy->minY = minY; cpy->minZ = minZ; 
      cpy->maxX = maxX; cpy->maxY = maxY; cpy->maxZ = maxZ;
    }
    return cpy;
  }
  virtual void copy(const GAGenome & chrom);

  friend ostream &
    operator<<(ostream & os, const GA3DBinaryStringGenome & arg)
      {arg._write(os); return os;}
  friend istream & 
    operator>>(istream & is, GA3DBinaryStringGenome & arg)
      {arg._read(is); return is;}
  friend int operator==(const GA3DBinaryStringGenome & a,
			const GA3DBinaryStringGenome & b)
    {return a._equal(b);}
  friend int operator!=(const GA3DBinaryStringGenome & a,
			const GA3DBinaryStringGenome & b)
    {return a._notequal(b);}

// specific to this class
  short gene(const unsigned int x,
	     const unsigned int y,
	     const unsigned int z) const {
    return GABinaryStringGenome::bit(z*ny*nx + y*nx + x);
  }
  short gene(const unsigned int x,
	     const unsigned int y,
	     const unsigned int z, const short value) {
    return((GABinaryStringGenome::bit(z*ny*nx+y*nx+x) == value) ? value :
	   GABinaryStringGenome::bit(z*ny*nx+y*nx+x, value));
  }
  int width() const {return nx;}
  int width(const int w){resize(w, ny, nz); return nx;}
  int height() const {return ny;}
  int height(const int h){resize(nx, h, nz); return ny;}
  int depth() const {return nz;}
  int depth(const int d){resize(nx, ny, d); return nz;}
  int resize(const int x, const int y, const int z);
  int resizeBehaviour(const GADimension which) const {
    return((which == gaWidth) ? (maxX == minX ? gaNoResize : maxX) : 
	   ((which == gaHeight) ? (maxY == minY ? gaNoResize : maxY) : 
	    ((which == gaDepth) ? (maxZ == minZ ? gaNoResize : maxZ) : 0)));
  }
  int resizeBehaviour(const GADimension which, 
		      const unsigned int low, const unsigned int upper);
  int resizeBehaviour(const unsigned int lowerX, const unsigned int upperX, 
		      const unsigned int lowerY, const unsigned int upperY, 
		      const unsigned int lowerZ, const unsigned int upperZ){
    return(resizeBehaviour(gaWidth,  lowerX, upperX) * 
	   resizeBehaviour(gaHeight, lowerY, upperY) * 
	   resizeBehaviour(gaDepth,  lowerZ, upperZ));
  }
  void copy(const GA3DBinaryStringGenome &, 
	    const unsigned int, const unsigned int, const unsigned int,
	    const unsigned int, const unsigned int, const unsigned int,
	    const unsigned int, const unsigned int, const unsigned int);
  void set(const unsigned int, const unsigned int,
	   const unsigned int, const unsigned int,
	   const unsigned int, const unsigned int);
  void unset(const unsigned int, const unsigned int,
	     const unsigned int, const unsigned int,
	     const unsigned int, const unsigned int);

protected:
  unsigned int nx, ny, nz, minX, minY, minZ;
  int maxX, maxY, maxZ;
  void _read(istream & is=cin);
  void _write (ostream & os=cout) const;
  int _equal(const GAGenome & c) const;
  int _notequal(const GAGenome & c) const;
};

#endif

