// $Header: /usr/people/mbwall/src/ga/ga/RCS/binstr3.op.C,v 1.2 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  binstr3.op.C
  mbwall 19apr95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Source file for the 3D binary string genome built-in operators.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <string.h>
#include <ga/config.h>
#include <ga/error.h>
#include <ga/random.h>
#include <ga/binstr3.ch.h>
#include <ga/binstr3.xs.h>
#include <ga/binstr3.op.h>


/* ----------------------------------------------------------------------------
3D Binary String Genome
  The order for looping through indices is depth-then-height-then-width
(ie depth loops before a single height increment, height loops before a single
width increment)
---------------------------------------------------------------------------- */
#if USE_RANDOM_INITIALIZER == 1
void 
GA3DBinStrRandomInitializer(GAGenome & c)
{
  GA3DBinaryStringGenome &child=(GA3DBinaryStringGenome &)c;
  child.resize(gaResizeAny, gaResizeAny, gaResizeAny);
  for(int i=child.width()-1; i>=0; i--)
    for(int j=child.height()-1; j>=0; j--)
      for(int k=child.depth()-1; k>=0; k--)
	child.gene(i,j,k, GARandomBit());
}
#endif


#if USE_UNSET_INITIALIZER == 1
void 
GA3DBinStrUnsetInitializer(GAGenome & c)
{
  GA3DBinaryStringGenome &child=(GA3DBinaryStringGenome &)c;
  child.resize(gaResizeAny, gaResizeAny, gaResizeAny);
  child.unset(0, 0, 0, child.width(), child.height(), child.depth());
}
#endif


#if USE_SET_INITIALIZER == 1
void 
GA3DBinStrSetInitializer(GAGenome & c)
{
  GA3DBinaryStringGenome &child=(GA3DBinaryStringGenome &)c;
  child.resize(gaResizeAny, gaResizeAny, gaResizeAny);	
  child.set(0, 0, 0, child.width(), child.height(), child.depth());
}
#endif


#if USE_FLIP_MUTATOR == 1
int 
GA3DBinStrFlipMutator(GAGenome & c, float pmut)
{
  GA3DBinaryStringGenome &child=(GA3DBinaryStringGenome &)c;
  register int n, m, i, j, k, d;
  if(pmut <= 0.0) return(0);

  float nMut = pmut * (float)(child.size());
  if(nMut < 1.0){		// we have to do a flip test on each bit
    nMut = 0;
    for(i=child.width()-1; i>=0; i--){
      for(j=child.height()-1; j>=0; j--){
	for(k=child.depth()-1; k>=0; k--){
	  if(GAFlipCoin(pmut)){
	    child.gene(i, j, k, ((child.gene(i,j,k) == 0) ? 1 : 0));
	    nMut++;
	  }
	}
      }
    }
  }
  else{				// only flip the number of bits we need to flip
    for(n=0; n<nMut; n++){
      m = GARandomInt(0, child.size()-1);
      d = child.height() * child.depth();
      i = m / d;
      j = (m % d) / child.depth();
      k = (m % d) % child.depth();
      child.gene(i, j, k, ((child.gene(i,j,k) == 0) ? 1 : 0));
    }
  }
  return((int)nMut);
}
#endif



// Randomly take bits from each parent.  For each bit we flip a coin to see if
// that bit should come from the mother or the father.  This will not work with
// resizable genomes!  If the genome is not the same size as the parents then
// we post an error message.  The crossover method does not need a crossover
// site.
#if USE_UNIFORM_CROSSOVER == 1
void
GA3DBinStrUniformCrossover(GAGenome &c, const GAGenome &c1, const GAGenome &c2)
{
  GA3DBinaryStringGenome &child=(GA3DBinaryStringGenome &)c;
  GA3DBinaryStringGenome &mom=(GA3DBinaryStringGenome &)c1;
  GA3DBinaryStringGenome &dad=(GA3DBinaryStringGenome &)c2;

  if(child.resizeBehaviour(gaWidth) == gaNoResize ||
     child.resizeBehaviour(gaHeight) == gaNoResize ||
     child.resizeBehaviour(gaDepth) == gaNoResize){
    if(mom.width() != dad.width() ||
       mom.height() != dad.height() ||
       mom.depth() != dad.depth()){
      GAErr(c.className(), "GA3DBinStrUniformCrossover", 
	    gaErrParentSizeMismatch);
      return;
    }
    if(child.width() != mom.width() ||
       child.height() != mom.height() || 
       child.depth() != mom.depth()){
      GAErr(c.className(), "GA3DBinStrUniformCrossover",
	    gaErrChildSizeMismatch);
      return;
    }
  }
  else{
    GAErr(c.className(), "GA3DBinStrUniformCrossover", gaErrFixedOnly);
    return;
  }
  
  for(int i=child.width()-1; i>=0; i--)
    for(int j=child.height()-1; j>=0; j--)
      for(int k=child.depth()-1; k>=0; k--)
      child.gene(i,j,k,(GARandomBit() ? mom.gene(i,j,k) : dad.gene(i,j,k)));
}
#endif




#if USE_1PT_PAD_CROSSOVER == 1
void
GA3DBinStr1PtPadCrossover(GAGenome & c,
			  const GAGenome & c1, const GAGenome & c2)
{
  GA3DBinaryStringGenome &child=(GA3DBinaryStringGenome &)c;
  GA3DBinaryStringGenome &mom=(GA3DBinaryStringGenome &)c1;
  GA3DBinaryStringGenome &dad=(GA3DBinaryStringGenome &)c2;
  GA3DBinStr1PtCrossSite & xsite =
    (GA3DBinStr1PtCrossSite &)child.crossoverSite();

  unsigned int x, y, z, x1, y1, z1, x2, y2, z2;
  if(child.resizeBehaviour(gaWidth) == gaNoResize){
    if(mom.width() != dad.width()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.width() != mom.width()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.x == gaNoSite) xsite.x = GARandomInt(0,child.width());
    xsite.x1 = xsite.x2 = xsite.x;
    x1 = x2 = x = child.width() - xsite.x;
  }
  else{
    if(xsite.x1 == gaNoSite) xsite.x1 = GARandomInt(0, mom.width());
    if(xsite.x2 == gaNoSite) xsite.x2 = GARandomInt(0, dad.width());
    xsite.x = Max(xsite.x1,xsite.x2);
    x1 = mom.width() - xsite.x1;
    x2 = dad.width() - xsite.x2; 
    x = Max(x1,x2);
  }
  
  if(child.resizeBehaviour(gaHeight) == gaNoResize){
    if(mom.height() != dad.height()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.height() != mom.height()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.y == gaNoSite) xsite.y = GARandomInt(0,child.height());
    xsite.y1 = xsite.y2 = xsite.y;
    y1 = y2 = y = child.height() - xsite.y;
  }
  else{
    if(xsite.y1 == gaNoSite) xsite.y1 = GARandomInt(0, mom.height());
    if(xsite.y2 == gaNoSite) xsite.y2 = GARandomInt(0, dad.height());
    xsite.y = Max(xsite.y1,xsite.y2);
    y1 = mom.height() - xsite.y1;
    y2 = dad.height() - xsite.y2; 
    y = Max(y1,y2);
  }
  
  if(child.resizeBehaviour(gaDepth) == gaNoResize){
    if(mom.depth() != dad.depth()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.depth() != mom.depth()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.z == gaNoSite) xsite.z = GARandomInt(0,child.depth());
    xsite.z1 = xsite.z2 = xsite.z;
    z1 = z2 = z = child.depth() - xsite.z;
  }
  else{
    if(xsite.z1 == gaNoSite) xsite.z1 = GARandomInt(0, mom.depth());
    if(xsite.z2 == gaNoSite) xsite.z2 = GARandomInt(0, dad.depth());
    xsite.z = Max(xsite.z1,xsite.z2);
    z1 = mom.depth() - xsite.z1;
    z2 = dad.depth() - xsite.z2; 
    z = Max(z1,z2);
  }
  child.resize(xsite.x+x, xsite.y+y, xsite.z+z);

  child.copy(mom,
	     xsite.x-xsite.x1, xsite.y-xsite.y1, xsite.z-xsite.z1,
	     0, 0, 0,
	     xsite.x1, xsite.y1, xsite.z1);
  child.copy(mom,
	     xsite.x, xsite.y, xsite.z-xsite.z1,
	     xsite.x1, xsite.y1, 0,
	     x1, y1, xsite.z1);
  child.copy(mom,
	     xsite.x-xsite.x1, xsite.y, xsite.z,
	     0, xsite.y1, xsite.z1,
	     xsite.x1, y1, z1);
  child.copy(mom,
	     xsite.x, xsite.y-xsite.y1, xsite.z,
	     xsite.x1, 0, xsite.z1,
	     x1, xsite.y1, z1);
  child.copy(dad,
	     xsite.x-xsite.x2, xsite.y, xsite.z-xsite.z2,
	     0, xsite.y2, 0,
	     xsite.x2, y2, xsite.z2);
  child.copy(dad,
	     xsite.x, xsite.y-xsite.y2, xsite.z-xsite.z2,
	     xsite.x2, 0, 0,
	     x2, xsite.y2, xsite.z2);
  child.copy(dad,
	     xsite.x-xsite.x2, xsite.y-xsite.y2, xsite.z,
	     0, 0, xsite.z2,
	     xsite.x2, xsite.y2, z2);
  child.copy(dad,
	     xsite.x, xsite.y, xsite.z,
	     xsite.x2, xsite.y2, xsite.z2,
	     x2, y2, z2);

// *** now need to set padded quandrants to random values
}
#endif



#if USE_1PT_CLIP_CROSSOVER == 1
void
GA3DBinStr1PtClipCrossover(GAGenome & c,
			   const GAGenome & c1, const GAGenome & c2)
{
  GA3DBinaryStringGenome &child=(GA3DBinaryStringGenome &)c;
  GA3DBinaryStringGenome &mom=(GA3DBinaryStringGenome &)c1;
  GA3DBinaryStringGenome &dad=(GA3DBinaryStringGenome &)c2;
  GA3DBinStr1PtCrossSite & xsite =
    (GA3DBinStr1PtCrossSite &)child.crossoverSite();

  unsigned int x, y, z, x1, y1, z1, x2, y2, z2;
  if(child.resizeBehaviour(gaWidth) == gaNoResize){
    if(mom.width() != dad.width()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.width() != mom.width()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.x == gaNoSite) xsite.x = GARandomInt(0,child.width());
    xsite.x1 = xsite.x2 = xsite.x;
    x1 = x2 = x = child.width() - xsite.x;
  }
  else{
    if(xsite.x1 == gaNoSite) xsite.x1 = GARandomInt(0, mom.width());
    if(xsite.x2 == gaNoSite) xsite.x2 = GARandomInt(0, dad.width());
    xsite.x = Min(xsite.x1,xsite.x2);
    x1 = mom.width() - xsite.x1;
    x2 = dad.width() - xsite.x2; 
    x = Min(x1,x2);
  }
  
  if(child.resizeBehaviour(gaHeight) == gaNoResize){
    if(mom.height() != dad.height()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.height() != mom.height()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.y == gaNoSite) xsite.y = GARandomInt(0,child.height());
    xsite.y1 = xsite.y2 = xsite.y;
    y1 = y2 = y = child.height() - xsite.y;
  }
  else{
    if(xsite.y1 == gaNoSite) xsite.y1 = GARandomInt(0, mom.height());
    if(xsite.y2 == gaNoSite) xsite.y2 = GARandomInt(0, dad.height());
    xsite.y = Min(xsite.y1,xsite.y2);
    y1 = mom.height() - xsite.y1;
    y2 = dad.height() - xsite.y2; 
    y = Min(y1,y2);
  }
  
  if(child.resizeBehaviour(gaDepth) == gaNoResize){
    if(mom.depth() != dad.depth()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrParentSizeMismatch);
      return;
    }
    if(child.depth() != mom.depth()){
      GAErr(c.className(), "GA3DBinStr1PtCrossover", gaErrChildSizeMismatch);
      return;
    }
    if(xsite.z == gaNoSite) xsite.z = GARandomInt(0,child.depth());
    xsite.z1 = xsite.z2 = xsite.z;
    z1 = z2 = z = child.depth() - xsite.z;
  }
  else{
    if(xsite.z1 == gaNoSite) xsite.z1 = GARandomInt(0, mom.depth());
    if(xsite.z2 == gaNoSite) xsite.z2 = GARandomInt(0, dad.depth());
    xsite.z = Min(xsite.z1,xsite.z2);
    z1 = mom.depth() - xsite.z1;
    z2 = dad.depth() - xsite.z2; 
    z = Min(z1,z2);
  }
  child.resize(xsite.x+x, xsite.y+y, xsite.z+z);

  child.copy(mom,
	     0, 0, 0,
	     xsite.x1-xsite.x, xsite.y1-xsite.y, xsite.z1-xsite.z,
	     xsite.x, xsite.y, xsite.z);
  child.copy(mom,
	     xsite.x, xsite.y, 0,
	     xsite.x1, xsite.y1, xsite.z1-xsite.z,
	     x, y, xsite.z);
  child.copy(mom,
	     0, xsite.y, xsite.z,
	     xsite.x1-xsite.x, xsite.y1, xsite.z1,
	     xsite.x, y, z);
  child.copy(mom,
	     xsite.x, 0, xsite.z,
	     xsite.x1, xsite.y1-xsite.y, xsite.z1,
	     x, xsite.y, z);
  child.copy(dad,
	     0, xsite.y, 0,
	     xsite.x2-xsite.x, xsite.y2, xsite.z2-xsite.z,
	     xsite.x, y, xsite.z);
  child.copy(dad,
	     xsite.x, 0, 0,
	     xsite.x2, xsite.y2-xsite.y, xsite.z2-xsite.z,
	     x, xsite.y, xsite.z);
  child.copy(dad,
	     0, 0, xsite.z,
	     xsite.x2-xsite.x, xsite.y2-xsite.y, xsite.z2,
	     xsite.x, xsite.y, z);
  child.copy(dad,
	     xsite.x, xsite.y, xsite.z,
	     xsite.x2, xsite.y2, xsite.z2,
	     x, y, z);
}
#endif


#if USE_EVEN_CROSSOVER == 1
void
GA3DBinStrEvenCrossover(GAGenome & c,
			const GAGenome & c1, const GAGenome & c2)
{
  GA3DBinaryStringGenome &child=(GA3DBinaryStringGenome &)c;
  GA3DBinaryStringGenome &mom=(GA3DBinaryStringGenome &)c1;
  GA3DBinaryStringGenome &dad=(GA3DBinaryStringGenome &)c2;

  if(child.size() > mom.size() || child.size() > dad.size()){
    GAErr(c.className(), "GA3DBinStrEvenCrossover", gaErrChildSizeMismatch);
    return;
  }
  for(int i=0; i<child.width(); i++)
    for(int j=0; j<child.height(); j++)
      for(int k=0; j<child.depth(); k++)
	child.gene(i, j, k, 
		   (((i*child.height()*child.depth()+j*child.depth()+k)%2==0) ?
		    mom.gene(i,j,k) : dad.gene(i,j,k)));
}
#endif


#if USE_ODD_CROSSOVER == 1
void
GA3DBinStrOddCrossover(GAGenome & c,
		       const GAGenome & c1, const GAGenome & c2)
{
  GA3DBinaryStringGenome &child=(GA3DBinaryStringGenome &)c;
  GA3DBinaryStringGenome &mom=(GA3DBinaryStringGenome &)c1;
  GA3DBinaryStringGenome &dad=(GA3DBinaryStringGenome &)c2;

  if(child.size() > mom.size() || child.size() > dad.size()){
    GAErr(c.className(), "GA3DBinStrOddCrossover", gaErrChildSizeMismatch);
    return;
  }
  for(int i=0; i<child.width(); i++)
    for(int j=0; j<child.height(); j++)
      for(int k=0; j<child.depth(); k++)
	child.gene(i, j, k, 
		   (((i*child.height()*child.depth()+j*child.depth()+k)%2==0) ?
		    dad.gene(i,j,k) : mom.gene(i,j,k)));
}
#endif
