// $Header: /usr/people/mbwall/src/ga/ga/RCS/config.h,v 1.3 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  config.h
  mbwall 27jun95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Configuration header file for the GA library.  Use this file to set the
parameters that are specific to machine/compiler/operating system.  This file
also contains macros that control which parts of the library will be compiled
and which operators will be the defaults.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_config_h_
#define _ga_config_h_


/* ----------------------------------------------------------------------------
PREPROCESSOR DIRECTIVES

  Here are the preprocessor directives that the library understands.  If you 
are using a makefile, then put these in the line with the DEFINES macro.  For
example, to define the NO_TEMPLATES and BORLAND_INST options then do

       DEFINES = -DNO_TEMPLATES -DBORLAND_INST

  In Mac or DOS, use your development environment's preprocessor directives 
option to set the values.
  Some of these are already set up for the OSes with which I am familiar.  See
below for the pre-defined sets.


   NO_PID             For machines that do not have the getpid() function or
                        the unistd.h header file.
   NO_TEMPLATES       For compilers that do not do templates.  The only type
                        of genome available when you use this is binary 
                        string.  list, tree, and array all use templates.
   BORLAND_INST       For compilers that use the Borland instantiation model.
                        These compilers expect all of the template code to be
		        in the header file.  The Cfront model, on the other
                        hand, expects source files with names similar to the
			header files, but all of the template code does not
			need to be in the header files.
   NO_AUTO_INST       For compilers that do not do automatic instantiation
                        (such as g++ version 2.6.8) you will have to force
                        instantiations.
   NO_CPLUSPLUS_MATH  For machines that have random and srandom defined
                        as C functions, not C++ functions.
   NO_RANDOM          For machines that do not have random and srandom.
                        When you define this the rand and srand functions
                        will be used instead of random and srandom.  If you are
			using a special version of random seed and/or random
			number generator then you will have to edit the
			random.h header file.
   BITS_IN_LONG       How many bits in a long integer?

---------------------------------------------------------------------------- */
// I'm assuming that most people will use a 32-bit OS.  If not, then define the
// BITS_IN_LONG to 16 or 64 or whatever your OS uses to represent a long int.
#ifndef BITS_IN_LONG
#define BITS_IN_LONG 32
#endif



// Here are the defines needed for some of the compilers/OSes on which I have
// been able to test the GA library.  You may have to remove and/or modify
// these to get things to work on your system.
#if defined(__MWERKS__)
#define NO_PID
#define NO_RANDOM
#endif

#if defined(__BORLANDC__)
#define NO_PID
#define NO_RANDOM
#define BORLAND_INST
#endif

// for g++ 2.6.8 (later versions may actually do auto instantiation)
#if defined(__GNUG__)
#define NO_AUTO_INST
#endif

#if defined(SOLARIS)
#define NO_CPLUSPLUS_MATH
#endif

#if defined(_IBMR2)
#define NO_CPLUSPLUS_MATH
#endif

#if (defined(ultrix) && defined(mips))
#define NO_CPLUSPLUS_MATH
#endif

#if defined(__hpux)
#define NO_CPLUSPLUS_MATH
#endif



/* ----------------------------------------------------------------------------
SPACE SAVERS and DEFAULT OPERATORS

  If space is limited, you may want to compile the library with only the parts
that you need (compiling in DOS comes to mind).  By default, the library 
includes everything.  If you want to trim it down, set to 0 the define(s) for 
the part(s) that you do not want to compile.  If you do not want a specific
genome type in the library, then don't include the sources for that genome.
  You can also specify which operators will be the default.  Be sure to make 
the defaults match the part that you have specified for inclusion.  For 
example, if you tell the compiler to USE_SIGMA_TRUNCATION and no other scaling
methods, be sure to set the DEFAULT_SCALING to sigma truncation.
---------------------------------------------------------------------------- */
// Initialization operators for all genomes
#define USE_RANDOM_INITIALIZER       1
#define USE_ORDERED_INITIALIZER      1
#define USE_SET_INITIALIZER          1
#define USE_UNSET_INITIALIZER        1

// Mutation operators for all genomes
#define USE_FLIP_MUTATOR             1
#define USE_SWAP_MUTATOR             1
#define USE_SWAP_SUBTREE_MUTATOR     1
#define USE_DESTRUCTIVE_MUTATOR      1

// Crossover operators for all genomes
#define USE_UNIFORM_CROSSOVER        1
#define USE_EVEN_CROSSOVER           1
#define USE_ODD_CROSSOVER            1
#define USE_1PT_CROSSOVER            1
#define USE_2PT_CROSSOVER            1
#define USE_1PT_PAD_CROSSOVER        1
#define USE_1PT_CLIP_CROSSOVER       1
#define USE_PARTIAL_MATCH_CROSSOVER  1
#define USE_ORDER_CROSSOVER          1
#define USE_CYCLE_CROSSOVER          1

// Scaling schemes
#define USE_LINEAR_SCALING           1
#define USE_SIGMA_TRUNC_SCALING      1
#define USE_POWER_LAW_SCALING        1
#define USE_SHARING                  1

// Selection schemes
#define USE_RANK_SELECTOR            1
#define USE_ROULETTE_SELECTOR        1
#define USE_TOURNAMENT_SELECTOR      1
#define USE_DS_SELECTOR              1
#define USE_SRS_SELECTOR             1
#define USE_SUS_SELECTOR             1

// genetic algorithms
#define USE_SIMPLE_GA                1
#define USE_STEADY_STATE_GA          1
#define USE_REPLACEMENT_GA           1


//   If you change the default operators at compile time, be sure that the
// default and the components you have selected match up.  Notice that you
// can change default operators of template classes by specializing the
// template creators (see the examples).  Also, be sure that the crossover
// site and crossover operator match (do not use an ordered cross site with
// single point crossover, for example).
//   You can set default initializer, crossover, and mutator only on non-
// template classes.  To set the defaults on template classes you must do a
// specialization of the template class creator (see creators.C for examples).
#define DEFAULT_SCALING              GALinearScaling
#define DEFAULT_SELECTOR             GARouletteWheelSelector
#define DEFAULT_TERMINATOR           GATerminateUponGeneration

#define DEFAULT_1DBINSTR_INITIALIZER GA1DBinStrRandomInitializer
#define DEFAULT_1DBINSTR_MUTATOR     GA1DBinStrFlipMutator
#define DEFAULT_1DBINSTR_CROSSOVER   GA1DBinStr1PtCrossover
#define DEFAULT_1DBINSTR_CROSSSITE   GA1DBinStr1PtCrossSite
#define DEFAULT_2DBINSTR_INITIALIZER GA2DBinStrRandomInitializer
#define DEFAULT_2DBINSTR_MUTATOR     GA2DBinStrFlipMutator
#define DEFAULT_2DBINSTR_CROSSOVER   GA2DBinStr1PtClipCrossover
#define DEFAULT_2DBINSTR_CROSSSITE   GA2DBinStr1PtCrossSite
#define DEFAULT_3DBINSTR_INITIALIZER GA3DBinStrRandomInitializer
#define DEFAULT_3DBINSTR_MUTATOR     GA3DBinStrFlipMutator
#define DEFAULT_3DBINSTR_CROSSOVER   GA3DBinStr1PtClipCrossover
#define DEFAULT_3DBINSTR_CROSSSITE   GA3DBinStr1PtCrossSite

#define DEFAULT_STRING_INITIALIZER   GAStringRandomInitializer
#define DEFAULT_STRING_MUTATOR       GAStringFlipMutator
#define DEFAULT_STRING_CROSSOVER     GAString1PtCrossover
#define DEFAULT_STRING_CROSSSITE     GAString1PtCrossSite


#endif
