// $Header: /usr/people/mbwall/src/ga/ga/RCS/creators.C,v 1.3 1995/06/30 21:27:55 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  creators.C
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Outline code for setting default initializers, mutators, and crossovers.  
This code is not part of the library but is provided as outlines of how to do
the overrides in your own code.
  In each case, we create an instance of a crossover site (of the type that we
want to use as the default) then we assign the default values for the creator
of each genome type.
  You should copy the appropriate chunk of code into your own source then 
modify the *_TYPE macro to fit the type that you are going to instantiate.  You
can also modify the initializer, mutator, crossover as needed.
  Do not modify the functionality of what's going on in the curly braces in 
each initializer unless you know what you're doing.  In most cases, the 
assignments that take place there are critical to the operation of the object.
  There is template code here for 1D, 2D, 3D arrays, trees, and lists.

 MODIFICATIONS:
---------------------------------------------------------------------------- */

static GA1DArray1PtCrossSite<ARRAY_TYPE> crossSite;
//static GA1DArray2PtCrossSite<ARRAY_TYPE> crossSite;
//static GA1DArrayOrderCrossSite<ARRAY_TYPE> crossSite;

GA1DArrayGenome<ARRAY_TYPE>::
GA1DArrayGenome(const unsigned int length, 
		    GAAlleleSet<ARRAY_TYPE> & s,
		    GAObjectiveFunction f,
		    void * u,
		    GA * g) :
GAGenome(GA1DArrayRandomInitializer,
	     GA1DArrayFlipMutator,
	     GA1DArray1PtCrossover,
	     &crossSite),
GAArray<ARRAY_TYPE>(length)
{
  of=f; ud=u; ga=g; as=&s;
  nx=minX=maxX=length;
}









static GA2DArray1PtCrossSite<ARRAY_TYPE> crossSite;

GA2DArrayGenome<ARRAY_TYPE>::
GA2DArrayGenome(const unsigned int width, const unsigned int height, 
		    GAAlleleSet<ARRAY_TYPE> & s,
		    GAObjectiveFunction f,
		    void * u,
		    GA * g) :
GAGenome(GA2DArrayRandomInitializer,
	     GA2DArrayFlipMutator,
	     GA2DArray1PtCrossover,
	     &crossSite),
GAArray<ARRAY_TYPE>(width*height)
{
  of=f; ud=u; ga=g; as=&s;
  nx=minX=maxX=width; ny=minY=maxY=height;
}








static GA3DArray1PtCrossSite<ARRAY_TYPE> crossSite;

GA3DArrayGenome<ARRAY_TYPE>::
GA3DArrayGenome(const unsigned int width,
		    const unsigned int height, 
		    const unsigned int depth,
		    GAAlleleSet<ARRAY_TYPE> & s,
		    GAObjectiveFunction f,
		    void * u,
		    GA * g) :
GAGenome(GA3DArrayRandomInitializer,
	     GA3DArrayFlipMutator,
	     GA3DArray1PtCrossover,
	     &crossSite),
GAArray<ARRAY_TYPE>(width*height*depth)
{
  of=f; ud=u; ga=g; as=&s;
  nx=minX=maxX=width; ny=minY=maxY=height; nz=minZ=maxZ=depth;
}













static GAList1PtCrossSite<LIST_TYPE> crossSite;

GAListGenome<LIST_TYPE>::
GAListGenome(GAObjectiveFunction f, void * u, GA * g) : 
GAGenome(GANoInitializer,
	     GAListSwapMutator,
	     GAList1PtCrossover,
	     &crossSite),
GAList<LIST_TYPE>()
{of=f; ud=u; ga=g;}










static GATree1PtCrossSite<TREE_TYPE> crossSite;

GATreeGenome<TREE_TYPE>::
GATreeGenome(GAObjectiveFunction f, void * u, GA * g) : 
GAGenome(GANoInitializer,
	     GATreeSwapNodeMutator,
	     GATree1PtCrossover,
	     &crossSite),
GATree<TREE_TYPE>()
{of=f; ud=u; ga=g;}
