// $Header: /usr/people/mbwall/src/ga/ga/RCS/id.h,v 1.2 1995/05/30 14:22:55 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  id.h
  mbwall 7may95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This defines the identifiers for polymorphic classes.  The IDs are used 
primarily for checking to be see if the two objects are the same type before
doing a copy, for example.  The name is often used when printing out error
messages so you'll know where things are happening.

 TO DO:
*** it would be nice to make the ids idiot proof - right now it is possible to
    assign an id that is already in use.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_id_h_
#define _ga_id_h_

class GAID {
public:
  int sameClass(const GAID &b) const {return(classID() == b.classID());} 
  virtual char * className() const {return 0;} 
  virtual int classID() const {return 0;}
};


// Here are the ID values for the GA library:
//
//    class                                  range
//    --------------------------------------------
//    GA                                      1-9
//    Population                             10-14
//    Scaling                                15-34
//    Genomes                                50-74
//    Crossover Sites                        75-99
//    Genome-related                        100-149
//    reserved                              150-199
//    anyone can use these                  200-255
//
// Use these macros to define the identity for your class.  If you do not use
// these macros then your object will be identity-less.
//   Here's how to use the macro.  The : at the end is critical (its a hack so
// that the c++ mode in my editor will still work correctly).
//
//     class GASimpleGA : public GA {
//        GADefineClassIdentity("GASimpleGA", 1):
//     public:
//       ...
//     };
//
//   Notice that your template classes will all have the same name using this
// method.  For example, GA1DArrayGenome<char> and GA1DArrayGenome<int>
// will both have the name GA1DArrayGenome.
//   Notice also that you do not need to define the identity of a virtual
// class - it will never be instantiated anyway.

#define GADefineIdentity(name,id) \
public: \
     char * className() const {return name;} \
     int classID() const {return id;} \
public

#endif

