// $Header: /usr/people/mbwall/src/ga/ga/RCS/list.ch.C,v 1.6 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  list.ch.C
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Source file for the list genome.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <ga/list.ch.h>


template <class T> 
GAListGenome<T>::GAListGenome(GAObjectiveFunction f, void * u, GA * g):
GAList<T>(){
  of=f; ud=u; ga=g;
}


template <class T> 
GAListGenome<T>::GAListGenome(const GAListGenome<T> & orig) : 
GAList<T>(){
  copy(orig);
}


template <class T> void
GAListGenome<T>::copy(const GAGenome & orig)
{
  if(&orig == this) return;
  if(!sameClass(orig)){
    GAErr(className(), "copy", gaErrObjectTypeMismatch);
    return;
  }
  GAListGenome<T> & c = (GAListGenome<T> &)orig;
  GAGenome::copy(orig);
  GAList<T>::copy(c);
}


// Traverse the list (breadth-first) and dump the contents as best we can to
// the stream.  We don't try to write the contents of the nodes - we simply 
// write a . for each node in the list.
template <class T> void
GAListGenome<T>::_write(ostream & os) const 
{
  os << "node       next       prev       contents\n";
  if(!hd) return;
  os.width(10); os << hd << " ";
  os.width(10); os << hd->next << " ";
  os.width(10); os << hd->prev << " ";
  os.width(10); os << &(((GANode<T> *)hd)->contents) << "\n";

  for(GANodeBASE * tmp=hd->next; tmp && tmp != hd; tmp=tmp->next){
    os.width(10); os << tmp << " ";
    os.width(10); os << tmp->next << " ";
    os.width(10); os << tmp->prev << " ";
    os.width(10); os << &(((GANode<T> *)tmp)->contents) << "\n";
  }
}


// Both the == and != operators assume that both operator== and operator!= are
// defined for the object that is store in the node of your list.  If it is
// not, you'll get an error message.  If you're storing pointers in the nodes,
// then you have nothing to worry about.
//   Neither of these operators affects the internal iterator of either
// list genome in any way.
template <class T> int 
GAListGenome<T>::_equal(const GAGenome & c) const
{
  if(this == &c) return gaTrue;
  GAListGenome<T> & b = (GAListGenome<T> &)c;
  if(size() != b.size()) return gaFalse;

  GAListIter<T> iterA(*this), iterB(b);
  T *tmpA = iterA.head(), *tmpB = iterB.head();
  T *head = tmpA;
  do{
    if(tmpA && tmpB && *tmpA != *tmpB) return gaFalse;
    tmpB = iterB.next();
    tmpA = iterA.next();
  }while(tmpA && tmpA != head);
  return gaTrue;
}


template <class T> int 
GAListGenome<T>::_notequal(const GAGenome & c) const
{
  if(this == &c) return gaFalse;
  GAListGenome<T> & b = (GAListGenome<T> &)c;
  if(size() != b.size()) return gaTrue;

  GAListIter<T> iterA(*this), iterB(b);
  T *tmpA = iterA.head(), *tmpB = iterB.head();
  T *head = tmpA;
  do{
    if(tmpA && tmpB && *tmpA != *tmpB) return gaTrue;
    tmpB = iterB.next();
    tmpA = iterA.next();
  }while(tmpA && tmpA != head);
  return gaFalse;
}
