// $Header: /usr/people/mbwall/src/ga/ga/RCS/list.xs.C,v 1.5 1995/06/30 21:27:55 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  list.xs.C
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Source file for the list genome crossover sites.
  Why not put these in the header file?  I can put more comments here, and 
they're virtual functions anyway, so inlining is not possible.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <ga/random.h>
#include <ga/list.ch.h>
#include <ga/list.xs.h>


#if USE_1PT_CROSSOVER == 1
/* ----------------------------------------------------------------------------
List Single Point Crossover Site
---------------------------------------------------------------------------- */
// Pick a crossover site at random.  The default list crossover just swaps two
// sublists in the parents.  All we need to represent this is a pair of indices
// to the nodes that are going to get swapped (we don't actually swap the nodes
// in the parents - only in the copy to the children during crossover).
//   The default crossover doesn't really make any sense without knowing 
// something about the parents (we can't pick nodes in the parents if we don't
// know the parents).  So all we do here is flag the cross sites as unset.
template <class T> void 
GAList1PtCrossSite<T>::pick(GAGenome &){
  a = b = gaNoSite;
}


// Use the genomes to pick a crossover site.  This is the method that the
// standard GAs use to do the crossover site selection (because the standard
// GAs do sexual crossover - they use both a mother and a father [ok, so it
// could be asexual if a single genome gets selected twice and has to mate
// with itself]).
//   All we do here is randomly pick a node in each parent.  The root node is
// node 0, then we count up depth-first from there.
template <class T> void 
GAList1PtCrossSite<T>::
pick(GAGenome &, const GAGenome &c1, const GAGenome &c2){
  GAListGenome<T> & mom = (GAListGenome<T> &)c1;
  GAListGenome<T> & dad = (GAListGenome<T> &)c2;
  a = GARandomInt(0, mom.size());
  b = GARandomInt(0, dad.size());
}


// This method mirrors the crossover site in the specified genome.  The
// standard GAs use this method when they generate two offspring from a 
// crossover.
template <class T> void 
GAList1PtCrossSite<T>::mirror(GAGenome &, const GAGenome & c1)
{
  a = ((GAList1PtCrossSite<T> &)(c1.crossoverSite())).b;
  b = ((GAList1PtCrossSite<T> &)(c1.crossoverSite())).a;
}
#endif




#if USE_PARTIAL_MATCH_CROSSOVER == 1 || USE_ORDER_CROSSOVER == 1 || USE_CYCLE_CROSSOVER == 1
/* ----------------------------------------------------------------------------
List Ordered Crossover Site
-------------------------------------------------------------------------------
  This crossover site will work with any GAListGenome.  It is designed to
be used with the partial match, cycle, and order crossover methods (any order-
based list should work with this crossover object).
---------------------------------------------------------------------------- */
// Pick a crossover site at random.  The partial match crossover needs two
// points to do its thing.  The parents must be the same size as the child, so
// we can use the child's size information to pick where the crossover will
// occur.  The two points are picked evenly across the length of the 
// genome.  We don't check here to make sure the parents are the same size
// as the child - do that in the crossover method.
//   See Goldberg for a complete description of the parial match and order
// crossover methods.
template <class T> void 
GAListOrderedCrossSite<T>::pick(GAGenome & c){
  GAListGenome<T> & child = (GAListGenome<T> &)c;
  a = GARandomInt(0, child.size());
  b = GARandomInt(0, child.size());
  if(b<a){
    int tmp=a;
    a=b;
    b=tmp;
  }
}


template <class T> void 
GAListOrderedCrossSite<T>::
pick(GAGenome & c, const GAGenome &, const GAGenome &){
  pick(c);
}


// When you mirror a matched crossover, the crossover site is the same.  Just
// swap parents to get the mirrored children.
template <class T> void 
GAListOrderedCrossSite<T>::mirror(GAGenome &, const GAGenome & c1)
{
  a = ((GAListOrderedCrossSite<T> &)(c1.crossoverSite())).a;
  b = ((GAListOrderedCrossSite<T> &)(c1.crossoverSite())).b;
}
#endif
