// $Header: /usr/people/mbwall/src/ga/ga/RCS/list.xs.h,v 1.6 1995/07/11 20:35:22 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  list.xs.h
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  This header defines the interface for the list genome crossover sites.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_list_xs_h_
#define _ga_list_xs_h_

#include <ga/config.h>
#include <ga/xsite.h>


#if USE_1PT_CROSSOVER == 1
/* ----------------------------------------------------------------------------
List Single Point Crossover
---------------------------------------------------------------------------- */
template <class T>
class GAList1PtCrossSite : public GACrossSite {
  GADefineIdentity("GAList1PtCrossSite", 95):

public:
  GAList1PtCrossSite(){a=gaNoSite; b=gaNoSite;}
  GAList1PtCrossSite(int x, int y){a=x; b=y;}
  GAList1PtCrossSite(const GAList1PtCrossSite<T> & orig){copy(orig);}
  GAList1PtCrossSite<T> & operator=(const GACrossSite & orig)
    {copy(orig); return *this;}
  GAList1PtCrossSite<T> & operator=(const GAList1PtCrossSite<T> & orig)
    {copy(orig); return *this;}
  virtual ~GAList1PtCrossSite(){}
  virtual void copy(const GACrossSite & xs){
    if(&xs == this || !sameClass(xs)) return;
    GAList1PtCrossSite<T> & orig = (GAList1PtCrossSite<T> &)xs;
    a=orig.a; b=orig.b;
  }
  virtual GACrossSite * clone() const
    {return new GAList1PtCrossSite<T>(*this);}

  virtual void pick(GAGenome &);
  virtual void pick(GAGenome &, const GACrossSite & s){copy(s);}
  virtual void pick(GAGenome &,const GAGenome&, const GAGenome&);
  virtual void mirror(GAGenome &, const GAGenome &);
  int a, b;
};
#endif


#if USE_PARTIAL_MATCH_CROSSOVER == 1 || USE_ORDER_CROSSOVER == 1 || USE_CYCLE_CROSSOVER == 1
/* ----------------------------------------------------------------------------
List Match Crossover - for partial match and order crossover methods
---------------------------------------------------------------------------- */
template <class T>
class GAListOrderedCrossSite : public GACrossSite {
  GADefineIdentity("GAListOrderedCrossSite", 96):

public:
  GAListOrderedCrossSite(){a=gaNoSite; b=gaNoSite;}
  GAListOrderedCrossSite(int x, int y){a=x; b=y;}
  GAListOrderedCrossSite(const GAListOrderedCrossSite & orig){copy(orig);}
  GAListOrderedCrossSite<T> & operator=(const GACrossSite & orig)
    {copy(orig); return *this;}
  GAListOrderedCrossSite<T> & operator=(const GAListOrderedCrossSite<T> & orig)
    {copy(orig); return *this;}
  virtual ~GAListOrderedCrossSite(){}
  virtual void copy(const GACrossSite & xs){
    if(&xs == this || !sameClass(xs)) return;
    GAListOrderedCrossSite<T> & orig = (GAListOrderedCrossSite<T> &)xs;
    a=orig.a; b=orig.b;
  }
  virtual GACrossSite * clone() const 
    {return new GAListOrderedCrossSite<T>(*this);}

  virtual void pick(GAGenome &);
  virtual void pick(GAGenome &, const GACrossSite & s){copy(s);}
  virtual void pick(GAGenome &c,const GAGenome&, const GAGenome&);
  virtual void mirror(GAGenome &, const GAGenome & c1);
  int a, b;
};
#endif

#ifdef BORLAND_INST
#include <ga/list.xs.C>
#endif

#endif
