// $Header: /usr/people/mbwall/src/ga/ga/RCS/parameters.h,v 1.1 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  parameters.h
  mbwall 14jul95
  Copyright 1995 Massachusetts Institute of Technology - all rights reserved

 DESCRIPTION:
  Header for the parameters object used by the GA objects.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_parameters_h_
#define _ga_parameters_h_

#include <iostream.h>

// These constants are the default values for the GA parameters.
const int   gaDefPopSize          = 30;
const int   gaDefNumGen           = 250;
const float gaDefPConv            = 0.05;
const int   gaDefNConv            = 20;
const float gaDefPMut             = 0.01;
const float gaDefPCross           = 0.90;

const int   gaDefNumBestChrom     = 1;
const int   gaDefScoreFrequency1  = 1;
const int   gaDefScoreFrequency2  = 100;


/* ----------------------------------------------------------------------------
Parameters class (public)
  The parameters structure contains the values for running the GAs.  This
is the basic information.  Specialized GAs may have to derive a new set of
parameters from this class so they can add additional parameters as needed.
---------------------------------------------------------------------------- */
struct GAParameters {
  GAParameters(const unsigned int ps=gaDefPopSize,
	       const float pc=gaDefPCross,
	       const float pm=gaDefPMut,
	       const unsigned int ng=gaDefNumGen, 
	       const float pcv=gaDefPConv,
	       const unsigned int ncv=gaDefNConv,
	       const unsigned int nbc=gaDefNumBestChrom, 
	       const unsigned int sf=gaDefScoreFrequency1){
    popSize=ps; pCross=pc; pMut=pm; nGen=ng; pConv=pcv; nConv=ncv;
    nBestChrom=nbc; scoreFreq=sf;
  }
  GAParameters(const GAParameters & p){
    popSize = p.popSize;
    pCross = p.pCross;
    pMut = p.pMut;
    nGen = p.nGen;
    pConv = p.pConv;
    nConv = p.nConv;
    nBestChrom = p.nBestChrom;
    scoreFreq = p.scoreFreq;
  }
  unsigned int popSize;		// population size
  float pCross;			// crossover probability
  float pMut;			// mutation probability
  unsigned int nGen;		// number of generations
  float pConv;			// convergence percentage
  unsigned int nConv;		// number of generations for convergence test
  unsigned int nBestChrom;	// how many of the best too keep
  unsigned int scoreFreq;	// how often to record a score

  friend ostream & operator<<(ostream & os, const GAParameters & arg){
    os << arg.popSize << "\t# population size\n";
    os << arg.pCross << "\t# crossover probability\n";
    os << arg.pMut << "\t# mutation probability\n";
    os << arg.nGen << "\t# number of generations\n";
    os << arg.pConv << "\t# convergence percentage\n";
    os << arg.nConv << "\t# number generations back for convergence test\n";
    os << arg.nBestChrom << "\t# number of best solutions to keep\n";
    os << arg.scoreFreq << "\t# how often to record the best-of-generation\n";
    return(os);
  }
};

#endif
