// $Header: /usr/people/mbwall/src/ga/ga/RCS/selector.h,v 1.15 1995/05/30 14:22:55 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  selector.h
  mbwall 10aug94
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  The selectors are functions that use information in fitness objects to pick
genomes from a population.  There are a number of selection criteria, and
not every selection method will work with every fitness object.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_selector_h_
#define _ga_selector_h_

#include <ga/population.h>
#include <ga/genome.h>


/* ----------------------------------------------------------------------------
This is the function prototype for all selection functions.  Each selection 
function takes a population.  The population object contains any scaling info
used to do the selection.
---------------------------------------------------------------------------- */
typedef GAGenome & (*GASelectionFunction)(const GAPopulation & p);



/* ----------------------------------------------------------------------------
Selection methods

  These are the pre-defined selection methods.  A brief outline of what each 
one does is given here, but for more details (especially the theoretical 
foundations for each method), see Goldberg.
  The selection methods can, to some extent, be mixed and matched with the
various scaling methods.
  If you want to do speciating selection (e.g. DeJong-style crowding with
Hamming distance) then you will have to write your own selection method.  The
example from Goldberg's book is included here as an example of how to write a
speciating selector.

RouletteWheel
Tournament
Stochastic Remainder Sampling (SRS)
Stochastic Universal Sampling (SUS)
---------------------------------------------------------------------------- */
GAGenome & GARankSelector(const GAPopulation &);
GAGenome & GARouletteWheelSelector(const GAPopulation &);
GAGenome & GATournamentSelector(const GAPopulation &);
GAGenome & GADSSelector(const GAPopulation &);
GAGenome & GASRSSelector(const GAPopulation &);
GAGenome & GASUSSelector(const GAPopulation &);


#endif
