// $Header: /usr/people/mbwall/src/ga/ga/RCS/sort.C,v 1.1 1995/06/30 21:27:55 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  sort.C
  mbwall 29jun95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Sorting routines for the GA library.

  These sorting routines were adapted from Sedgewick's "Algorithms".  Pick the
one(s) that suit you best.
  There are two versions of each - one has an array of genomes and uses the
evaluation methods in the genomes as the ranking criterion.  The other has
an array of genomes as well as an array of floats, and the floats are used
as the ranking criterion.
  For the merge sort, 'a' should be the array of pointers to genomes, 'b'
should be an array of pointers (not assigned to actual genomes).  You can
delete 'b' after the sort is finished.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <stdlib.h>
#include <ga/sort.h>
#include <ga/genome.h>

#ifndef NDEBUG
#define NDEBUG
#endif

void
GAQuickSort(GAGenome **c, const int l, const int r)
{
#ifndef NDEBUG
  static int count=0;
  count++;
  cerr << "GAQuickSort(Genome**,int,int): count is " << count << "\n";
#endif

  int i,j; 
  float v;
  GAGenome *t;

  if(r > l){
    v = c[r]->score(); i = l-1; j = r;
    for(;;){
      while(c[++i]->score() < v && i <= r);
      while(c[--j]->score() > v && j > 0);
      if(i >= j) break;
      t = c[i]; c[i] = c[j]; c[j] = t;
    }
    t = c[i]; c[i] = c[r]; c[r] = t;
    GAQuickSort(c,l,i-1);
    GAQuickSort(c,i+1,r);
  }
}

void
GAQuickSort(GAGenome **c, float *s, const int l, const int r)
{
#ifndef NDEBUG
  static int count=0;
  count++;
  cerr << "GAQuickSort(Genome**,float*,int,int): count is " << count << "\n";
#endif

  int i,j; 
  float v;
  GAGenome *tc;
  float ts;

  if(r > l){
    v = s[r]; i = l-1; j = r;
    for(;;){
      while(s[++i] < v); // might exceed max array limit here
      while(s[--j] > v && j > 0);
      if(i >= j) break;
      tc = c[i]; c[i] = c[j]; c[j] = tc;
      ts = s[i]; s[i] = s[j]; s[j] = ts;
    }
    tc = c[i]; c[i] = c[r]; c[r] = tc;
    ts = s[i]; s[i] = s[r]; s[r] = ts;
    GAQuickSort(c,s,l,i-1);
    GAQuickSort(c,s,i+1,r);
  }
}

void
GAQuickSort(unsigned int *c, float *s, const int l, const int r)
{
#ifndef NDEBUG
  static int count=0;
  count++;
  cerr << "GAQuickSort(Genome**,float*,int,int): count is " << count << "\n";
#endif

  int i,j; 
  float v;
  unsigned int tc;
  float ts;

  if(r > l){
    v = s[r]; i = l-1; j = r;
    for(;;){
      while(s[++i] < v); // might exceed max array limit here
      while(s[--j] > v && j > 0);
      if(i >= j) break;
      tc = c[i]; c[i] = c[j]; c[j] = tc;
      ts = s[i]; s[i] = s[j]; s[j] = ts;
    }
    tc = c[i]; c[i] = c[r]; c[r] = tc;
    ts = s[i]; s[i] = s[r]; s[r] = ts;
    GAQuickSort(c,s,l,i-1);
    GAQuickSort(c,s,i+1,r);
  }
}



void 
GAMergeSort(GAGenome ** a, GAGenome ** b, const int l, const int r)
{
  int i,j,k,m;

  if(r > l){
    m = (r+l)/2;
    GAMergeSort(a,b,l,m);
    GAMergeSort(a,b,m+1,r);
    for(i = m+1; i > l; i--) b[i-1] = a[i-1];
    for(j = m; j < r; j++) b[r+m-j] = a[j+1];
    for(k = l; k <= r; k++)
      a[k] = (b[i]->score() < b[j]->score()) ? b[i++] : b[j--];
  }
}

