// $Header: /usr/people/mbwall/src/ga/ga/RCS/tree.ch.C,v 1.6 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  tree.ch.C
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Source file for the tree genome.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <ga/tree.ch.h>


template <class T>
GATreeGenome<T>::GATreeGenome(GAObjectiveFunction f, void * u, GA * g):
GATree<T>(){
  of=f; ud=u; ga=g;
}


template <class T>
GATreeGenome<T>::GATreeGenome(const GATreeGenome<T> & orig) :
GATree<T>(){
  copy(orig);
}


template <class T> void
GATreeGenome<T>::copy(const GAGenome & orig)
{
  if(&orig == this) return;
  if(!sameClass(orig)){
    GAErr(className(), "copy", gaErrObjectTypeMismatch);
    return;
  }
  GATreeGenome<T> & c = (GATreeGenome<T> &)orig;
  GAGenome::copy(orig);
  GATree<T>::copy(c);
}


// Traverse the tree (breadth-first) and dump the contents as best we can to
// the stream.  We don't try to write the contents of the nodes - we simply 
// write a . for each node in the tree.
//   We allocate space for x,y coord pair for each node in the tree.  Then we
// do a depth-first traversal of the tree and assign coords to the nodes in the
// order we get them in the traversal.  Each coord pair is measured relative to
// the parent of the node.
template <class T> void
_tt(ostream & os, GANode<T> * n)
{
  if(!n) return;
  GANodeBASE * node = (GANodeBASE *)n;

  os.width(10); os << node << " ";
  os.width(10); os << node->parent << " ";
  os.width(10); os << node->child << " ";
  os.width(10); os << node->next << " ";
  os.width(10); os << node->prev << " ";
  os.width(10); os << &(n->contents) << "\n";
  _tt(os, (GANode<T> *)node->child);

  for(GANodeBASE * tmp=node->next; tmp && tmp != node; tmp=tmp->next){
    os.width(10); os << tmp << " ";
    os.width(10); os << tmp->parent << " ";
    os.width(10); os << tmp->child << " ";
    os.width(10); os << tmp->next << " ";
    os.width(10); os << tmp->prev << " ";
    os.width(10); os << &(((GANode<T> *)tmp)->contents) << "\n";
    _tt(os, (GANode<T> *)tmp->child);
  }
}

template <class T> void
GATreeGenome<T>::_write(ostream & os) const 
{
  os << "node       parent     child      next       prev       contents\n";
  _tt(os, (GANode<T> *)rt);
}


// Comparison operators for a a tree check for identical tree structure.  We do
// NOT check for identical node contents (we cannot know if the node contents
// have valid comparison operators defined for them).  These simply check for
// similary tree structure - we traverse one tree and expect the second to be
// just like it (depth-first traversal).  If there is any difference along the
// way, then we return a not equal.
//   Neither of these operators affects the internal iterator of either
// tree genome in any way.
//  Users should specialize these methods so that they check node contents
// (if needed).

//   Recursively traverse two trees at the same time.  Return 1 if they are 
// different, return a 0 if they are identical.  This checks only the tree
// structure, not the node contents.
static int
_GATreeCompare(GANodeBASE * anode, GANodeBASE * bnode)
{
  if(!anode && !bnode) return 0;
  if(!anode || !bnode) return 1;
  int count = _GATreeCompare(anode->child, bnode->child);
  for(GANodeBASE * atmp=anode->next, * btmp=bnode->next;
      atmp && atmp != anode;
      atmp=atmp->next, btmp=btmp->next){
    count += _GATreeCompare(atmp->child, btmp->child);
  }
  return count;
}


template <class T> int  
GATreeGenome<T>::_equal(const GAGenome & c) const
{
  if(this == &c) return 1;
  GATreeGenome<T> & b = (GATreeGenome<T> &)c;
  return _GATreeCompare(rt, b.rt) ? gaFalse : gaTrue;
}


template <class T> int 
GATreeGenome<T>::_notequal(const GAGenome & c) const
{
  if(this == &c) return 0;
  GATreeGenome<T> & b = (GATreeGenome<T> &)c;
  return _GATreeCompare(rt, b.rt) ? gaTrue : gaFalse;
}
