// $Header: /usr/people/mbwall/src/ga/ga/RCS/tree.xs.C,v 1.5 1995/06/30 21:27:55 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  tree.xs.C
  mbwall 25feb95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Source file for the tree genome crossover sites.
  Why not put these in the header file?  I can put more comments here, and 
they're virtual functions anyway, so inlining is not possible.

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#include <ga/random.h>
#include <ga/tree.ch.h>
#include <ga/tree.xs.h>


/* ----------------------------------------------------------------------------
Tree Single Point Crossover Site
---------------------------------------------------------------------------- */
// Pick a crossover site at random.  The default tree crossover just swaps two
// subtrees in the parents.  All we need to represent this is a pair of indices
// to the nodes that are going to get swapped (we don't actually swap the nodes
// in the parents - only in the copy to the children during crossover).
//   The default crossover doesn't really make any sense without knowing 
// something about the parents (we can't pick nodes in the parents if we don't
// know the parents).  So all we do here is flag the cross sites as unset.
template <class T> void
GATree1PtCrossSite<T>::pick(GAGenome &){
  a = b = gaNoSite;
}


// Use the genomes to pick a crossover site.  This is the method that the
// standard GAs use to do the crossover site selection (because the standard
// GAs do sexual crossover - they use both a mother and a father [ok, so it
// could be asexual if a single genome gets selected twice and has to mate
// with itself]).
//   All we do here is randomly pick a node in each parent.  The root node is
// node 0, then we count up depth-first from there.
template <class T> void
GATree1PtCrossSite<T>::
pick(GAGenome &, const GAGenome &c1, const GAGenome &c2){
  GATreeGenome<T> & mom = (GATreeGenome<T> &)c1;
  GATreeGenome<T> & dad = (GATreeGenome<T> &)c2;
  a = GARandomInt(0, mom.size()-1);
  b = GARandomInt(0, dad.size()-1);
}


// This method mirrors the crossover site in the specified genome.  The
// standard GAs use this method when they generate two offspring from a 
// crossover.
template <class T> void
GATree1PtCrossSite<T>::mirror(GAGenome &, const GAGenome & c1)
{
  a = ((GATree1PtCrossSite<T> &)(c1.crossoverSite())).b;
  b = ((GATree1PtCrossSite<T> &)(c1.crossoverSite())).a;
}
