// $Header: /usr/people/mbwall/src/ga/ga/RCS/xsite.h,v 1.11 1995/09/13 22:30:10 mbwall Exp mbwall $
/* ----------------------------------------------------------------------------
  xsite.h
  mbwall 4jan95
  Copyright 1995 Massachusetts Institute of Technology

 DESCRIPTION:
  Class definitions for the crossover site objects.  These are closely coupled
functionally to the genome classes, but the class definitions don't depend
directly on the genomes (only the methods do).

 MODIFICATIONS:
---------------------------------------------------------------------------- */
#ifndef _ga_xsite_h_
#define _ga_xsite_h_

#include <ga/id.h>
#include <ga/error.h>

class GAGenome;

const int gaNoSite = -1;

/* ----------------------------------------------------------------------------
 Crossover Sites
-------------------------------------------------------------------------------
  This is a generic crossover site.  Each genome-crossover pair may have
its own type of information needed for crossover.  This is where to keep that
information.  If you do a custom crossover method, you may need to derive a
crossover site from this base (public) class.  All of the default crossover
sites (eg one point crossover for binary strings) are derived from this class.
  We don't really need to define the creator, copy constructor, operator=, and
destructor for this structure, but you should for any non-trivial derived
classes of with this as the ancestor.
  When a genome gets copied, the cross site object gets copied as well, so
be sure that you define an appropriate copy constructor/operator= if your cross
site object is non-trivial.
---------------------------------------------------------------------------- */
class GACrossSite : public GAID {
  GADefineIdentity("GACrossSite", 75):

public:
  GACrossSite(){}
  GACrossSite(const GACrossSite & orig){copy(orig);}
  GACrossSite & operator=(const GACrossSite & orig){copy(orig); return *this;}
  virtual ~GACrossSite(){}
  virtual void copy(const GACrossSite &){}
  virtual GACrossSite * clone() const {return new GACrossSite;}
  virtual void pick(GAGenome &) {}
  virtual void pick(GAGenome &, const GACrossSite &) {}
  virtual void pick(GAGenome&, const GAGenome&, const GAGenome&) {}
  virtual void mirror(GAGenome &, const GAGenome &) {}
};

#endif
